@@ -289,13 +289,15 @@ error:
 typedef enum {
 	OPT_MAXLYRS,
 	OPT_MAXPKTS,
-	OPT_DEBUG
+	OPT_DEBUG,
+	OPT_RAW_PALETTE
 } optid_t;
 
 jas_taginfo_t decopts[] = {
 	{OPT_MAXLYRS, "maxlyrs"},
 	{OPT_MAXPKTS, "maxpkts"},
 	{OPT_DEBUG, "debug"},
+	{OPT_RAW_PALETTE, "raw"},
 	{-1, 0}
 };
 
@@ -323,6 +325,9 @@ static int jpc_dec_parseopts(char *optstr, jpc_dec_importopts_t *opts)
 		case OPT_MAXPKTS:
 			opts->maxpkts = atoi(jas_tvparser_getval(tvp));
 			break;
+		case OPT_RAW_PALETTE:
+			/* this is just for passthrough from jp2_decode */
+			break;
 		default:
 			jas_eprintf("warning: ignoring invalid option %s\n",
 			  jas_tvparser_gettag(tvp));
