@@ -1,22 +1,16 @@
-/* Copyright (C) 1992, 1994, 1998 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-
-/*$Id: gdev3b1.c,v 1.2.6.1.2.1 2003/01/17 00:49:00 giles Exp $*/
+/* $Id: gdev3b1.c 8022 2007-06-05 22:23:38Z giles $*/
 /*
  * This is a driver for the AT&T 3b1/7300/UnixPC console display.
  *
@@ -157,7 +151,7 @@ att3b1_open(gx_device *dev)
     }
 
     if (att3b1dev->screen != NULL) {
-	gs_free((char *)att3b1dev->screen,
+	gs_free(dev->memory, (char *)att3b1dev->screen,
 		att3b1dev->screen_size, 1, "att3b1_open");
 	att3b1dev->screen = 0;
 	att3b1dev->screen_size = 0;
@@ -182,7 +176,7 @@ att3b1_open(gx_device *dev)
     att3b1dev->screen_size = att3b1dev->line_size * att3b1dev->height;
 
     att3b1dev->screen =
-	(uchar *)gs_malloc(att3b1dev->screen_size, 1, "att3b1_open");
+	(uchar *)gs_malloc(dev->memory, att3b1dev->screen_size, 1, "att3b1_open");
     if (att3b1dev->screen == NULL) {
 	att3b1_close(dev);
 	return_error(gs_error_VMerror);
@@ -208,7 +202,7 @@ att3b1_close(gx_device *dev)
     }
 
     if (att3b1dev->screen != NULL) {
-	gs_free((char *)att3b1dev->screen,
+	gs_free(dev->memory, (char *)att3b1dev->screen,
 		att3b1dev->screen_size, 1, "att3b1_close");
 	att3b1dev->screen = 0;
 	att3b1dev->screen_size = 0;
