@@ -1,20 +1,16 @@
-/* Copyright (C) 1994, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
-*/
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-/*$Id: gdevabuf.c,v 1.3.6.1 2002/02/22 19:45:55 ray Exp $ */
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+/* $Id: gdevabuf.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Alpha-buffering memory devices */
 #include "memory_.h"
 #include "gx.h"
@@ -58,11 +54,10 @@ gs_make_mem_alpha_device(gx_device_memory * adev, gs_memory_t * mem,
 
 /* Reimplement color mapping. */
 private gx_color_index
-mem_alpha_map_rgb_color(gx_device * dev, gx_color_value r, gx_color_value g,
-			gx_color_value b)
+mem_alpha_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
-    gx_color_index color = gx_forward_map_rgb_color(dev, r, g, b);
+    gx_color_index color = gx_forward_map_rgb_color(dev, cv);
 
     return (color == 0 || color == gx_no_color_index ? color :
 	    (gx_color_index) ((1 << mdev->log2_alpha_bits) - 1));
@@ -81,7 +76,11 @@ mem_alpha_map_rgb_alpha_color(gx_device * dev, gx_color_value r,
 		   gx_color_value g, gx_color_value b, gx_color_value alpha)
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
-    gx_color_index color = gx_forward_map_rgb_color(dev, r, g, b);
+    gx_color_index color;
+    gx_color_value cv[3];
+
+    cv[0] = r; cv[1] = g; cv[2] = b;
+    color = gx_forward_map_rgb_color(dev, cv);
 
     return (color == 0 || color == gx_no_color_index ? color :
 	    (gx_color_index) (alpha >> (gx_color_value_bits -
@@ -278,7 +277,7 @@ y_transfer_init(y_transfer * pyt, gx_device * dev, int ty, int th)
     pyt->transfer_height = 0;
 }
 /* while ( yt.height_left > 0 ) { y_transfer_next(&yt, mdev); ... } */
-private void
+private int
 y_transfer_next(y_transfer * pyt, gx_device * dev)
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
@@ -293,7 +292,10 @@ y_transfer_next(y_transfer * pyt, gx_device * dev)
 
     if (ty == my + mh) {	/* Add a new block at my1. */
 	if (mh == mdev->height) {
-	    abuf_flush_block(mdev, my);
+	    int code = abuf_flush_block(mdev, my);
+
+	    if (code < 0)
+		return code;
 	    mdev->mapped_y = my += bh;
 	    if ((mdev->mapped_start = ms += bh) == mh)
 		mdev->mapped_start = ms = 0;
@@ -323,6 +325,7 @@ y_transfer_next(y_transfer * pyt, gx_device * dev)
     pyt->height_left = th - tbh;
     pyt->transfer_y = tby;
     pyt->transfer_height = tbh;
+    return 0;
 }
 
 /* Copy a monobit image. */
@@ -343,7 +346,10 @@ mem_abuf_copy_mono(gx_device * dev,
     mdev->save_color = one;
     y_transfer_init(&yt, dev, y, h);
     while (yt.height_left > 0) {
-	y_transfer_next(&yt, dev);
+	int code = y_transfer_next(&yt, dev);
+
+	if (code < 0)
+	    return code;
 	(*dev_proc(&mem_mono_device, copy_mono)) (dev,
 					   base + (yt.y_next - y) * sraster,
 					  sourcex, sraster, gx_no_bitmap_id,
@@ -368,7 +374,10 @@ mem_abuf_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
     mdev->save_color = color;
     y_transfer_init(&yt, dev, y, h);
     while (yt.height_left > 0) {
-	y_transfer_next(&yt, dev);
+	int code = y_transfer_next(&yt, dev);
+
+	if (code < 0)
+	    return code;
 	(*dev_proc(&mem_mono_device, fill_rectangle)) (dev,
 				    x, yt.transfer_y, w, yt.transfer_height,
 						       (gx_color_index) 1);
