@@ -1,22 +1,17 @@
-/* Copyright (C) 1989-1994, 1998 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevepsn.c,v 1.3.2.1.2.1 2003/01/17 00:49:00 giles Exp $*/
+/* $Id: gdevepsn.c 8022 2007-06-05 22:23:38Z giles $*/
 /*
  * Epson (and similar) dot-matrix printer driver for Ghostscript.
  *
@@ -138,7 +133,7 @@ const gx_device_printer far_data gs_ibmpro_device =
 /* ------ Driver procedures ------ */
 
 /* Forward references */
-private void eps_output_run(P6(byte *, int, int, char, FILE *, int));
+private void eps_output_run(byte *, int, int, char, FILE *, int);
 
 /* Send the page to the printer. */
 #define DD 0x40				/* double density flag */
@@ -163,12 +158,12 @@ eps_print_page(gx_device_printer *pdev, FILE *prn_stream, int y_9pin_high,
 	int line_size = gdev_mem_bytes_per_scan_line((gx_device *)pdev);
 	/* Note that in_size is a multiple of 8. */
 	int in_size = line_size * (8 * in_y_mult);
-	byte *buf1 = (byte *)gs_malloc(in_size, 1, "eps_print_page(buf1)");
-	byte *buf2 = (byte *)gs_malloc(in_size, 1, "eps_print_page(buf2)");
+	byte *buf1 = (byte *)gs_malloc(pdev->memory, in_size, 1, "eps_print_page(buf1)");
+	byte *buf2 = (byte *)gs_malloc(pdev->memory, in_size, 1, "eps_print_page(buf2)");
 	byte *in = buf1;
 	byte *out = buf2;
 	int out_y_mult = (y_24pin ? 3 : 1);
-	int x_dpi = pdev->x_pixels_per_inch;
+	int x_dpi = (int)pdev->x_pixels_per_inch;
 	char start_graphics =
 		(y_24pin ? graphics_modes_24 : graphics_modes_9)[x_dpi / 60];
 	int first_pass = (start_graphics & DD ? 1 : 0);
@@ -182,9 +177,9 @@ eps_print_page(gx_device_printer *pdev, FILE *prn_stream, int y_9pin_high,
 	/* Check allocations */
 	if ( buf1 == 0 || buf2 == 0 )
 	{	if ( buf1 ) 
-		  gs_free((char *)buf1, in_size, 1, "eps_print_page(buf1)");
+		  gs_free(pdev->memory, (char *)buf1, in_size, 1, "eps_print_page(buf1)");
 		if ( buf2 ) 
-		  gs_free((char *)buf2, in_size, 1, "eps_print_page(buf2)");
+		  gs_free(pdev->memory, (char *)buf2, in_size, 1, "eps_print_page(buf2)");
 		return_error(gs_error_VMerror);
 	}
 
@@ -395,8 +390,8 @@ eps_print_page(gx_device_printer *pdev, FILE *prn_stream, int y_9pin_high,
 	fputs(end_string, prn_stream);
 	fflush(prn_stream);
 
-	gs_free((char *)buf2, in_size, 1, "eps_print_page(buf2)");
-	gs_free((char *)buf1, in_size, 1, "eps_print_page(buf1)");
+	gs_free(pdev->memory, (char *)buf2, in_size, 1, "eps_print_page(buf2)");
+	gs_free(pdev->memory, (char *)buf1, in_size, 1, "eps_print_page(buf1)");
 	return 0;
 }
 
@@ -411,7 +406,7 @@ eps_output_run(byte *data, int count, int y_mult,
 	fputc(033, prn_stream);
 	if ( !(start_graphics & ~3) )
 	{	
-		fputc("KLYZ"[start_graphics], prn_stream);
+		fputc("KLYZ"[(int)start_graphics], prn_stream);
 	}
 	else
 	{	
