@@ -1,22 +1,16 @@
-/* Copyright (C) 1993, 1994 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-
-/*$Id: gdevescp.c,v 1.3.2.1.2.1 2003/01/17 00:49:00 giles Exp $*/
+/* $Id: gdevescp.c 8022 2007-06-05 22:23:38Z giles $*/
 /*
  * Epson 'ESC/P 2' language printer driver.
  *
@@ -117,8 +111,8 @@ escp2_print_page(gx_device_printer *pdev, FILE *prn_stream)
 	int band_size = 24;	/* 1, 8, or 24 */
 	int in_size = line_size * band_size;
 
-	byte *buf1 = (byte *)gs_malloc(in_size, 1, "escp2_print_page(buf1)");
-	byte *buf2 = (byte *)gs_malloc(in_size, 1, "escp2_print_page(buf2)");
+	byte *buf1 = (byte *)gs_malloc(pdev->memory, in_size, 1, "escp2_print_page(buf1)");
+	byte *buf2 = (byte *)gs_malloc(pdev->memory, in_size, 1, "escp2_print_page(buf2)");
 	byte *in = buf1;
 	byte *out = buf2;
 
@@ -148,9 +142,9 @@ escp2_print_page(gx_device_printer *pdev, FILE *prn_stream)
 
 	if ( buf1 == 0 || buf2 == 0 )
 	{	if ( buf1 ) 
-		  gs_free((char *)buf1, in_size, 1, "escp2_print_page(buf1)");
+		  gs_free(pdev->memory, (char *)buf1, in_size, 1, "escp2_print_page(buf1)");
 		if ( buf2 ) 
-		  gs_free((char *)buf2, in_size, 1, "escp2_print_page(buf2)");
+		  gs_free(pdev->memory, (char *)buf2, in_size, 1, "escp2_print_page(buf2)");
 		return_error(gs_error_VMerror);
 	}
 
@@ -192,8 +186,9 @@ escp2_print_page(gx_device_printer *pdev, FILE *prn_stream)
         */
 
         if( auto_feed ) {
-           top = dev_t_margin(pdev) * pdev->y_pixels_per_inch;
-           bottom = pdev->height - dev_b_margin(pdev) * pdev->y_pixels_per_inch;
+           top = (int)(dev_t_margin(pdev) * pdev->y_pixels_per_inch);
+           bottom = (int)(pdev->height - 
+			dev_b_margin(pdev) * pdev->y_pixels_per_inch);
         } else {
            top = 0;
            bottom = pdev->height;
@@ -409,7 +404,7 @@ escp2_print_page(gx_device_printer *pdev, FILE *prn_stream)
 	fputs("\f\033@", prn_stream);
 	fflush(prn_stream);
 
-	gs_free((char *)buf2, in_size, 1, "escp2_print_page(buf2)");
-	gs_free((char *)buf1, in_size, 1, "escp2_print_page(buf1)");
+	gs_free(pdev->memory, (char *)buf2, in_size, 1, "escp2_print_page(buf2)");
+	gs_free(pdev->memory, (char *)buf1, in_size, 1, "escp2_print_page(buf1)");
 	return 0;
 }
