@@ -1,22 +1,16 @@
-/* Copyright (C) 1990, 1991, 1993 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-
-/*$Id: gdevherc.c,v 1.2.6.1.2.1 2003/01/17 00:49:00 giles Exp $*/
+/* $Id: gdevherc.c 8022 2007-06-05 22:23:38Z giles $*/
 /* IBM PC-compatible Hercules Graphics display driver */
 /* using direct access to frame buffer */
 
@@ -90,8 +84,8 @@ gx_device far_data gs_herc_device = {
 
 
 /* Forward declarations */
-private int herc_get_mode(P0());
-private void herc_set_mode(P1(int));
+private int herc_get_mode(void);
+private void herc_set_mode(int);
 
 /* Save the HERC mode */
 private int herc_save_mode = -1;
