@@ -1,23 +1,21 @@
-/* Copyright (C) 1994-2003 artofcode LLC. All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevmac.c,v 1.1.4.3.2.2 2003/04/12 16:37:26 giles Exp $ */
-
+/* $Id: gdevmac.c 8022 2007-06-05 22:23:38Z giles $ */
+/* MacOS bitmap output device. This code is superceeded by
+   the newer gsapi_* interface and the DISPLAY device. Please
+   use that instead. See doc/API.htm for more information */
+   
 #include "gdevmac.h"
 #include "gsparam.h"
 #include "gsdll.h"
@@ -628,7 +626,9 @@ mac_put_params(gx_device *dev, gs_param_list *plist)
 		return code;
 	} else if (code == 0) {
 		
-		if (dev->LockSafetyParams) {
+		if (dev->LockSafetyParams &&
+			bytes_compare(outputFile.data, outputFile.size,
+			    (const byte *)mdev->outputFileName, strlen(mdev->outputFileName))) {
 			param_signal_error(plist, "OutputFile", gs_error_invalidaccess);
 			return gs_error_invalidaccess;
 		}
