@@ -1,22 +1,16 @@
-/* Copyright (C) 1989, 1995, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-
-/*$Id: gdevmem.c,v 1.2.6.1.2.1 2003/01/17 00:49:00 giles Exp $ */
+/* $Id: gdevmem.c 8130 2007-07-16 17:25:30Z ray $ */
 /* Generic "memory" (stored bitmap) device */
 #include "memory_.h"
 #include "gx.h"
@@ -28,6 +22,7 @@
 #include "gxgetbit.h"
 #include "gxdevmem.h"		/* semi-public definitions */
 #include "gdevmem.h"		/* private definitions */
+#include "gstrans.h"
 
 /* Structure descriptor */
 public_st_device_memory();
@@ -82,31 +77,39 @@ const gs_const_string mem_mono_w_b_palette = {
 
 /* Return the appropriate memory device for a given */
 /* number of bits per pixel (0 if none suitable). */
-private const gx_device_memory *const mem_devices[33] = {
+private const gx_device_memory *const mem_devices[65] = {
     0, &mem_mono_device, &mem_mapped2_device, 0, &mem_mapped4_device,
     0, 0, 0, &mem_mapped8_device,
     0, 0, 0, 0, 0, 0, 0, &mem_true16_device,
     0, 0, 0, 0, 0, 0, 0, &mem_true24_device,
-    0, 0, 0, 0, 0, 0, 0, &mem_true32_device
+    0, 0, 0, 0, 0, 0, 0, &mem_true32_device,
+    0, 0, 0, 0, 0, 0, 0, &mem_true40_device,
+    0, 0, 0, 0, 0, 0, 0, &mem_true48_device,
+    0, 0, 0, 0, 0, 0, 0, &mem_true56_device,
+    0, 0, 0, 0, 0, 0, 0, &mem_true64_device
 };
 const gx_device_memory *
 gdev_mem_device_for_bits(int bits_per_pixel)
 {
-    return ((uint)bits_per_pixel > 32 ? (const gx_device_memory *)0 :
+    return ((uint)bits_per_pixel > 64 ? (const gx_device_memory *)0 :
 	    mem_devices[bits_per_pixel]);
 }
 /* Do the same for a word-oriented device. */
-private const gx_device_memory *const mem_word_devices[33] = {
+private const gx_device_memory *const mem_word_devices[65] = {
     0, &mem_mono_device, &mem_mapped2_word_device, 0, &mem_mapped4_word_device,
     0, 0, 0, &mem_mapped8_word_device,
     0, 0, 0, 0, 0, 0, 0, 0 /*no 16-bit word device*/,
     0, 0, 0, 0, 0, 0, 0, &mem_true24_word_device,
-    0, 0, 0, 0, 0, 0, 0, &mem_true32_word_device
+    0, 0, 0, 0, 0, 0, 0, &mem_true32_word_device,
+    0, 0, 0, 0, 0, 0, 0, &mem_true40_word_device,
+    0, 0, 0, 0, 0, 0, 0, &mem_true48_word_device,
+    0, 0, 0, 0, 0, 0, 0, &mem_true56_word_device,
+    0, 0, 0, 0, 0, 0, 0, &mem_true64_word_device
 };
 const gx_device_memory *
 gdev_mem_word_device_for_bits(int bits_per_pixel)
 {
-    return ((uint)bits_per_pixel > 32 ? (const gx_device_memory *)0 :
+    return ((uint)bits_per_pixel > 64 ? (const gx_device_memory *)0 :
 	    mem_word_devices[bits_per_pixel]);
 }
 
@@ -121,7 +124,7 @@ gs_device_is_memory(const gx_device * dev)
     int bits_per_pixel = dev->color_info.depth;
     const gx_device_memory *mdproto;
 
-    if ((uint)bits_per_pixel > 32)
+    if ((uint)bits_per_pixel > 64)
 	return false;
     mdproto = mem_devices[bits_per_pixel];
     if (mdproto != 0 && dev_proc(dev, draw_thin_line) == dev_proc(mdproto, draw_thin_line))
@@ -164,12 +167,13 @@ gs_make_mem_device(gx_device_memory * dev, const gx_device_memory * mdproto,
 	gx_device_copy_color_procs((gx_device *)dev, target);
 	dev->cached_colors = target->cached_colors;
     }
-    if (dev->color_info.depth == 1)
+    if (dev->color_info.depth == 1) {
 	gdev_mem_mono_set_inverted(dev,
-				   (target == 0 ||
-				    (*dev_proc(target, map_rgb_color))
-			    (target, (gx_color_value) 0, (gx_color_value) 0,
-			     (gx_color_value) 0) != 0));
+				   (target == 0 || 
+                                    dev->color_info.polarity == GX_CINFO_POLARITY_SUBTRACTIVE));
+    }
+    check_device_separable((gx_device *)dev);
+    gx_device_fill_in_procs((gx_device *)dev);
 }
 /* Make a monobit memory device.  This is never a page device. */
 /* Note that white=0, black=1. */
@@ -182,6 +186,8 @@ gs_make_mem_mono_device(gx_device_memory * dev, gs_memory_t * mem,
     set_dev_proc(dev, get_page_device, gx_default_get_page_device);
     gx_device_set_target((gx_device_forward *)dev, target);
     gdev_mem_mono_set_inverted(dev, true);
+    check_device_separable((gx_device *)dev);
+    gx_device_fill_in_procs((gx_device *)dev);
 }
 
 /* Define whether a monobit memory device is inverted (black=1). */
@@ -203,9 +209,11 @@ gdev_mem_mono_set_inverted(gx_device_memory * dev, bool black_is_1)
  * The computation for planar devices is a little messier.  Each plane
  * must pad its scan lines, and then we must pad again for the pointer
  * tables (one table per plane).
+ *
+ * Return VMerror if the size exceeds max ulong 
  */
-ulong
-gdev_mem_bits_size(const gx_device_memory * dev, int width, int height)
+int
+gdev_mem_bits_size(const gx_device_memory * dev, int width, int height, ulong *psize)
 {
     int num_planes = dev->num_planes;
     gx_render_plane_t plane1;
@@ -219,37 +227,70 @@ gdev_mem_bits_size(const gx_device_memory * dev, int width, int height)
 	planes = &plane1, plane1.depth = dev->color_info.depth, num_planes = 1;
     for (size = 0, pi = 0; pi < num_planes; ++pi)
 	size += bitmap_raster(width * planes[pi].depth);
-    return ROUND_UP(size * height, ARCH_ALIGN_PTR_MOD);
+    if (height != 0)
+	if (size > (max_ulong - ARCH_ALIGN_PTR_MOD) / (ulong)height)
+	    return_error(gs_error_VMerror);
+    *psize = ROUND_UP(size * height, ARCH_ALIGN_PTR_MOD);
+    return 0;
 }
 ulong
 gdev_mem_line_ptrs_size(const gx_device_memory * dev, int width, int height)
 {
     return (ulong)height * sizeof(byte *) * max(dev->num_planes, 1);
 }
-ulong
-gdev_mem_data_size(const gx_device_memory * dev, int width, int height)
+int 
+gdev_mem_data_size(const gx_device_memory * dev, int width, int height, ulong *psize)
 {
-    return gdev_mem_bits_size(dev, width, height) +
-	gdev_mem_line_ptrs_size(dev, width, height);
+    ulong bits_size;
+    ulong line_ptrs_size = gdev_mem_line_ptrs_size(dev, width, height);
+
+    if (gdev_mem_bits_size(dev, width, height, &bits_size) < 0 ||
+	bits_size > max_ulong - line_ptrs_size)
+	return_error(gs_error_VMerror);
+    *psize = bits_size + line_ptrs_size;
+    return 0;
 }
 /*
  * Do the inverse computation: given a width (in pixels) and a buffer size,
  * compute the maximum height.
  */
 int
-gdev_mem_max_height(const gx_device_memory * dev, int width, ulong size)
+gdev_mem_max_height(const gx_device_memory * dev, int width, ulong size,
+		bool page_uses_transparency)
 {
-    ulong max_height = size /
-	(bitmap_raster(width * dev->color_info.depth) +
-	 sizeof(byte *) * max(dev->num_planes, 1));
-    int height = (int)min(max_height, max_int);
+    int height;
+    ulong max_height;
+    ulong data_size;
 
-    /*
-     * Because of alignment rounding, the just-computed height might
-     * be too large by a small amount.  Adjust it the easy way.
-     */
-    while (gdev_mem_data_size(dev, width, height) > size)
-	--height;
+    if (page_uses_transparency) {
+        /*
+         * If the device is using PDF 1.4 transparency then we will need to
+         * also allocate image buffers for doing the blending operations.
+         * We can only estimate the space requirements.  However since it
+	 * is only an estimate, we may exceed our desired buffer space while
+	 * processing the file.
+	 */
+        max_height = size / (bitmap_raster(width
+		* dev->color_info.depth + ESTIMATED_PDF14_ROW_SPACE(width))
+		+ sizeof(byte *) * max(dev->num_planes, 1));
+        height = (int)min(max_height, max_int);
+    } else {
+	/* For non PDF 1.4 transparency, we can do an exact calculation */
+        max_height = size /
+	    (bitmap_raster(width * dev->color_info.depth) +
+	     sizeof(byte *) * max(dev->num_planes, 1));
+        height = (int)min(max_height, max_int);
+        /*
+         * Because of alignment rounding, the just-computed height might
+         * be too large by a small amount.  Adjust it the easy way.
+         */
+        do {
+	    gdev_mem_data_size(dev, width, height, &data_size);
+	    if (data_size <= size)
+	    	break;
+	    --height;
+	} while (data_size > size);
+    }
     return height;
 }
 
@@ -269,14 +310,16 @@ int
 gdev_mem_open_scan_lines(gx_device_memory *mdev, int setup_height)
 {
     bool line_pointers_adjacent = true;
+    ulong size;
 
     if (setup_height < 0 || setup_height > mdev->height)
 	return_error(gs_error_rangecheck);
     if (mdev->bitmap_memory != 0) {
 	/* Allocate the data now. */
-	ulong size = gdev_mem_bitmap_size(mdev);
+	if (gdev_mem_bitmap_size(mdev, &size) < 0)
+	    return_error(gs_error_VMerror);
 
-	if ((uint) size != size)
+	if ((uint) size != size)	/* ulong may be bigger than uint */
 	    return_error(gs_error_limitcheck);
 	mdev->base = gs_alloc_bytes(mdev->bitmap_memory, (uint)size,
 				    "mem_open");
@@ -295,9 +338,10 @@ gdev_mem_open_scan_lines(gx_device_memory *mdev, int setup_height)
 	mdev->foreign_line_pointers = false;
 	line_pointers_adjacent = false;
     }
-    if (line_pointers_adjacent)
-	mdev->line_ptrs = (byte **)
-	    (mdev->base + gdev_mem_bits_size(mdev, mdev->width, mdev->height));
+    if (line_pointers_adjacent) {
+	gdev_mem_bits_size(mdev, mdev->width, mdev->height, &size);
+	mdev->line_ptrs = (byte **)(mdev->base + size);
+    }
     mdev->raster = gdev_mem_raster(mdev);
     return gdev_mem_set_line_ptrs(mdev, NULL, 0, NULL, setup_height);
 }
@@ -518,38 +562,63 @@ mem_word_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
 /* (2, 4, or 8 bits per pixel.) */
 /* This requires searching the palette. */
 gx_color_index
-mem_mapped_map_rgb_color(gx_device * dev, gx_color_value r, gx_color_value g,
-			 gx_color_value b)
+mem_mapped_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
-    byte br = gx_color_value_to_byte(r);
-    byte bg = gx_color_value_to_byte(g);
-    byte bb = gx_color_value_to_byte(b);
+    byte br = gx_color_value_to_byte(cv[0]);
+    
     register const byte *pptr = mdev->palette.data;
     int cnt = mdev->palette.size;
     const byte *which = 0;	/* initialized only to pacify gcc */
     int best = 256 * 3;
 
-    while ((cnt -= 3) >= 0) {
-	register int diff = *pptr - br;
+    if (mdev->color_info.num_components != 1) {
+	/* not 1 component, assume three */
+	/* The comparison is rather simplistic, treating differences in	*/
+	/* all components as equal. Better choices would be 'distance'	*/
+	/* in HLS space or other, but these would be much slower.	*/
+	/* At least exact matches will be found.			*/
+	byte bg = gx_color_value_to_byte(cv[1]);
+	byte bb = gx_color_value_to_byte(cv[2]);
 
-	if (diff < 0)
-	    diff = -diff;
-	if (diff < best) {	/* quick rejection */
-	    int dg = pptr[1] - bg;
+	while ((cnt -= 3) >= 0) {
+	    register int diff = *pptr - br;
 
-	    if (dg < 0)
-		dg = -dg;
-	    if ((diff += dg) < best) {	/* quick rejection */
-		int db = pptr[2] - bb;
+	    if (diff < 0)
+		diff = -diff;
+	    if (diff < best) {	/* quick rejection */
+		    int dg = pptr[1] - bg;
 
-		if (db < 0)
-		    db = -db;
-		if ((diff += db) < best)
-		    which = pptr, best = diff;
+		if (dg < 0)
+		    dg = -dg;
+		if ((diff += dg) < best) {	/* quick rejection */
+		    int db = pptr[2] - bb;
+
+		    if (db < 0)
+			db = -db;
+		    if ((diff += db) < best)
+			which = pptr, best = diff;
+		}
 	    }
+	    if (diff == 0)	/* can't get any better than 0 diff */
+		break;
+	    pptr += 3;
+	}
+    } else {
+	/* Gray scale conversion. The palette is made of three equal	*/
+	/* components, so this case is simpler.				*/
+	while ((cnt -= 3) >= 0) {
+	    register int diff = *pptr - br;
+
+	    if (diff < 0)
+		diff = -diff;
+	    if (diff < best) {	/* quick rejection */
+		which = pptr, best = diff;
+	    }
+	    if (diff == 0)
+		break;
+	    pptr += 3;
 	}
-	pptr += 3;
     }
     return (gx_color_index) ((which - mdev->palette.data) / 3);
 }
