@@ -1,22 +1,16 @@
-/* Copyright (C) 1991, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-
-/*$Id: gdevmem.h,v 1.3.6.1.2.1 2003/01/17 00:49:00 giles Exp $ */
+/* $Id: gdevmem.h 8022 2007-06-05 22:23:38Z giles $ */
 /* Private definitions for memory devices. */
 
 #ifndef gdevmem_INCLUDED
@@ -193,7 +187,7 @@ dev_proc_strip_copy_rop(mem_default_strip_copy_rop);
 
 /* Swap a rectangle of bytes, for converting between word- and */
 /* byte-oriented representation. */
-void mem_swap_byte_rect(P6(byte *, uint, int, int, int, bool));
+void mem_swap_byte_rect(byte *, uint, int, int, int, bool);
 
 /* Copy a rectangle of bytes from a source to a destination. */
 #define mem_copy_byte_rect(mdev, base, sourcex, sraster, x, y, w, h)\
@@ -211,6 +205,10 @@ extern const gx_device_memory mem_mapped8_device;
 extern const gx_device_memory mem_true16_device;
 extern const gx_device_memory mem_true24_device;
 extern const gx_device_memory mem_true32_device;
+extern const gx_device_memory mem_true40_device;
+extern const gx_device_memory mem_true48_device;
+extern const gx_device_memory mem_true56_device;
+extern const gx_device_memory mem_true64_device;
 extern const gx_device_memory mem_planar_device;
 /*
  * We declare the RasterOp implementation procedures here because they are
@@ -227,6 +225,10 @@ dev_proc_strip_copy_rop(mem_gray8_rgb24_strip_copy_rop);
 #  define mem_mapped8_word_device mem_mapped8_device
 #  define mem_true24_word_device mem_true24_device
 #  define mem_true32_word_device mem_true32_device
+#  define mem_true40_word_device mem_true40_device
+#  define mem_true48_word_device mem_true48_device
+#  define mem_true56_word_device mem_true56_device
+#  define mem_true64_word_device mem_true64_device
 #else
 extern const gx_device_memory mem_mono_word_device;
 extern const gx_device_memory mem_mapped2_word_device;
@@ -234,6 +236,10 @@ extern const gx_device_memory mem_mapped4_word_device;
 extern const gx_device_memory mem_mapped8_word_device;
 extern const gx_device_memory mem_true24_word_device;
 extern const gx_device_memory mem_true32_word_device;
+extern const gx_device_memory mem_true40_word_device;
+extern const gx_device_memory mem_true48_word_device;
+extern const gx_device_memory mem_true56_word_device;
+extern const gx_device_memory mem_true64_word_device;
 
 #endif
 /* Provide standard palettes for 1-bit devices. */
