@@ -1,22 +1,16 @@
-/* Copyright (C) 1993, 1994, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-
-/*$Id: gdevmpla.c,v 1.2.6.1.2.1 2003/01/17 00:49:01 giles Exp $ */
+/* $Id: gdevmpla.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Any-depth planar "memory" (stored bitmap) device */
 #include "memory_.h"
 #include "gx.h"
@@ -260,7 +254,7 @@ mem_planar_copy_color(gx_device * dev, const byte * base, int sourcex,
 		    for (ix = 0; ix < cw; ++ix) {
 			gx_color_index value;
 
-			sample_load_next32(value, sptr, sbit, source_depth);
+			sample_load_next_any(value, sptr, sbit, source_depth);
 			value = (value >> shift) & mask;
 			sample_store_next16(value, dptr, dbit, plane_depth,
 					    dbbyte);
@@ -422,7 +416,7 @@ planar_to_chunky(gx_device_memory *mdev, int x, int y, int w, int h,
 		sample_load_next16(value, sptr[pi], sbit[pi], plane_depth);
 		color |= (gx_color_index)value << mdev->planes[pi].shift;
 	    }
-	    sample_store_next32(color, dptr, dbit, ddepth, dbbyte);
+	    sample_store_next_any(color, dptr, dbit, ddepth, dbbyte);
 	}
 	sample_store_flush(dptr, dbit, ddepth, dbbyte);
     }
