@@ -1,22 +1,17 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevpcfb.c,v 1.2.6.1.2.1 2003/01/17 00:49:01 giles Exp $ */
+/* $Id: gdevpcfb.c 8022 2007-06-05 22:23:38Z giles $ */
 /* IBM PC frame buffer (EGA/VGA) drivers */
 #include "memory_.h"
 #include "gconfigv.h"		/* for USE_ASM */
@@ -180,21 +175,21 @@ svga16_put_params(gx_device * dev, gs_param_list * plist)
 }
 
 /* Map a r-g-b color to an EGA color code. */
-#define Nb gx_color_value_bits
 private gx_color_index
-ega0_map_rgb_color(gx_device * dev, gx_color_value r, gx_color_value g,
-		   gx_color_value b)
+ega0_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 {
-    return pc_4bit_map_rgb_color(dev, r, r, r);
+    return pc_4bit_map_rgb_color(dev, cv);
 }
 private gx_color_index
-ega1_map_rgb_color(gx_device * dev, gx_color_value r, gx_color_value g,
-		   gx_color_value b)
+ega1_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 {
-#define cvtop (gx_color_value)(1 << (Nb - 1))
-    return pc_4bit_map_rgb_color(dev, r & cvtop, g & cvtop, b & cvtop);
+    const gx_color_value cvtop = (1 << (gx_color_value_bits - 1));
+    gx_color_value cvt[3];
+    cvt[0] = cv[0] & cvtop;
+    cvt[1] = cv[1] & cvtop;
+    cvt[2] = cv[2] & cvtop;
+    return pc_4bit_map_rgb_color(dev, cvt);
 }
-#undef Nb
 
 /* Map a color code to r-g-b. */
 #define icolor (int)color
@@ -232,7 +227,7 @@ typedef rop_params _ss *rop_ptr;
 /* Assembly language routines */
 
 #if USE_ASM
-void memsetcol(P1(rop_ptr));	/* dest, draster, height, data */
+void memsetcol(rop_ptr);	/* dest, draster, height, data */
 #else
 #define memsetcol cmemsetcol
 private void
@@ -252,7 +247,7 @@ cmemsetcol(rop_ptr rop)
 #endif
 
 #if USE_ASM
-void memsetrect(P1(rop_ptr));	/* dest, draster, width, height, data */
+void memsetrect(rop_ptr);	/* dest, draster, width, height, data */
 #else
 #define memsetrect cmemsetrect
 private void
@@ -293,7 +288,7 @@ cmemsetrect(rop_ptr rop)
 #endif
 
 #if USE_ASM
-void memrwcol(P1(rop_ptr));	/* dest, draster, src, sraster, height, shift, invert */
+void memrwcol(rop_ptr);	/* dest, draster, src, sraster, height, shift, invert */
 #  define memrwcol0(rop) memrwcol(rop)	/* same except shift = 0 */
 #else
 #  define memrwcol cmemrwcol
@@ -334,7 +329,7 @@ cmemrwcol0(rop_ptr rop)
 #endif
 
 #if USE_ASM
-void memrwcol2(P1(rop_ptr));	/* dest, draster, src, sraster, height, shift, invert */
+void memrwcol2(rop_ptr);	/* dest, draster, src, sraster, height, shift, invert */
 #else
 #define memrwcol2 cmemrwcol2
 private void
@@ -356,9 +351,9 @@ cmemrwcol2(rop_ptr rop)
 #endif
 
 /* Forward definitions */
-int ega_write_dot(P4(gx_device *, int, int, gx_color_index));
-private void fill_rectangle(P4(rop_ptr, int, int, int));
-private void fill_row_only(P4(byte *, int, int, int));
+int ega_write_dot(gx_device *, int, int, gx_color_index);
+private void fill_rectangle(rop_ptr, int, int, int);
+private void fill_row_only(byte *, int, int, int);
 
 /* Clean up after writing */
 #define dot_end()\
