@@ -1,22 +1,17 @@
-/* Copyright (C) 1995, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevphex.c,v 1.2.6.2.2.1 2003/01/17 00:49:01 giles Exp $ */
+/* $Id: gdevphex.c 8022 2007-06-05 22:23:38Z giles $ */
 
 /****************************************************************************/
 /*	Ghostscript printer driver for Epson Color Photo, Photo EX, Photo 700	*/
@@ -1297,14 +1292,14 @@ private	byte	dmatrix[ DMATRIX_Y ][ DMATRIX_X ] = {
 
 static	CCOMP	ctable[] = {
 
-	{ -255, -255,   0,   0, 255 },		// same as green
-	{  102,    0, 255,   0,   0 },		// cyan
-	{  255,  255, 255, 255,   0 },		// blue
-	{  560,  512,   0, 255,   0 },		// magenta
-	{  765,  765,   0, 255, 255 },		// red
-	{ 1045, 1020,   0,   0, 255 },		// yellow
-	{ 1275, 1275, 255,   0, 255 },		// green
-	{ 1632, 1530, 255,   0,   0 }		// same as cyan
+	{ -255, -255,   0,   0, 255 },		/* same as green */
+	{  102,    0, 255,   0,   0 },		/* cyan */
+	{  255,  255, 255, 255,   0 },		/* blue */
+	{  560,  512,   0, 255,   0 },		/* magenta */
+	{  765,  765,   0, 255, 255 },		/* red */
+	{ 1045, 1020,   0,   0, 255 },		/* yellow */
+	{ 1275, 1275, 255,   0, 255 },		/* green */
+	{ 1632, 1530, 255,   0,   0 }		/* same as cyan */
 };
 
 /*
@@ -1734,14 +1729,14 @@ double		psize;
 	   we need error buffers and stuff. All in all, we'll request
 	   about 1.5 ~ 2M. */
 		
-	if ( ! ( render = (RENDER *) gs_malloc( 1, sizeof( RENDER ), "PhotoEX" )))
+	if ( ! ( render = (RENDER *) gs_malloc( dev->memory, 1, sizeof( RENDER ), "PhotoEX" )))
 		
 		return_error( gs_error_VMerror );
 	
-	if ( ! ( render->dbuff = (byte *) gs_malloc( pixels, sizeof( long ), 
+	if ( ! ( render->dbuff = (byte *) gs_malloc( dev->memory, pixels, sizeof( long ), 
 			"PhotoEX" ) ) ) {
 			
-		gs_free( render, 1, sizeof( RENDER ), "PhotoEX" );
+		gs_free( dev->memory, render, 1, sizeof( RENDER ), "PhotoEX" );
 		return_error( gs_error_VMerror );
 	}
 	
@@ -1798,8 +1793,8 @@ double		psize;
 	
 	/* Release the memory and return */
 	
-	gs_free( render->dbuff, pixels, sizeof( long ), "PhotoEX" );
-	gs_free( render, 1, sizeof( RENDER ), "PhotoEX" );
+	gs_free( dev->memory, render->dbuff, pixels, sizeof( long ), "PhotoEX" );
+	gs_free( dev->memory, render, 1, sizeof( RENDER ), "PhotoEX" );
 	return( 0 );
 }
 
