@@ -1,22 +1,17 @@
-/* Copyright (C) 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevpsdp.c,v 1.8.6.1.2.1 2003/01/17 00:49:01 giles Exp $ */
+/* $Id: gdevpsdp.c 8022 2007-06-05 22:23:38Z giles $ */
 /* (Distiller) parameter handling for PostScript and PDF writers */
 #include "string_.h"
 #include "jpeglib_.h"		/* for sdct.h */
@@ -33,6 +28,14 @@
 #include "spprint.h"
 #include "srlx.h"
 #include "szlibx.h"
+#ifdef USE_LDF_JB2
+#include "sjbig2_luratech.h"
+#endif
+#ifdef USE_LWF_JP2
+#include "sjpx_luratech.h"
+#endif
+
+
 
 /* Define a (bogus) GC descriptor for gs_param_string. */
 /* The only ones we use are GC-able and not persistent. */
@@ -96,6 +99,9 @@ private const psdf_image_filter_name Poly_filters[] = {
     {"DCTEncode", &s_DCTE_template},
     {"FlateEncode", &s_zlibE_template, psdf_version_ll3},
     {"LZWEncode", &s_LZWE_template},
+#ifdef USE_LWF_JP2
+    {"JPXEncode", &s_jpxe_template},
+#endif
     {0, 0}
 };
 
@@ -104,6 +110,9 @@ private const psdf_image_filter_name Mono_filters[] = {
     {"FlateEncode", &s_zlibE_template, psdf_version_ll3},
     {"LZWEncode", &s_LZWE_template},
     {"RunLengthEncode", &s_RLE_template},
+#ifdef USE_LDF_JB2
+    {"JBIG2Encode", &s_jbig2encode_template},
+#endif
     {0, 0}
 };
 
@@ -114,14 +123,15 @@ typedef struct psdf_image_param_names_s {
     float DownsampleThreshold_default;
     const psdf_image_filter_name *filter_names;
     const char *Filter;
-    gs_param_item_t items[8];	/* AutoFilter (not used for mono), */
+    const char *AutoFilterStrategy;
+    gs_param_item_t items[9];	/* AutoFilter (not used for mono), */
 				/* AntiAlias, */
 				/* Depth, Downsample, DownsampleThreshold, */
-				/* Encode, Resolution, end marker */
+				/* Encode, Resolution, AutoFilterStrategy, end marker */
 } psdf_image_param_names_t;
 #define pi(key, type, memb) { key, type, offset_of(psdf_image_params, memb) }
-#define psdf_image_param_names(acs, aa, af, de, di, ds, dt, dst, dstd, e, f, fns, r)\
-    acs, di, dt, dstd, fns, f, {\
+#define psdf_image_param_names(acs, aa, af, de, di, ds, dt, dst, dstd, e, f, fns, r, afs)\
+    acs, di, dt, dstd, fns, f, afs, {\
       pi(af, gs_param_type_bool, AutoFilter),\
       pi(aa, gs_param_type_bool, AntiAlias),\
       pi(de, gs_param_type_int, Depth),\
@@ -129,6 +139,7 @@ typedef struct psdf_image_param_names_s {
       pi(dst, gs_param_type_float, DownsampleThreshold),\
       pi(e, gs_param_type_bool, Encode),\
       pi(r, gs_param_type_int, Resolution),\
+      pi(afs, gs_param_type_int, AutoFilterStrategy),\
       gs_param_item_end\
     }
 
@@ -139,7 +150,7 @@ private const psdf_image_param_names_t Color_names = {
 	"DownsampleColorImages", "ColorImageDownsampleType",
 	"ColorImageDownsampleThreshold", 1.5,
 	"EncodeColorImages", "ColorImageFilter", Poly_filters,
-	"ColorImageResolution"
+	"ColorImageResolution", 0
     )
 };
 private const psdf_image_param_names_t Gray_names = {
@@ -149,7 +160,7 @@ private const psdf_image_param_names_t Gray_names = {
 	"DownsampleGrayImages", "GrayImageDownsampleType",
 	"GrayImageDownsampleThreshold", 2.0,
 	"EncodeGrayImages", "GrayImageFilter", Poly_filters,
-	"GrayImageResolution"
+	"GrayImageResolution", 0
     )
 };
 private const psdf_image_param_names_t Mono_names = {
@@ -159,7 +170,27 @@ private const psdf_image_param_names_t Mono_names = {
 	"DownsampleMonoImages", "MonoImageDownsampleType",
 	"MonoImageDownsampleThreshold", 2.0,
 	"EncodeMonoImages", "MonoImageFilter", Mono_filters,
-	"MonoImageResolution"
+	"MonoImageResolution", 0
+    )
+};
+private const psdf_image_param_names_t Color_names15 = {
+    psdf_image_param_names(
+	"ColorACSImageDict", "AntiAliasColorImages", "AutoFilterColorImages",
+	"ColorImageDepth", "ColorImageDict",
+	"DownsampleColorImages", "ColorImageDownsampleType",
+	"ColorImageDownsampleThreshold", 1.5,
+	"EncodeColorImages", "ColorImageFilter", Poly_filters,
+	"ColorImageResolution", "ColorAutoFilterStrategy"
+    )
+};
+private const psdf_image_param_names_t Gray_names15 = {
+    psdf_image_param_names(
+	"GrayACSImageDict", "AntiAliasGrayImages", "AutoFilterGrayImages",
+	"GrayImageDepth", "GrayImageDict",
+	"DownsampleGrayImages", "GrayImageDownsampleType",
+	"GrayImageDownsampleThreshold", 2.0,
+	"EncodeGrayImages", "GrayImageFilter", Poly_filters,
+	"GrayImageResolution", "GrayAutoFilterStrategy"
     )
 };
 #undef pi
@@ -308,6 +339,13 @@ psdf_get_image_params(gs_param_list * plist,
 				    pnames->filter_names[0].pname :
 				    params->Filter))) < 0
 	   /* (Resolution) */
+#ifdef USE_LWF_JP2
+	   || 
+	   (pnames->AutoFilterStrategy != 0 &&
+	   (code = psdf_write_name(plist, pnames->AutoFilterStrategy,
+				   (params->AutoFilterStrategy == 0 ?
+				   "JPEG2000" : params->AutoFilterStrategy))) < 0)
+#endif
 	)
 	DO_NOTHING;
     return code;
@@ -351,7 +389,9 @@ gdev_psdf_get_params(gx_device * dev, gs_param_list * plist)
 
     /* Color sampled image parameters */
 
-	(code = psdf_get_image_params(plist, &Color_names, &pdev->params.ColorImage)) < 0 ||
+    (code = psdf_get_image_params(plist, 
+			(pdev->ParamCompatibilityLevel >= 1.5 ? &Color_names15 : &Color_names), 
+			&pdev->params.ColorImage)) < 0 ||
 	(code = psdf_write_name(plist, "ColorConversionStrategy",
 		ColorConversionStrategy_names[(int)pdev->params.ColorConversionStrategy])) < 0 ||
 	(code = psdf_write_string_param(plist, "CalCMYKProfile",
@@ -365,7 +405,9 @@ gdev_psdf_get_params(gx_device * dev, gs_param_list * plist)
 
     /* Gray sampled image parameters */
 
-	(code = psdf_get_image_params(plist, &Gray_names, &pdev->params.GrayImage)) < 0 ||
+	(code = psdf_get_image_params(plist,
+			(pdev->ParamCompatibilityLevel >= 1.5 ? &Gray_names15 : &Gray_names), 
+			&pdev->params.GrayImage)) < 0 ||
 
     /* Mono sampled image parameters */
 
@@ -398,7 +440,7 @@ psdf_read_string_param(gs_param_list *plist, const char *key,
     switch (code = param_read_string(plist, key, &ps)) {
     case 0: {
 	uint size = ps.size;
-	byte *data = gs_alloc_bytes(mem, size, "psdf_read_string_param");
+	byte *data = gs_alloc_string(mem, size, "psdf_read_string_param");
 
 	if (data == 0)
 	    return_error(gs_error_VMerror);
@@ -715,6 +757,42 @@ psdf_put_image_params(const gx_device_psdf * pdev, gs_param_list * plist,
 		      &ecode);
     /* (DownsampleThreshold) */
     /* (Encode) */
+#ifdef USE_LWF_JP2
+    /* Process AutoFilterStrategy before Filter, because it sets defaults 
+       for the latter. */
+    if (pnames->AutoFilterStrategy != NULL) {
+	switch (code = param_read_string(plist, pnames->AutoFilterStrategy, &fs)) {
+	    case 0:
+		{
+		    const psdf_image_filter_name *pn = pnames->filter_names;
+		    const char *param_name = 0;
+
+		    if (gs_param_string_eq(&fs, "/JPEG")) {
+			params->AutoFilterStrategy = "/JPEG";
+			param_name = "DCTEncode";
+		    } if (gs_param_string_eq(&fs, "/JPEG2000")) {
+			params->AutoFilterStrategy = "/JPEG2000";
+			param_name = "JPXEncode";
+		    } else {
+			ecode = gs_error_rangecheck;
+			goto ipe1;
+		    }
+		    while (pn->pname != 0 && !gs_param_string_eq(&fs, param_name))
+			pn++;
+		    if (pn->pname != 0 && pn->min_version <= pdev->version) {
+			params->Filter = pn->pname;
+			params->filter_template = pn->template;
+		    }
+		    break;
+		}
+	    default:
+		ecode = code;
+	    ipe1:param_signal_error(plist, pnames->AutoFilterStrategy, ecode);
+	    case 1:
+		break;
+        }
+    }
+#endif
     switch (code = param_read_string(plist, pnames->Filter, &fs)) {
 	case 0:
 	    {
@@ -771,76 +849,78 @@ gdev_psdf_put_params(gx_device * dev, gs_param_list * plist)
 
     /*
      * If LockDistillerParams was true and isn't being set to false,
-     * ignore all other parameters.
+     * ignore all other psdf parameters.  However, do not ignore the
+     * standard device parameters.
      */
     ecode = code = param_read_bool(plist, "LockDistillerParams",
-				   &params.LockDistillerParams);
-    if (pdev->params.LockDistillerParams && params.LockDistillerParams)
-	return ecode;
+  				   &params.LockDistillerParams);
+    if (!(pdev->params.LockDistillerParams && params.LockDistillerParams)) {
+  
+	/* General parameters. */
 
-    /* General parameters. */
+	code = gs_param_read_items(plist, &params, psdf_param_items);
+	if (code < 0)
+	    ecode = code;
+	params.AutoRotatePages = (enum psdf_auto_rotate_pages)
+	    psdf_put_enum(plist, "AutoRotatePages", (int)params.AutoRotatePages,
+			  AutoRotatePages_names, &ecode);
+	params.Binding = (enum psdf_binding)
+	    psdf_put_enum(plist, "Binding", (int)params.Binding,
+			  Binding_names, &ecode);
+	params.DefaultRenderingIntent = (enum psdf_default_rendering_intent)
+	    psdf_put_enum(plist, "DefaultRenderingIntent",
+			  (int)params.DefaultRenderingIntent,
+			  DefaultRenderingIntent_names, &ecode);
+	params.TransferFunctionInfo = (enum psdf_transfer_function_info)
+	    psdf_put_enum(plist, "TransferFunctionInfo",
+			  (int)params.TransferFunctionInfo,
+			  TransferFunctionInfo_names, &ecode);
+	params.UCRandBGInfo = (enum psdf_ucr_and_bg_info)
+	    psdf_put_enum(plist, "UCRandBGInfo", (int)params.UCRandBGInfo,
+			  UCRandBGInfo_names, &ecode);
+	ecode = param_put_bool(plist, "UseFlateCompression",
+			       &params.UseFlateCompression, ecode);
 
-    code = gs_param_read_items(plist, &params, psdf_param_items);
-    if (code < 0)
-	ecode = code;
-    params.AutoRotatePages = (enum psdf_auto_rotate_pages)
-	psdf_put_enum(plist, "AutoRotatePages", (int)params.AutoRotatePages,
-		      AutoRotatePages_names, &ecode);
-    params.Binding = (enum psdf_binding)
-	psdf_put_enum(plist, "Binding", (int)params.Binding,
-		      Binding_names, &ecode);
-    params.DefaultRenderingIntent = (enum psdf_default_rendering_intent)
-	psdf_put_enum(plist, "DefaultRenderingIntent",
-		      (int)params.DefaultRenderingIntent,
-		      DefaultRenderingIntent_names, &ecode);
-    params.TransferFunctionInfo = (enum psdf_transfer_function_info)
-	psdf_put_enum(plist, "TransferFunctionInfo",
-		      (int)params.TransferFunctionInfo,
-		      TransferFunctionInfo_names, &ecode);
-    params.UCRandBGInfo = (enum psdf_ucr_and_bg_info)
-	psdf_put_enum(plist, "UCRandBGInfo", (int)params.UCRandBGInfo,
-		      UCRandBGInfo_names, &ecode);
-    ecode = param_put_bool(plist, "UseFlateCompression",
-			   &params.UseFlateCompression, ecode);
+	/* Color sampled image parameters */
 
-    /* Color sampled image parameters */
+	ecode = psdf_put_image_params(pdev, plist,
+			(pdev->ParamCompatibilityLevel >= 1.5 ? &Color_names15 : &Color_names), 
+				      &params.ColorImage, ecode);
+	params.ColorConversionStrategy = (enum psdf_color_conversion_strategy)
+	    psdf_put_enum(plist, "ColorConversionStrategy",
+			  (int)params.ColorConversionStrategy,
+			  ColorConversionStrategy_names, &ecode);
+	ecode = psdf_read_string_param(plist, "CalCMYKProfile",
+				       &params.CalCMYKProfile, mem, ecode);
+	ecode = psdf_read_string_param(plist, "CalGrayProfile",
+				       &params.CalGrayProfile, mem, ecode);
+	ecode = psdf_read_string_param(plist, "CalRGBProfile",
+				       &params.CalRGBProfile, mem, ecode);
+	ecode = psdf_read_string_param(plist, "sRGBProfile",
+				       &params.sRGBProfile, mem, ecode);
 
-    ecode = psdf_put_image_params(pdev, plist, &Color_names,
-				  &params.ColorImage, ecode);
-    params.ColorConversionStrategy = (enum psdf_color_conversion_strategy)
-	psdf_put_enum(plist, "ColorConversionStrategy",
-		      (int)params.ColorConversionStrategy,
-		      ColorConversionStrategy_names, &ecode);
-    ecode = psdf_read_string_param(plist, "CalCMYKProfile",
-				   &params.CalCMYKProfile, mem, ecode);
-    ecode = psdf_read_string_param(plist, "CalGrayProfile",
-				   &params.CalGrayProfile, mem, ecode);
-    ecode = psdf_read_string_param(plist, "CalRGBProfile",
-				   &params.CalRGBProfile, mem, ecode);
-    ecode = psdf_read_string_param(plist, "sRGBProfile",
-				   &params.sRGBProfile, mem, ecode);
+	/* Gray sampled image parameters */
 
-    /* Gray sampled image parameters */
+	ecode = psdf_put_image_params(pdev, plist,
+			(pdev->ParamCompatibilityLevel >= 1.5 ? &Gray_names15 : &Gray_names), 
+				      &params.GrayImage, ecode);
 
-    ecode = psdf_put_image_params(pdev, plist, &Gray_names,
-				  &params.GrayImage, ecode);
+	/* Mono sampled image parameters */
 
-    /* Mono sampled image parameters */
+	ecode = psdf_put_image_params(pdev, plist, &Mono_names,
+				      &params.MonoImage, ecode);
 
-    ecode = psdf_put_image_params(pdev, plist, &Mono_names,
-				  &params.MonoImage, ecode);
-
-    /* Font embedding parameters */
-
-    ecode = psdf_put_embed_param(plist, "~AlwaysEmbed", ".AlwaysEmbed",
-				 &params.AlwaysEmbed, mem, ecode);
-    ecode = psdf_put_embed_param(plist, "~NeverEmbed", ".NeverEmbed",
-				 &params.NeverEmbed, mem, ecode);
-    params.CannotEmbedFontPolicy = (enum psdf_cannot_embed_font_policy)
-	psdf_put_enum(plist, "CannotEmbedFontPolicy",
-		      (int)params.CannotEmbedFontPolicy,
-		      CannotEmbedFontPolicy_names, &ecode);
+	/* Font embedding parameters */
 
+	ecode = psdf_put_embed_param(plist, "~AlwaysEmbed", ".AlwaysEmbed",
+				     &params.AlwaysEmbed, mem, ecode);
+	ecode = psdf_put_embed_param(plist, "~NeverEmbed", ".NeverEmbed",
+				     &params.NeverEmbed, mem, ecode);
+	params.CannotEmbedFontPolicy = (enum psdf_cannot_embed_font_policy)
+	    psdf_put_enum(plist, "CannotEmbedFontPolicy",
+			  (int)params.CannotEmbedFontPolicy,
+			  CannotEmbedFontPolicy_names, &ecode);
+    }
     if (ecode < 0)
 	return ecode;
     code = gdev_vector_put_params(dev, plist);
