@@ -1,22 +1,17 @@
-/* Copyright (C) 1991, 1995, 1996, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevsvga.c,v 1.2.6.1.2.1 2003/01/17 00:49:01 giles Exp $ */
+/* $Id: gdevsvga.c 8022 2007-06-05 22:23:38Z giles $ */
 /* SuperVGA display drivers */
 #include "memory_.h"
 #include "gconfigv.h"		/* for USE_ASM */
@@ -180,13 +175,13 @@ svga_close(gx_device * dev)
 /* for compatibility with the older display modes: */
 /* these are indexed as 0.0.R0.G0.B0.R1.G1.B1. */
 gx_color_index
-svga_map_rgb_color(gx_device * dev, gx_color_value r, gx_color_value g,
-		   gx_color_value b)
+svga_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 {
     ushort rgb;
+    gx_color_value r = cv[0], g = cv[1], b = cv[2];
 
     if (fb_dev->fixed_colors) {
-	gx_color_index ci = pc_8bit_map_rgb_color(dev, r, g, b);
+	gx_color_index ci = pc_8bit_map_rgb_color(dev, cv);
 
 	/* Here is where we should permute the index to match */
 	/* the old color map... but we don't yet. */
@@ -573,9 +568,9 @@ svga_copy_alpha(gx_device * dev, const byte * base, int sourcex,
 
 private dev_proc_open_device(vesa_open);
 private const gx_device_procs vesa_procs = svga_procs(vesa_open);
-int vesa_get_mode(P0());
-void vesa_set_mode(P1(int));
-private void vesa_set_page(P3(gx_device_svga *, int, int));
+int vesa_get_mode(void);
+void vesa_set_mode(int);
+private void vesa_set_page(gx_device_svga *, int, int);
 gx_device_svga far_data gs_vesa_device =
 svga_device(vesa_procs, "vesa", vesa_get_mode, vesa_set_mode, vesa_set_page);
 
@@ -608,7 +603,7 @@ typedef struct {
     ushort win_size;
     ushort win_a_segment;
     ushort win_b_segment;
-    void (*win_func_ptr) (P2(int, int));
+    void (*win_func_ptr) (int, int);
     ushort bytes_per_line;
     /* Optional information */
     ushort x_resolution;
@@ -755,7 +750,7 @@ private void
 vesa_set_page(gx_device_svga * dev, int pn, int wnum)
 {
 #if USE_ASM
-    extern void vesa_call_set_page(P3(void (*)(P2(int, int)), int, int));
+    extern void vesa_call_set_page(void (*)(int, int), int, int);
 
     if (dev->info.vesa.bios_set_page != NULL)
 	vesa_call_set_page(dev->info.vesa.bios_set_page, pn << dev->info.vesa.pn_shift, wnum);
@@ -776,9 +771,9 @@ vesa_set_page(gx_device_svga * dev, int pn, int wnum)
 
 private dev_proc_open_device(atiw_open);
 private const gx_device_procs atiw_procs = svga_procs(atiw_open);
-private int atiw_get_mode(P0());
-private void atiw_set_mode(P1(int));
-private void atiw_set_page(P3(gx_device_svga *, int, int));
+private int atiw_get_mode(void);
+private void atiw_set_mode(int);
+private void atiw_set_page(gx_device_svga *, int, int);
 gx_device_svga far_data gs_atiw_device =
 svga_device(atiw_procs, "atiw", atiw_get_mode, atiw_set_mode, atiw_set_page);
 
@@ -847,7 +842,7 @@ private dev_proc_open_device(tvga_open);
 private const gx_device_procs tvga_procs = svga_procs(tvga_open);
 
 /* We can use the atiw_get/set_mode procedures. */
-private void tvga_set_page(P3(gx_device_svga *, int, int));
+private void tvga_set_page(gx_device_svga *, int, int);
 gx_device_svga far_data gs_tvga_device =
 svga_device(tvga_procs, "tvga", atiw_get_mode, atiw_set_mode, tvga_set_page);
 
@@ -894,7 +889,7 @@ private const gx_device_procs tseng_procs =
 svga_procs(tseng_open);
 
 /* We can use the atiw_get/set_mode procedures. */
-private void tseng_set_page(P3(gx_device_svga *, int, int));
+private void tseng_set_page(gx_device_svga *, int, int);
 
 /* The 256-color Tseng device */
 gx_device_svga far_data gs_tseng_device =
@@ -957,7 +952,7 @@ private dev_proc_open_device(cirr_open);
 private gx_device_procs cirr_procs = svga_procs(cirr_open);
 
 /* We can use the atiw_get/set_mode procedures. */
-private void cirr_set_page(P3(gx_device_svga *, int, int));
+private void cirr_set_page(gx_device_svga *, int, int);
 gx_device_svga gs_cirr_device =
 svga_device(cirr_procs, "cirr", atiw_get_mode, atiw_set_mode, cirr_set_page);
 
@@ -1005,7 +1000,7 @@ private dev_proc_open_device(ali_open);
 private const gx_device_procs ali_procs = svga_procs(ali_open);
 
 /* We can use the atiw_get/set_mode procedures. */
-private void ali_set_page(P3(gx_device_svga *, int, int));
+private void ali_set_page(gx_device_svga *, int, int);
 
 /* The 256-color Avance Logic device */
 gx_device_svga gs_ali_device =
