@@ -1,20 +1,17 @@
-/* Copyright (C) 1992, 1995, 1996, 1997, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevwdib.c,v 1.3.2.1 2002/02/22 19:45:57 ray Exp $ */
+/* $Id: gdevwdib.c 8022 2007-06-05 22:23:38Z giles $ */
 /* MS Windows 3.n driver for Ghostscript using a DIB for buffering. */
 #include "gdevmswn.h"
 #include "gxdevmem.h"
@@ -99,7 +96,7 @@ private const gx_device_procs win_dib_procs =
 gx_device_win_dib far_data gs_mswindll_device =
 {
     std_device_std_body(gx_device_win_dib, &win_dib_procs, "mswindll",
-			INITIAL_WIDTH, INITIAL_HEIGHT,	/* win_open() fills these in later */
+			INITIAL_WIDTH, INITIAL_HEIGHT,/* win_open() fills these in later */
 			INITIAL_RESOLUTION, INITIAL_RESOLUTION	/* win_open() fills these in later */
     ),
     {0},			/* std_procs */
@@ -438,11 +435,10 @@ win_dib_make_dib(gx_device_win * dev, int orgx, int orgy, int wx, int wy)
     int palcount;
     int i;
     UINT lwidth;		/* line width in bytes rounded up to multiple of 4 bytes */
-
-#ifdef USE_SEGMENTS
     int loffset;		/* byte offset to start of line */
-    UINT lseg;			/* bytes remaining in this segment */
 
+#if USE_SEGMENTS
+    UINT lseg;			/* bytes remaining in this segment */
 #endif
 
     if (orgx + wx > wdev->width)
@@ -457,7 +453,7 @@ win_dib_make_dib(gx_device_win * dev, int orgx, int orgy, int wx, int wy)
     if (wdev->color_info.depth > 16)
 	palcount = 0;
     else if (wdev->color_info.depth > 8)
-	palcount = 3;		// 16-bit BI_BITFIELDS
+	palcount = 3;		/* 16-bit BI_BITFIELDS */
     else
 	palcount = wdev->nColors;
 
@@ -542,13 +538,12 @@ win_dib_alloc_bitmap(gx_device_win * dev, gx_device * param_dev)
     gx_device_memory mdev;
     HGLOBAL hmdata;
     byte FAR *base;
-    byte FAR *ptr_base;
     uint ptr_size;
     uint raster;
-
-#ifdef USE_SEGMENTS
     ulong data_size;
 
+#if USE_SEGMENTS
+   byte FAR *ptr_base;
 #endif
 
 #ifdef __WIN32__
