@@ -1,22 +1,17 @@
-/* Copyright (C) 1989, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevxini.c,v 1.10.4.1.2.1 2003/01/17 00:49:02 giles Exp $ */
+/* $Id: gdevxini.c 8056 2007-06-18 06:37:58Z ray $ */
 /* X Windows driver initialization/finalization */
 #include "memory_.h"
 #include "x_.h"
@@ -28,7 +23,7 @@
 #include "gdevbbox.h"
 #include "gdevx.h"
 
-extern char *getenv(P1(const char *));
+extern char *getenv(const char *);
 
 extern const gx_device_bbox gs_bbox_device;
 extern const gx_device_X gs_x11_device;
@@ -50,7 +45,7 @@ private_st_x11fontmap();
 /* ---------------- Opening/initialization ---------------- */
 
 /* Forward references */
-private void gdev_x_setup_fontmap(P1(gx_device_X *));
+private void gdev_x_setup_fontmap(gx_device_X *);
 
 /* Catch the alloc error when there is not enough resources for the
  * backing pixmap.  Automatically shut off backing pixmap and let the
@@ -98,7 +93,7 @@ gdev_x_open(gx_device_X * xdev)
     Display *dpy;
     XColor xc;
     int zero = 0;
-    int xid_height, xid_width;
+    int xid_height = 0, xid_width = 0;
     int code;
 
 #ifdef DEBUG
@@ -306,6 +301,9 @@ gdev_x_open(gx_device_X * xdev)
 	XCloseDisplay(xdev->dpy);
 	return code;
     }
+    /* Now that the color map is setup check if the device is separable. */
+    check_device_separable((gx_device *)xdev);
+
     gdev_x_setup_fontmap(xdev);
 
     if (!xdev->ghostview) {
@@ -552,8 +550,8 @@ x_set_buffer(gx_device_X * xdev)
 	    byte *buffer;
 	    ulong space;
 
-	    space = gdev_mem_data_size(mdev, xdev->width, xdev->height);
-	    if (space > xdev->MaxBitmap) {
+	    if (gdev_mem_data_size(mdev, xdev->width, xdev->height, &space) < 0 ||
+	    	space > xdev->MaxBitmap) {
 		buffered = false;
 		goto setup;
 	    }
@@ -608,11 +606,14 @@ x_set_buffer(gx_device_X * xdev)
 	COPY_PROC(text_begin);
 #undef COPY_PROC
 	if (xdev->is_buffered) {
+    	    check_device_separable((gx_device *)xdev);
 	    gx_device_forward_fill_in_procs((gx_device_forward *)xdev);
 	    xdev->box_procs = gdev_x_box_procs;
 	    xdev->box_proc_data = xdev;
-	} else
+	} else {
+    	    check_device_separable((gx_device *)xdev);
 	    gx_device_fill_in_procs((gx_device *)xdev);
+	}
     }
     return 0;
 }
