@@ -1,22 +1,17 @@
-/* Copyright (C) 1992, 1995, 1996, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gscie.c,v 1.4.2.1.2.1 2003/01/17 00:49:02 giles Exp $ */
+/* $Id: gscie.c 8022 2007-06-05 22:23:38Z giles $ */
 /* CIE color rendering cache management */
 #include "math_.h"
 #include "memory_.h"
@@ -33,24 +28,31 @@
 #include "gzstate.h"
 #include "gsicc.h"
 
+/*
+ * Define whether to optimize the CIE mapping process by combining steps.
+ * This should only be disabled (commented out) for debugging.
+ */
+#define OPTIMIZE_CIE_MAPPING
+
 /* Forward references */
-private int cie_joint_caches_init(P3(gx_cie_joint_caches *,
-				     const gs_cie_common *,
-				     gs_cie_render *));
-private void cie_joint_caches_complete(P4(gx_cie_joint_caches *,
-					  const gs_cie_common *,
-					  const gs_cie_abc *,
-					  const gs_cie_render *));
-private void cie_cache_restrict(P2(cie_cache_floats *, const gs_range *));
-private void cie_mult3(P3(const gs_vector3 *, const gs_matrix3 *,
-			  gs_vector3 *));
-private void cie_matrix_mult3(P3(const gs_matrix3 *, const gs_matrix3 *,
-				 gs_matrix3 *));
-private void cie_invert3(P2(const gs_matrix3 *, gs_matrix3 *));
-private void cie_matrix_init(P1(gs_matrix3 *));
+private int cie_joint_caches_init(gx_cie_joint_caches *,
+				  const gs_cie_common *,
+				  gs_cie_render *);
+private void cie_joint_caches_complete(gx_cie_joint_caches *,
+				       const gs_cie_common *,
+				       const gs_cie_abc *,
+				       const gs_cie_render *);
+private void cie_cache_restrict(cie_cache_floats *, const gs_range *);
+private void cie_mult3(const gs_vector3 *, const gs_matrix3 *,
+		       gs_vector3 *);
+private void cie_matrix_mult3(const gs_matrix3 *, const gs_matrix3 *,
+			      gs_matrix3 *);
+private void cie_invert3(const gs_matrix3 *, gs_matrix3 *);
+private void cie_matrix_init(gs_matrix3 *);
 
 /* Allocator structure types */
 private_st_joint_caches();
+extern_st(st_imager_state);
 
 #define RESTRICTED_INDEX(v, n, itemp)\
   ((uint)(itemp = (int)(v)) >= (n) ?\
@@ -66,19 +68,36 @@ private_st_joint_caches();
 	for (j = 0; j < countof(pcache); j++) {\
 	    cie_cache_floats *pcf = &(pcache)[j].floats;\
 	    int i;\
-	    gs_for_loop_params lp;\
+	    gs_sample_loop_params_t lp;\
 \
 	    gs_cie_cache_init(&pcf->params, &lp, &(domains)[j], cname);\
-	    for (i = 0; i < gx_cie_cache_size; lp.init += lp.step, i++) {\
-		pcf->values[i] = (*(rprocs)->procs[j])(lp.init, pcie);\
+	    for (i = 0; i <= lp.N; ++i) {\
+		float v = SAMPLE_LOOP_VALUE(i, lp);\
+		pcf->values[i] = (*(rprocs)->procs[j])(v, pcie);\
 		if_debug5('C', "[C]%s[%d,%d] = %g => %g\n",\
-			  cname, j, i, lp.init, pcf->values[i]);\
+			  cname, j, i, v, pcf->values[i]);\
 	    }\
 	    pcf->params.is_identity =\
 		(rprocs)->procs[j] == (dprocs).procs[j];\
 	}\
   END
 
+/* Define cache interpolation threshold values. */
+#ifdef CIE_CACHE_INTERPOLATE
+#  ifdef CIE_INTERPOLATE_THRESHOLD
+#    define CACHE_THRESHOLD CIE_INTERPOLATE_THRESHOLD
+#  else
+#    define CACHE_THRESHOLD 0	/* always interpolate */
+#  endif
+#else
+#  define CACHE_THRESHOLD 1.0e6	/* never interpolate */
+#endif
+#ifdef CIE_RENDER_TABLE_INTERPOLATE
+#  define RENDER_TABLE_THRESHOLD 0
+#else
+#  define RENDER_TABLE_THRESHOLD 1.0e6
+#endif
+
 /*
  * Determine whether a function is a linear transformation of the form
  * f(x) = scale * x + origin.
@@ -96,7 +115,8 @@ cache_is_linear(cie_linear_params_t *params, const cie_cache_floats *pcf)
 	if (fabs(pcf->values[i] - test) >= 0.5 / countof(pcf->values))
 	    return (params->is_linear = false);
     params->origin = origin - pcf->params.base;
-    params->scale = diff * pcf->params.factor / (countof(pcf->values) - 1);
+    params->scale =
+	diff * pcf->params.factor / (countof(pcf->values) - 1);
     return (params->is_linear = true);
 }
 
@@ -104,7 +124,7 @@ private void
 cache_set_linear(cie_cache_floats *pcf)
 {
 	if (pcf->params.is_identity) {
-	    if_debug1('c', "[c]linear(0x%lx) = true (is_identity)\n",
+	    if_debug1('c', "[c]is_linear(0x%lx) = true (is_identity)\n",
 		      (ulong)pcf);
 	    pcf->params.linear.is_linear = true;
 	    pcf->params.linear.origin = 0;
@@ -114,7 +134,7 @@ cache_set_linear(cie_cache_floats *pcf)
 		fabs(pcf->params.linear.scale - 1) < 0.00001)
 		pcf->params.is_identity = true;
 	    if_debug4('c',
-		      "[c]linear(0x%lx) = true, origin = %g, scale = %g%s\n",
+		      "[c]is_linear(0x%lx) = true, origin = %g, scale = %g%s\n",
 		      (ulong)pcf, pcf->params.linear.origin,
 		      pcf->params.linear.scale,
 		      (pcf->params.is_identity ? " (=> is_identity)" : ""));
@@ -125,11 +145,11 @@ cache_set_linear(cie_cache_floats *pcf)
 #endif
 }
 private void
-cache3_set_linear(gx_cie_vector_cache *caches /*[3]*/)
+cache3_set_linear(gx_cie_vector_cache3_t *pvc)
 {
-    cache_set_linear(&caches[0].floats);
-    cache_set_linear(&caches[1].floats);
-    cache_set_linear(&caches[2].floats);
+    cache_set_linear(&pvc->caches[0].floats);
+    cache_set_linear(&pvc->caches[1].floats);
+    cache_set_linear(&pvc->caches[2].floats);
 }
 
 #ifdef DEBUG
@@ -174,17 +194,17 @@ abc_identity(floatp in, const gs_cie_abc * pcie)
 private float
 abc_from_cache_0(floatp in, const gs_cie_abc * pcie)
 {
-    return gs_cie_cached_value(in, &pcie->caches.DecodeABC[0].floats);
+    return gs_cie_cached_value(in, &pcie->caches.DecodeABC.caches[0].floats);
 }
 private float
 abc_from_cache_1(floatp in, const gs_cie_abc * pcie)
 {
-    return gs_cie_cached_value(in, &pcie->caches.DecodeABC[1].floats);
+    return gs_cie_cached_value(in, &pcie->caches.DecodeABC.caches[1].floats);
 }
 private float
 abc_from_cache_2(floatp in, const gs_cie_abc * pcie)
 {
-    return gs_cie_cached_value(in, &pcie->caches.DecodeABC[2].floats);
+    return gs_cie_cached_value(in, &pcie->caches.DecodeABC.caches[2].floats);
 }
 
 private float
@@ -376,17 +396,17 @@ gx_restrict_CIEA(gs_client_color * pcc, const gs_color_space * pcs)
 
 /* ------ Install a CIE color space ------ */
 
-private void cie_cache_mult(P3(gx_cie_vector_cache *, const gs_vector3 *,
-			       const cie_cache_floats *));
-private bool cie_cache_mult3(P2(gx_cie_vector_cache *,
-				const gs_matrix3 *));
+private void cie_cache_mult(gx_cie_vector_cache *, const gs_vector3 *,
+			    const cie_cache_floats *, floatp);
+private bool cie_cache_mult3(gx_cie_vector_cache3_t *,
+			     const gs_matrix3 *, floatp);
 
 private int
 gx_install_cie_abc(gs_cie_abc *pcie, gs_state * pgs)
 {
     if_debug_matrix3("[c]CIE MatrixABC =", &pcie->MatrixABC);
     cie_matrix_init(&pcie->MatrixABC);
-    CIE_LOAD_CACHE_BODY(pcie->caches.DecodeABC, pcie->RangeABC.ranges,
+    CIE_LOAD_CACHE_BODY(pcie->caches.DecodeABC.caches, pcie->RangeABC.ranges,
 			&pcie->DecodeABC, DecodeABC_default, pcie,
 			"DecodeABC");
     gx_cie_load_common_cache(&pcie->common, pgs);
@@ -395,10 +415,26 @@ gx_install_cie_abc(gs_cie_abc *pcie, gs_state * pgs)
 }
 
 int
-gx_install_CIEDEFG(const gs_color_space * pcs, gs_state * pgs)
+gx_install_CIEDEFG(gs_color_space * pcs, gs_state * pgs)
 {
     gs_cie_defg *pcie = pcs->params.defg;
 
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        /*
+         * Check if we want to use the callback color processing for this
+         * color space.
+         */
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) pgs->custom_color_callback;
+
+        if (pcb != NULL) {
+	    if (pcb->client_procs->install_CIEBasedDEFG(pcb, pcs, pgs))
+    	        /* Exit if the client will handle the colorspace completely */
+		return 0;
+        }
+    }
+#endif
     CIE_LOAD_CACHE_BODY(pcie->caches_defg.DecodeDEFG, pcie->RangeDEFG.ranges,
 			&pcie->DecodeDEFG, DecodeDEFG_default, pcie,
 			"DecodeDEFG");
@@ -406,10 +442,26 @@ gx_install_CIEDEFG(const gs_color_space * pcs, gs_state * pgs)
 }
 
 int
-gx_install_CIEDEF(const gs_color_space * pcs, gs_state * pgs)
+gx_install_CIEDEF(gs_color_space * pcs, gs_state * pgs)
 {
     gs_cie_def *pcie = pcs->params.def;
 
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        /*
+         * Check if we want to use the callback color processing for this
+         * color space.
+         */
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) pgs->custom_color_callback;
+
+        if (pcb != NULL) {
+	    if (pcb->client_procs->install_CIEBasedDEF(pcb, pcs, pgs))
+    	        /* Exit if the client will handle the colorspace completely */
+		return 0;
+        }
+    }
+#endif
     CIE_LOAD_CACHE_BODY(pcie->caches_def.DecodeDEF, pcie->RangeDEF.ranges,
 			&pcie->DecodeDEF, DecodeDEF_default, pcie,
 			"DecodeDEF");
@@ -417,22 +469,55 @@ gx_install_CIEDEF(const gs_color_space * pcs, gs_state * pgs)
 }
 
 int
-gx_install_CIEABC(const gs_color_space * pcs, gs_state * pgs)
+gx_install_CIEABC(gs_color_space * pcs, gs_state * pgs)
 {
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        /*
+         * Check if we want to use the callback color processing for this
+         * color space.
+         */
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) pgs->custom_color_callback;
+
+        if (pcb != NULL) {
+	    if (pcb->client_procs->install_CIEBasedABC(pcb, pcs, pgs))
+    	        /* Exit if the client will handle the colorspace completely */
+		return 0;
+        }
+    }
+#endif
     return gx_install_cie_abc(pcs->params.abc, pgs);
 }
 
 int
-gx_install_CIEA(const gs_color_space * pcs, gs_state * pgs)
+gx_install_CIEA(gs_color_space * pcs, gs_state * pgs)
 {
     gs_cie_a *pcie = pcs->params.a;
+    gs_sample_loop_params_t lp;
     int i;
-    gs_for_loop_params lp;
-    float in;
 
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        /*
+         * Check if we want to use the callback color processing for this
+         * color space.
+         */
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) pgs->custom_color_callback;
+
+        if (pcb != NULL) {
+	    if (pcb->client_procs->install_CIEBasedA(pcb, pcs, pgs))
+    	        /* Exit if the client will handle the colorspace completely */
+		return 0;
+        }
+    }
+#endif
     gs_cie_cache_init(&pcie->caches.DecodeA.floats.params, &lp,
 		      &pcie->RangeA, "DecodeA");
-    for (i = 0, in = lp.init; i < gx_cie_cache_size; in += lp.step, i++) {
+    for (i = 0; i <= lp.N; ++i) {
+	float in = SAMPLE_LOOP_VALUE(i, lp);
+
 	pcie->caches.DecodeA.floats.values[i] = (*pcie->DecodeA)(in, pcie);
 	if_debug3('C', "[C]DecodeA[%d] = %g => %g\n",
 		  i, in, pcie->caches.DecodeA.floats.values[i]);
@@ -516,9 +601,10 @@ gs_cie_def_complete(gs_cie_def * pcie)
 void
 gs_cie_abc_complete(gs_cie_abc * pcie)
 {
-    cache3_set_linear(pcie->caches.DecodeABC);
+    cache3_set_linear(&pcie->caches.DecodeABC);
     pcie->caches.skipABC =
-	cie_cache_mult3(pcie->caches.DecodeABC, &pcie->MatrixABC);
+	cie_cache_mult3(&pcie->caches.DecodeABC, &pcie->MatrixABC,
+			CACHE_THRESHOLD);
     gx_cie_common_complete((gs_cie_common *)pcie);
 }
 
@@ -528,18 +614,79 @@ void
 gs_cie_a_complete(gs_cie_a * pcie)
 {
     cie_cache_mult(&pcie->caches.DecodeA, &pcie->MatrixA,
-		   &pcie->caches.DecodeA.floats);
+		   &pcie->caches.DecodeA.floats,
+		   CACHE_THRESHOLD);
     cache_set_linear(&pcie->caches.DecodeA.floats);
     gx_cie_common_complete((gs_cie_common *)pcie);
 }
 
-/* Convert a scalar cache to a vector cache by multiplying */
-/* the scalar values by a vector. */
-/* This procedure is idempotent. */
+/*
+ * Set the ranges where interpolation is required in a vector cache.
+ * This procedure is idempotent.
+ */
+typedef struct cie_cache_range_temp_s {
+    cie_cached_value prev;
+    int imin, imax;
+} cie_cache_range_temp_t;
+private void
+check_interpolation_required(cie_cache_range_temp_t *pccr,
+			     cie_cached_value cur, int i, floatp threshold)
+{
+    cie_cached_value prev = pccr->prev;
+
+    if (any_abs(cur - prev) > threshold * min(any_abs(prev), any_abs(cur))) {
+	if (i - 1 < pccr->imin)
+	    pccr->imin = i - 1;
+	if (i > pccr->imax)
+	    pccr->imax = i;
+    }
+    pccr->prev = cur;
+}
+private void
+cie_cache_set_interpolation(gx_cie_vector_cache *pcache, floatp threshold)
+{
+    cie_cached_value base = pcache->vecs.params.base;
+    cie_cached_value factor = pcache->vecs.params.factor;
+    cie_cache_range_temp_t temp[3];
+    int i, j;
+
+    for (j = 0; j < 3; ++j)
+	temp[j].imin = gx_cie_cache_size, temp[j].imax = -1;
+    temp[0].prev = pcache->vecs.values[0].u;
+    temp[1].prev = pcache->vecs.values[0].v;
+    temp[2].prev = pcache->vecs.values[0].w;
+
+    for (i = 0; i < gx_cie_cache_size; ++i) {
+	check_interpolation_required(&temp[0], pcache->vecs.values[i].u, i,
+				     threshold);
+	check_interpolation_required(&temp[1], pcache->vecs.values[i].v, i,
+				     threshold);
+	check_interpolation_required(&temp[2], pcache->vecs.values[i].w, i,
+				     threshold);
+    }
+
+    for (j = 0; j < 3; ++j) {
+	pcache->vecs.params.interpolation_ranges[j].rmin =
+	    base + (cie_cached_value)((double)temp[j].imin / factor);
+	pcache->vecs.params.interpolation_ranges[j].rmax =
+	    base + (cie_cached_value)((double)temp[j].imax / factor);
+	if_debug3('c', "[c]interpolation_ranges[%d] = %g, %g\n", j,
+		  cie_cached2float(pcache->vecs.params.interpolation_ranges[j].rmin),
+		  cie_cached2float(pcache->vecs.params.interpolation_ranges[j].rmax));
+    }
+
+}
+
+/*
+ * Convert a scalar cache to a vector cache by multiplying the scalar
+ * values by a vector.  Also set the range where interpolation is needed.
+ * This procedure is idempotent.
+ */
 private void
 cie_cache_mult(gx_cie_vector_cache * pcache, const gs_vector3 * pvec,
-	       const cie_cache_floats * pcf)
+	       const cie_cache_floats * pcf, floatp threshold)
 {
+    float u = pvec->u, v = pvec->v, w = pvec->w;
     int i;
 
     pcache->vecs.params.base = float2cie_cached(pcf->params.base);
@@ -550,22 +697,56 @@ cie_cache_mult(gx_cie_vector_cache * pcache, const gs_vector3 * pvec,
     for (i = 0; i < gx_cie_cache_size; ++i) {
 	float f = pcf->values[i];
 
-	pcache->vecs.values[i].u = float2cie_cached(f * pvec->u);
-	pcache->vecs.values[i].v = float2cie_cached(f * pvec->v);
-	pcache->vecs.values[i].w = float2cie_cached(f * pvec->w);
+	pcache->vecs.values[i].u = float2cie_cached(f * u);
+	pcache->vecs.values[i].v = float2cie_cached(f * v);
+	pcache->vecs.values[i].w = float2cie_cached(f * w);
+    }
+    cie_cache_set_interpolation(pcache, threshold);
+}
+
+/*
+ * Set the interpolation ranges in a 3-vector cache, based on the ranges in
+ * the individual vector caches.  This procedure is idempotent.
+ */
+private void
+cie_cache3_set_interpolation(gx_cie_vector_cache3_t * pvc)
+{
+    int j, k;
+
+    /* Iterate over output components. */
+    for (j = 0; j < 3; ++j) {
+	/* Iterate over sub-caches. */
+	cie_interpolation_range_t *p = 
+		&pvc->caches[0].vecs.params.interpolation_ranges[j];
+        cie_cached_value rmin = p->rmin, rmax = p->rmax;
+
+	for (k = 1; k < 3; ++k) {
+	    p = &pvc->caches[k].vecs.params.interpolation_ranges[j];
+	    rmin = min(rmin, p->rmin), rmax = max(rmax, p->rmax);
+	}
+	pvc->interpolation_ranges[j].rmin = rmin;
+	pvc->interpolation_ranges[j].rmax = rmax;
+	if_debug3('c', "[c]Merged interpolation_ranges[%d] = %g, %g\n",
+		  j, rmin, rmax);
     }
 }
 
-/* Convert 3 scalar caches to vector caches by multiplying by a matrix. */
-/* Return true iff the resulting cache is an identity transformation. */
+/*
+ * Convert 3 scalar caches to vector caches by multiplying by a matrix.
+ * Return true iff the resulting cache is an identity transformation.
+ * This procedure is idempotent.
+ */
 private bool
-cie_cache_mult3(gx_cie_vector_cache * pc /*[3] */ , const gs_matrix3 * pmat)
+cie_cache_mult3(gx_cie_vector_cache3_t * pvc, const gs_matrix3 * pmat,
+		floatp threshold)
 {
-    cie_cache_mult(pc, &pmat->cu, &pc->floats);
-    cie_cache_mult(pc + 1, &pmat->cv, &pc[1].floats);
-    cie_cache_mult(pc + 2, &pmat->cw, &pc[2].floats);
-    return pmat->is_identity & pc[0].floats.params.is_identity &
-	pc[1].floats.params.is_identity & pc[2].floats.params.is_identity;
+    cie_cache_mult(&pvc->caches[0], &pmat->cu, &pvc->caches[0].floats, threshold);
+    cie_cache_mult(&pvc->caches[1], &pmat->cv, &pvc->caches[1].floats, threshold);
+    cie_cache_mult(&pvc->caches[2], &pmat->cw, &pvc->caches[2].floats, threshold);
+    cie_cache3_set_interpolation(pvc);
+    return pmat->is_identity & pvc->caches[0].floats.params.is_identity &
+	pvc->caches[1].floats.params.is_identity &
+	pvc->caches[2].floats.params.is_identity;
 }
 
 /* ------ Install a rendering dictionary ------ */
@@ -606,13 +787,13 @@ gs_currentcolorrendering(const gs_state * pgs)
 
 /* Unshare (allocating if necessary) the joint caches. */
 gx_cie_joint_caches *
-gx_currentciecaches(gs_state * pgs)
+gx_unshare_cie_caches(gs_state * pgs)
 {
     gx_cie_joint_caches *pjc = pgs->cie_joint_caches;
 
     rc_unshare_struct(pgs->cie_joint_caches, gx_cie_joint_caches,
 		      &st_joint_caches, pgs->memory,
-		      return 0, "gx_currentciecaches");
+		      return 0, "gx_unshare_cie_caches");
     if (pgs->cie_joint_caches != pjc) {
 	pjc = pgs->cie_joint_caches;
 	pjc->cspace_id = pjc->render_id = gs_no_id;
@@ -621,65 +802,107 @@ gx_currentciecaches(gs_state * pgs)
     return pjc;
 }
 
+gx_cie_joint_caches *
+gx_currentciecaches(gs_state * pgs)
+{
+    return pgs->cie_joint_caches;
+}
+
 /* Compute the parameters for loading a cache, setting base and factor. */
 /* This procedure is idempotent. */
 void
-gs_cie_cache_init(cie_cache_params * pcache, gs_for_loop_params * pflp,
+gs_cie_cache_init(cie_cache_params * pcache, gs_sample_loop_params_t * pslp,
 		  const gs_range * domain, client_name_t cname)
 {
-	/* We need to map the values in the range
-	 * [domain->rmin..domain->rmax].  However, if rmin < 0 < rmax and
-	 * the function is non-linear, this can lead to anomalies at zero,
-	 * which is the default value for CIE colors.  The "correct" way to
-	 * approach this is to run the mapping functions on demand, but we
-	 * don't want to deal with the complexities of the callbacks this
-	 * would involve (especially in the middle of rendering images);
-	 * instead, we adjust the range so that zero maps precisely to a
-	 * cache slot.  Define:
-	 *      a = domain->rmin;
-	 *      b = domain->rmax;
-	 *      R = b - a;
-	 *      N = gx_cie_cache_size - 1;
-	 *      f(v) = N(v-a)/R;
-	 *      x = f(0).
-	 * If x is not an integer, we can either increase b or
-	 * decrease a to make it one.  In the former case, compute:
-	 *      Kb = floor(x); R'b = N(0-a)/Kb; b' = a + R'b.
-	 * In the latter case, compute:
-	 *      Ka = ceiling(x-N); R'a = N(0-b)/Ka; a' = b - R'a.
-	 * We choose whichever method stretches the range the least,
-	 * i.e., the one whose R' value (R'a or R'b) is smaller.
-	 */
-    double a = domain->rmin, b = domain->rmax;
-    double R = b - a;
-    double delta;
-#define N (gx_cie_cache_size - 1)
+    /*
+      We need to map the values in the range [domain->rmin..domain->rmax].
+      However, if rmin < 0 < rmax and the function is non-linear, this can
+      lead to anomalies at zero, which is the default value for CIE colors.
+      The "correct" way to approach this is to run the mapping functions on
+      demand, but we don't want to deal with the complexities of the
+      callbacks this would involve (especially in the middle of rendering
+      images); instead, we adjust the range so that zero maps precisely to a
+      cache slot.  Define:
+
+      A = domain->rmin;
+      B = domain->rmax;
+      N = gx_cie_cache_size - 1;
+
+      R = B - A;
+      h(v) = N * (v - A) / R;		// the index of v in the cache
+      X = h(0).
+
+      If X is not an integer, we can decrease A and/increase B to make it
+      one.  Let A' and B' be the adjusted values of A and B respectively,
+      and let K be the integer derived from X (either floor(X) or ceil(X)).
+      Define
+
+      f(K) = (K * B' + (N - K) * A') / N).
+
+      We want f(K) = 0.  This occurs precisely when, for any real number
+      C != 0,
+
+      A' = -K * C;
+      B' = (N - K) * C.
+
+      In order to ensure A' <= A and B' >= B, we require
+
+      C >= -A / K;
+      C >= B / (N - K).
+
+      Since A' and B' must be exactly representable as floats, we round C
+      upward to ensure that it has no more than M mantissa bits, where
+
+      M = ARCH_FLOAT_MANTISSA_BITS - ceil(log2(N)).
+    */
+    float A = domain->rmin, B = domain->rmax;
+    double R = B - A, delta;
+#define NN (gx_cie_cache_size - 1) /* 'N' is a member name, see end of proc */
+#define N NN
+#define CEIL_LOG2_N CIE_LOG2_CACHE_SIZE
 
     /* Adjust the range if necessary. */
-    if (a < 0 && b >= 0) {
-	double x = -N * a / R;	/* must be > 0 */
-	double Kb = floor(x);	/* must be >= 0 */
-	double Ka = ceil(x) - N;	/* must be <= 0 */
+    if (A < 0 && B >= 0) {
+	const double X = -N * A / R;	/* know X > 0 */
+	/* Choose K to minimize range expansion. */
+	const int K = (int)(A + B < 0 ? floor(X) : ceil(X)); /* know 0 < K < N */
+	const double Ca = -A / K, Cb = B / (N - K); /* know Ca, Cb > 0 */
+	double C = max(Ca, Cb);	/* know C > 0 */
+	const int M = ARCH_FLOAT_MANTISSA_BITS - CEIL_LOG2_N;
+	int cexp;
+	const double cfrac = frexp(C, &cexp);
 
-	if (Kb == 0 || (Ka != 0 && -b / Ka < -a / Kb))	/* use R'a */
-	    R = -N * b / Ka, a = b - R;
-	else			/* use R'b */
-	    R = -N * a / Kb, b = a + R;
+	if_debug4('c', "[c]adjusting cache_init(%8g, %8g), X = %8g, K = %d:\n",
+		  A, B, X, K);
+	/* Round C to no more than M significant bits.  See above. */
+	C = ldexp(ceil(ldexp(cfrac, M)), cexp - M);
+	/* Finally, compute A' and B'. */
+	A = -K * C;
+	B = (N - K) * C;
+	if_debug2('c', "[c]  => %8g, %8g\n", A, B);
+	R = B - A;
     }
     delta = R / N;
 #ifdef CIE_CACHE_INTERPOLATE
-    pcache->base = a;		/* no rounding */
+    pcache->base = A;		/* no rounding */
 #else
-    pcache->base = a - delta / 2;	/* so lookup will round */
+    pcache->base = A - delta / 2;	/* so lookup will round */
 #endif
-    pcache->factor = (delta == 0 ? 0 : N / R);
+    /*
+     * If size of the domain is zero, then use 1.0 as the scaling
+     * factor.  This prevents divide by zero errors in later calculations.
+     * This should only occurs with zero matrices.  It does occur with
+     * Genoa test file 050-01.ps.
+     */
+    pcache->factor = (any_abs(delta) < 1e-30 ? 1.0 : N / R);
     if_debug4('c', "[c]cache %s 0x%lx base=%g, factor=%g\n",
 	      (const char *)cname, (ulong) pcache,
 	      pcache->base, pcache->factor);
-    pflp->init = a;
-    pflp->step = delta;
-    pflp->limit = b + delta / 2;
+    pslp->A = A;
+    pslp->B = B;
 #undef N
+    pslp->N = NN;
+#undef NN
 }
 
 /* ------ Complete a rendering structure ------ */
@@ -688,8 +911,8 @@ gs_cie_cache_init(cie_cache_params * pcache, gs_for_loop_params * pflp,
  * Compute the derived values in a CRD that don't involve the cached
  * procedure values.  This procedure is idempotent.
  */
-private void cie_transform_range3(P3(const gs_range3 *, const gs_matrix3 *,
-				     gs_range3 *));
+private void cie_transform_range3(const gs_range3 *, const gs_matrix3 *,
+				  gs_range3 *);
 int
 gs_cie_render_init(gs_cie_render * pcrd)
 {
@@ -730,19 +953,19 @@ gs_cie_render_sample(gs_cie_render * pcrd)
     code = gs_cie_render_init(pcrd);
     if (code < 0)
 	return code;
-    CIE_LOAD_CACHE_BODY(pcrd->caches.EncodeLMN, pcrd->DomainLMN.ranges,
+    CIE_LOAD_CACHE_BODY(pcrd->caches.EncodeLMN.caches, pcrd->DomainLMN.ranges,
 			&pcrd->EncodeLMN, Encode_default, pcrd, "EncodeLMN");
-    cache3_set_linear(pcrd->caches.EncodeLMN);
+    cache3_set_linear(&pcrd->caches.EncodeLMN);
     CIE_LOAD_CACHE_BODY(pcrd->caches.EncodeABC, pcrd->DomainABC.ranges,
 			&pcrd->EncodeABC, Encode_default, pcrd, "EncodeABC");
     if (pcrd->RenderTable.lookup.table != 0) {
 	int i, j, m = pcrd->RenderTable.lookup.m;
-	gs_for_loop_params flp;
+	gs_sample_loop_params_t lp;
 	bool is_identity = true;
 
 	for (j = 0; j < m; j++) {
 	    gs_cie_cache_init(&pcrd->caches.RenderTableT[j].fracs.params,
-			      &flp, &Range3_default.ranges[0],
+			      &lp, &Range3_default.ranges[0],
 			      "RenderTableT");
 	    is_identity &= pcrd->RenderTable.T.procs[j] ==
 		RenderTableT_default.procs[j];
@@ -839,7 +1062,7 @@ gs_cie_render_complete(gs_cie_render * pcrd)
 	for (c = 0; c < 3; c++) {
 	    gx_cie_float_fixed_cache *pcache = &pcrd->caches.EncodeABC[c];
 
-	    cie_cache_restrict(&pcrd->caches.EncodeLMN[c].floats,
+	    cie_cache_restrict(&pcrd->caches.EncodeLMN.caches[c].floats,
 			       &pcrd->RangeLMN.ranges[c]);
 	    cie_cache_restrict(&pcrd->caches.EncodeABC[c].floats,
 			       &pcrd->RangeABC.ranges[c]);
@@ -903,7 +1126,8 @@ gs_cie_render_complete(gs_cie_render * pcrd)
 #undef MABC
 	pcrd->MatrixABCEncode.is_identity = 0;
     }
-    cie_cache_mult3(pcrd->caches.EncodeLMN, &pcrd->MatrixABCEncode);
+    cie_cache_mult3(&pcrd->caches.EncodeLMN, &pcrd->MatrixABCEncode,
+		    CACHE_THRESHOLD);
     pcrd->status = CIE_RENDER_STATUS_COMPLETED;
     return 0;
 }
@@ -987,7 +1211,7 @@ gs_cie_cs_common(const gs_state * pgs)
 int
 gs_cie_cs_complete(gs_state * pgs, bool init)
 {
-    gx_cie_joint_caches *pjc = gx_currentciecaches(pgs);
+    gx_cie_joint_caches *pjc = gx_unshare_cie_caches(pgs);
 
     if (pjc == 0)
 	return_error(gs_error_VMerror);
@@ -1070,23 +1294,23 @@ cie_joint_caches_init(gx_cie_joint_caches * pjc,
     is_identity = pcrd->TransformPQR.proc == TransformPQR_default.proc;
     for (j = 0; j < 3; j++) {
 	int i;
-	gs_for_loop_params lp;
+	gs_sample_loop_params_t lp;
 
-	gs_cie_cache_init(&pjc->TransformPQR[j].floats.params, &lp,
+	gs_cie_cache_init(&pjc->TransformPQR.caches[j].floats.params, &lp,
 			  &pcrd->RangePQR.ranges[j], "TransformPQR");
-	for (i = 0; i < gx_cie_cache_size; lp.init += lp.step, i++) {
+	for (i = 0; i <= lp.N; ++i) {
+	    float in = SAMPLE_LOOP_VALUE(i, lp);
 	    float out;
-	    int code =
-		(*pcrd->TransformPQR.proc)(j, lp.init, &pjc->points_sd,
-					   pcrd, &out);
+	    int code = (*pcrd->TransformPQR.proc)(j, in, &pjc->points_sd,
+						  pcrd, &out);
 
 	    if (code < 0)
 		return code;
-	    pjc->TransformPQR[j].floats.values[i] = out;
+	    pjc->TransformPQR.caches[j].floats.values[i] = out;
 	    if_debug4('C', "[C]TransformPQR[%d,%d] = %g => %g\n",
-		      j, i, lp.init, out);
+		      j, i, in, out);
 	}
-	pjc->TransformPQR[j].floats.params.is_identity = is_identity;
+	pjc->TransformPQR.caches[j].floats.params.is_identity = is_identity;
     }
     return 0;
 }
@@ -1101,10 +1325,12 @@ cie_joint_caches_complete(gx_cie_joint_caches * pjc,
 			  const gs_cie_abc * pabc /* NULL if CIEA */,
 			  const gs_cie_render * pcrd)
 {
-    gs_matrix3 mat3, mat2, mat1;
+    gs_matrix3 mat3, mat2;
     gs_matrix3 MatrixLMN_PQR;
     int j;
 
+    pjc->remap_finish = gx_cie_real_remap_finish;
+
     /*
      * We number the pipeline steps as follows:
      *   1 - DecodeABC/MatrixABC
@@ -1117,70 +1343,172 @@ cie_joint_caches_complete(gx_cie_joint_caches * pjc,
      * transform, but we could do it whenever the procedure is linear.
      * A project for another day....
      */
-    /* Step 4 */
-    if (pcrd->caches.EncodeLMN[0].floats.params.is_identity &&
-	pcrd->caches.EncodeLMN[1].floats.params.is_identity &&
-	pcrd->caches.EncodeLMN[2].floats.params.is_identity) {
+
+	/* Step 4 */
+
+#ifdef OPTIMIZE_CIE_MAPPING
+    if (pcrd->caches.EncodeLMN.caches[0].floats.params.is_identity &&
+	pcrd->caches.EncodeLMN.caches[1].floats.params.is_identity &&
+	pcrd->caches.EncodeLMN.caches[2].floats.params.is_identity
+	) {
 	/* Fold step 4 into step 3. */
+	if_debug0('c', "[c]EncodeLMN is identity, folding MatrixABC(Encode) into MatrixPQR'+LMN.\n");
 	cie_matrix_mult3(&pcrd->MatrixABCEncode, &pcrd->MatrixPQR_inverse_LMN,
 			 &mat3);
 	pjc->skipEncodeLMN = true;
-    } else {
+    } else
+#endif /* OPTIMIZE_CIE_MAPPING */
+    {
+	if_debug0('c', "[c]EncodeLMN is not identity.\n");
 	mat3 = pcrd->MatrixPQR_inverse_LMN;
 	pjc->skipEncodeLMN = false;
     }
-    /* Step 3 */
-    cache3_set_linear(pjc->TransformPQR);
+
+	/* Step 3 */
+
+    cache3_set_linear(&pjc->TransformPQR);
     cie_matrix_mult3(&pcrd->MatrixPQR, &pcie->MatrixLMN,
 		     &MatrixLMN_PQR);
-    if (pjc->TransformPQR[0].floats.params.is_identity &
-	pjc->TransformPQR[1].floats.params.is_identity &
-	pjc->TransformPQR[2].floats.params.is_identity) {
+
+#ifdef OPTIMIZE_CIE_MAPPING
+    if (pjc->TransformPQR.caches[0].floats.params.is_identity &
+	pjc->TransformPQR.caches[1].floats.params.is_identity &
+	pjc->TransformPQR.caches[2].floats.params.is_identity
+	) {
 	/* Fold step 3 into step 2. */
+	if_debug0('c', "[c]TransformPQR is identity, folding MatrixPQR'+LMN into MatrixLMN+PQR.\n");
 	cie_matrix_mult3(&mat3, &MatrixLMN_PQR, &mat2);
 	pjc->skipPQR = true;
-    } else {
+    } else
+#endif /* OPTIMIZE_CIE_MAPPING */
+    {
+	if_debug0('c', "[c]TransformPQR is not identity.\n");
 	mat2 = MatrixLMN_PQR;
 	for (j = 0; j < 3; j++) {
-	    cie_cache_restrict(&pjc->TransformPQR[j].floats,
+	    cie_cache_restrict(&pjc->TransformPQR.caches[j].floats,
 			       &pcrd->RangePQR.ranges[j]);
 	}
-	cie_cache_mult3(pjc->TransformPQR, &mat3);
+	cie_cache_mult3(&pjc->TransformPQR, &mat3, CACHE_THRESHOLD);
 	pjc->skipPQR = false;
     }
-    /* Steps 2 & 1 */
+
+	/* Steps 2 & 1 */
+
+#ifdef OPTIMIZE_CIE_MAPPING
     if (pcie->caches.DecodeLMN[0].floats.params.is_identity &
 	pcie->caches.DecodeLMN[1].floats.params.is_identity &
-	pcie->caches.DecodeLMN[2].floats.params.is_identity) {
+	pcie->caches.DecodeLMN[2].floats.params.is_identity
+	) {
+	if_debug0('c', "[c]DecodeLMN is identity, folding MatrixLMN+PQR into MatrixABC.\n");
 	if (!pabc) {
 	    pjc->skipDecodeLMN = mat2.is_identity;
 	    pjc->skipDecodeABC = false;
-	    if (!pjc->skipDecodeLMN)
+	    if (!pjc->skipDecodeLMN) {
 		for (j = 0; j < 3; j++) {
-		    cie_cache_mult(&pjc->DecodeLMN[j], &mat2.cu + j,
-				   &pcie->caches.DecodeLMN[j].floats);
+		    cie_cache_mult(&pjc->DecodeLMN.caches[j], &mat2.cu + j,
+				   &pcie->caches.DecodeLMN[j].floats,
+				   CACHE_THRESHOLD);
 		}
+		cie_cache3_set_interpolation(&pjc->DecodeLMN);
+	    }
 	} else {
 	    /*
 	     * Fold step 2 into step 1.  This is a little different because
 	     * the data for step 1 are in the color space structure.
 	     */
+	    gs_matrix3 mat1;
+
 	    cie_matrix_mult3(&mat2, &pabc->MatrixABC, &mat1);
 	    for (j = 0; j < 3; j++) {
-		cie_cache_mult(&pjc->DecodeLMN[j], &mat1.cu + j,
-			       &pabc->caches.DecodeABC[j].floats);
+		cie_cache_mult(&pjc->DecodeLMN.caches[j], &mat1.cu + j,
+			       &pabc->caches.DecodeABC.caches[j].floats,
+			       CACHE_THRESHOLD);
 	    }
+	    cie_cache3_set_interpolation(&pjc->DecodeLMN);
 	    pjc->skipDecodeLMN = false;
 	    pjc->skipDecodeABC = true;
 	}
-    } else {
+    } else
+#endif /* OPTIMIZE_CIE_MAPPING */
+    {
+	if_debug0('c', "[c]DecodeLMN is not identity.\n");
 	for (j = 0; j < 3; j++) {
-	    cie_cache_mult(&pjc->DecodeLMN[j], &mat2.cu + j,
-			   &pcie->caches.DecodeLMN[j].floats);
+	    cie_cache_mult(&pjc->DecodeLMN.caches[j], &mat2.cu + j,
+			   &pcie->caches.DecodeLMN[j].floats,
+			   CACHE_THRESHOLD);
 	}
+	cie_cache3_set_interpolation(&pjc->DecodeLMN);
 	pjc->skipDecodeLMN = false;
 	pjc->skipDecodeABC = pabc != 0 && pabc->caches.skipABC;
     }
+
+}
+
+/*
+ * Initialize (just enough of) an imager state so that "concretizing" colors
+ * using this imager state will do only the CIE->XYZ mapping.  This is a
+ * semi-hack for the PDF writer.
+ */
+int
+gx_cie_to_xyz_alloc(gs_imager_state **ppis, const gs_color_space *pcs,
+		    gs_memory_t *mem)
+{
+    /*
+     * In addition to the imager state itself, we need the joint caches.
+     */
+    gs_imager_state *pis =
+	gs_alloc_struct(mem, gs_imager_state, &st_imager_state,
+			"gx_cie_to_xyz_alloc(imager state)");
+    gx_cie_joint_caches *pjc;
+    const gs_cie_abc *pabc;
+    const gs_cie_common *pcie = cie_cs_common_abc(pcs, &pabc);
+    int j;
+
+    if (pis == 0)
+	return_error(gs_error_VMerror);
+    memset(pis, 0, sizeof(*pis));	/* mostly paranoia */
+    pis->memory = mem;
+
+    pjc = gs_alloc_struct(mem, gx_cie_joint_caches, &st_joint_caches,
+			  "gx_cie_to_xyz_free(joint caches)");
+    if (pjc == 0) {
+	gs_free_object(mem, pis, "gx_cie_to_xyz_alloc(imager state)");
+	return_error(gs_error_VMerror);
+    }
+
+    /*
+     * Perform an abbreviated version of cie_joint_caches_complete.
+     * Don't bother with any optimizations.
+     */
+    for (j = 0; j < 3; j++) {
+	cie_cache_mult(&pjc->DecodeLMN.caches[j], &pcie->MatrixLMN.cu + j,
+		       &pcie->caches.DecodeLMN[j].floats,
+		       CACHE_THRESHOLD);
+    }
+    cie_cache3_set_interpolation(&pjc->DecodeLMN);
+    pjc->skipDecodeLMN = false;
+    pjc->skipDecodeABC = pabc != 0 && pabc->caches.skipABC;
+    /* Mark the joint caches as completed. */
+    pjc->remap_finish = gx_cie_xyz_remap_finish;
+    pjc->cspace_id = pcs->id;
+    pjc->status = CIE_JC_STATUS_COMPLETED;
+    pis->cie_joint_caches = pjc;
+    /*
+     * Set a non-zero CRD to pacify CIE_CHECK_RENDERING.  (It will never
+     * actually be referenced, aside from the zero test.)
+     */
+    pis->cie_render = (void *)~0;
+    *ppis = pis;
+    return 0;
+}
+void
+gx_cie_to_xyz_free(gs_imager_state *pis)
+{
+    gs_memory_t *mem = pis->memory;
+
+    gs_free_object(mem, pis->cie_joint_caches,
+		   "gx_cie_to_xyz_free(joint caches)");
+    gs_free_object(mem, pis, "gx_cie_to_xyz_free(imager state)");
 }
 
 /* ================ Utilities ================ */
@@ -1279,3 +1607,24 @@ cie_matrix_init(register gs_matrix3 * mat)
 	mat->cv.v == 1.0 && is_fzero2(mat->cv.u, mat->cv.w) &&
 	mat->cw.w == 1.0 && is_fzero2(mat->cw.u, mat->cw.v);
 }
+
+bool 
+gx_color_space_needs_cie_caches(const gs_color_space * pcs)
+{
+    switch (pcs->type->index) {
+    	case gs_color_space_index_CIEDEFG:
+    	case gs_color_space_index_CIEDEF:
+    	case gs_color_space_index_CIEABC:
+    	case gs_color_space_index_CIEA:
+    	case gs_color_space_index_CIEICC:
+	    return true;
+	case gs_color_space_index_DevicePixel:
+    	case gs_color_space_index_DeviceN:
+    	case gs_color_space_index_Separation:
+    	case gs_color_space_index_Indexed:
+    	case gs_color_space_index_Pattern:
+	    return gx_color_space_needs_cie_caches(pcs->base_space);
+	default:
+	    return false;
+    }
+}
