@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsciemap.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gsciemap.c 8022 2007-06-05 22:23:38Z giles $ */
 /* CIE color rendering */
 #include "math_.h"
 #include "gx.h"
@@ -90,6 +90,39 @@ cie_lookup_map3(cie_cached_vector3 * pvec,
 #  define cie_lookup_map3(pvec, pc, cname) cie_lookup_mult3(pvec, pc)
 #endif
 
+
+/*
+ * Test whether a CIE rendering has been defined; ensure that the joint
+ * caches are loaded.  Note that the procedure may return 1 if no rendering
+ * has been defined.
+ */
+private inline int 
+gx_cie_check_rendering_inline(const gs_color_space * pcs, frac * pconc, const gs_imager_state * pis)
+{
+    if (pis->cie_render == 0) {
+	/* No rendering has been defined yet: return black. */
+	pconc[0] = pconc[1] = pconc[2] = frac_0;
+	return 1;
+    }
+    if (pis->cie_joint_caches->status == CIE_JC_STATUS_COMPLETED) {
+	if (pis->cie_joint_caches->cspace_id != pcs->id)
+	    pis->cie_joint_caches->status = CIE_JC_STATUS_BUILT;
+    }
+    if (pis->cie_joint_caches->status != CIE_JC_STATUS_COMPLETED) {
+	int     code = gs_cie_jc_complete(pis, pcs);
+
+	if (code < 0)
+	    return code;
+    }
+    return 0;
+}
+int 
+gx_cie_check_rendering(const gs_color_space * pcs, frac * pconc, const gs_imager_state * pis)
+{
+    return gx_cie_check_rendering_inline(pcs, pconc, pis);
+}
+
+
 /* Render a CIEBasedDEFG color. */
 int
 gx_concretize_CIEDEFG(const gs_client_color * pc, const gs_color_space * pcs,
@@ -100,11 +133,16 @@ gx_concretize_CIEDEFG(const gs_client_color * pc, const gs_color_space * pcs,
     fixed hijk[4];
     frac abc[3];
     cie_cached_vector3 vec3;
+    int code;
 
     if_debug4('c', "[c]concretize DEFG [%g %g %g %g]\n",
 	      pc->paint.values[0], pc->paint.values[1],
 	      pc->paint.values[2], pc->paint.values[3]);
-    CIE_CHECK_RENDERING(pcs, pconc, pis, return 0);
+    code = gx_cie_check_rendering_inline(pcs, pconc, pis);
+    if (code < 0)
+	return code;
+    if (code == 1)
+	return 0;
 
     /*
      * Apply DecodeDEFG, including restriction to RangeHIJK and scaling to
@@ -116,10 +154,9 @@ gx_concretize_CIEDEFG(const gs_client_color * pc, const gs_color_space * pcs,
 	double v0 = pc->paint.values[i];
 	const gs_range *const rangeDEFG = &pcie->RangeDEFG.ranges[i];
 	double value =
-	    (v0 < rangeDEFG->rmin ? 0.0 :
-	     v0 > rangeDEFG->rmax ? factor :
-	     (v0 - rangeDEFG->rmin) * factor /
-	       (rangeDEFG->rmax - rangeDEFG->rmin));
+	    (v0 < rangeDEFG->rmin ? 0.0 : factor *
+	    (v0 > rangeDEFG->rmax ? rangeDEFG->rmax - rangeDEFG->rmin :
+	     v0 - rangeDEFG->rmin ));
 	int vi = (int)value;
 	double vf = value - vi;
 	double v = pcie->caches_defg.DecodeDEFG[i].floats.values[vi];
@@ -159,11 +196,16 @@ gx_concretize_CIEDEF(const gs_client_color * pc, const gs_color_space * pcs,
     fixed hij[3];
     frac abc[3];
     cie_cached_vector3 vec3;
+    int code;
 
     if_debug3('c', "[c]concretize DEF [%g %g %g]\n",
 	      pc->paint.values[0], pc->paint.values[1],
 	      pc->paint.values[2]);
-    CIE_CHECK_RENDERING(pcs, pconc, pis, return 0);
+    code = gx_cie_check_rendering_inline(pcs, pconc, pis);
+    if (code < 0)
+	return code;
+    if (code == 1)
+	return 0;
 
     /*
      * Apply DecodeDEF, including restriction to RangeHIJ and scaling to
@@ -175,10 +217,9 @@ gx_concretize_CIEDEF(const gs_client_color * pc, const gs_color_space * pcs,
 	double v0 = pc->paint.values[i];
 	const gs_range *const rangeDEF = &pcie->RangeDEF.ranges[i];
 	double value =
-	    (v0 < rangeDEF->rmin ? 0.0 :
-	     v0 > rangeDEF->rmax ? factor :
-	     (v0 - rangeDEF->rmin) * factor /
-	       (rangeDEF->rmax - rangeDEF->rmin));
+	    (v0 < rangeDEF->rmin ? 0.0 : factor *
+	    (v0 > rangeDEF->rmax ? rangeDEF->rmax - rangeDEF->rmin :
+	     v0 - rangeDEF->rmin ));
 	int vi = (int)value;
 	double vf = value - vi;
 	double v = pcie->caches_def.DecodeDEF[i].floats.values[vi];
@@ -204,6 +245,80 @@ gx_concretize_CIEDEF(const gs_client_color * pc, const gs_color_space * pcs,
 }
 #undef SCALE_TO_RANGE
 
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+/*
+ * This routine is only used if ENABLE_CUSTOM_COLOR_CALLBACK is true.
+ * Otherwise we use gx_default_remap_color directly for CIEBasedDEFG color
+ * spaces.
+ *
+ * Render a CIEBasedDEFG color.
+ */
+int
+gx_remap_CIEDEFG(const gs_client_color * pc, const gs_color_space * pcs,
+	gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
+		gs_color_select_t select)
+{
+    client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+    if (pcb != NULL) {
+	if (pcb->client_procs->remap_CIEBasedDEFG(pcb, pc, pcs,
+			   			pdc, pis, dev, select) == 0)
+	    return 0;
+    }
+    /* Use default routine for non custom color processing. */
+    return gx_default_remap_color(pc, pcs, pdc, pis, dev, select);
+}
+
+/*
+ * This routine is only used if ENABLE_CUSTOM_COLOR_CALLBACK is true.
+ * Otherwise we use gx_default_remap_color directly for CIEBasedDEF color
+ * spaces.
+ *
+ * Render a CIEBasedDEF color.
+ */
+int
+gx_remap_CIEDEF(const gs_client_color * pc, const gs_color_space * pcs,
+	gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
+		gs_color_select_t select)
+{
+    client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+    if (pcb != NULL) {
+	if (pcb->client_procs->remap_CIEBasedDEF(pcb, pc, pcs,
+			   			pdc, pis, dev, select) == 0)
+	    return 0;
+    }
+    /* Use default routine for non custom color processing. */
+    return gx_default_remap_color(pc, pcs, pdc, pis, dev, select);
+}
+
+/*
+ * This routine is only used if ENABLE_CUSTOM_COLOR_CALLBACK is true.
+ * Otherwise we use gx_default_remap_color directly for CIEBasedA color
+ * spaces.
+ *
+ * Render a CIEBasedA color.
+ */
+int
+gx_remap_CIEA(const gs_client_color * pc, const gs_color_space * pcs,
+	gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
+		gs_color_select_t select)
+{
+    client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+    if (pcb != NULL) {
+	if (pcb->client_procs->remap_CIEBasedA(pcb, pc, pcs,
+			   			pdc, pis, dev, select) == 0)
+	    return 0;
+    }
+    /* Use default routine for non custom color processing. */
+    return gx_default_remap_color(pc, pcs, pdc, pis, dev, select);
+}
+#endif
+
 /* Render a CIEBasedABC color. */
 /* We provide both remap and concretize, but only the former */
 /* needs to be efficient. */
@@ -214,11 +329,29 @@ gx_remap_CIEABC(const gs_client_color * pc, const gs_color_space * pcs,
 {
     frac conc[4];
     cie_cached_vector3 vec3;
+    int code;
 
     if_debug3('c', "[c]remap CIEABC [%g %g %g]\n",
 	      pc->paint.values[0], pc->paint.values[1],
 	      pc->paint.values[2]);
-    CIE_CHECK_RENDERING(pcs, conc, pis, goto map3);
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+        if (pcb != NULL) {
+	    if (pcb->client_procs->remap_CIEBasedABC(pcb, pc, pcs,
+			   			pdc, pis, dev, select) == 0)
+	        return 0;
+	}
+    }
+#endif
+
+    code = gx_cie_check_rendering_inline(pcs, conc, pis);
+    if (code < 0)
+	return code;
+    if (code == 1)
+	goto map3;
     vec3.u = float2cie_cached(pc->paint.values[0]);
     vec3.v = float2cie_cached(pc->paint.values[1]);
     vec3.w = float2cie_cached(pc->paint.values[2]);
@@ -263,11 +396,16 @@ gx_concretize_CIEABC(const gs_client_color * pc, const gs_color_space * pcs,
 {
     const gs_cie_abc *pcie = pcs->params.abc;
     cie_cached_vector3 vec3;
+    int code;
 
     if_debug3('c', "[c]concretize CIEABC [%g %g %g]\n",
 	      pc->paint.values[0], pc->paint.values[1],
 	      pc->paint.values[2]);
-    CIE_CHECK_RENDERING(pcs, pconc, pis, return 0);
+    code = gx_cie_check_rendering_inline(pcs, pconc, pis);
+    if (code < 0)
+	return code;
+    if (code == 1)
+	return 0;
 
     vec3.u = float2cie_cached(pc->paint.values[0]);
     vec3.v = float2cie_cached(pc->paint.values[1]);
@@ -287,9 +425,14 @@ gx_concretize_CIEA(const gs_client_color * pc, const gs_color_space * pcs,
     const gs_cie_a *pcie = pcs->params.a;
     cie_cached_value a = float2cie_cached(pc->paint.values[0]);
     cie_cached_vector3 vlmn;
+    int code;
 
     if_debug1('c', "[c]concretize CIEA %g\n", pc->paint.values[0]);
-    CIE_CHECK_RENDERING(pcs, pconc, pis, return 0);
+    code = gx_cie_check_rendering_inline(pcs, pconc, pis);
+    if (code < 0)
+	return code;
+    if (code == 1)
+	return 0;
 
     /* Apply DecodeA and MatrixA. */
     if (!pis->cie_joint_caches->skipDecodeABC)
