@@ -1,22 +1,17 @@
-/* Copyright (C) 1989, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gscoord.c,v 1.5.6.1.2.1 2003/01/17 00:49:02 giles Exp $ */
+/* $Id: gscoord.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Coordinate system operators for Ghostscript library */
 #include "math_.h"
 #include "gx.h"
@@ -32,13 +27,13 @@
 #include "gxdevice.h"
 
 /* Choose whether to enable the rounding code in update_ctm. */
-#define ROUND_CTM_FIXED 1
+#define ROUND_CTM_FIXED 0
 
 /* Forward declarations */
 #ifdef DEBUG
 #define trace_ctm(pgs) trace_matrix_fixed(&(pgs)->ctm)
-private void trace_matrix_fixed(P1(const gs_matrix_fixed *));
-private void trace_matrix(P1(const gs_matrix *));
+private void trace_matrix_fixed(const gs_matrix_fixed *);
+private void trace_matrix(const gs_matrix *);
 
 #endif
 
@@ -234,8 +229,8 @@ gs_translate(gs_state * pgs, floatp dx, floatp dy)
 
     if ((code = gs_distance_transform(dx, dy, &ctm_only(pgs), &pt)) < 0)
 	return code;
-    pt.x += pgs->ctm.tx;
-    pt.y += pgs->ctm.ty;
+    pt.x = (float)pt.x + pgs->ctm.tx;
+    pt.y = (float)pt.y + pgs->ctm.ty;
     update_ctm(pgs, pt.x, pt.y);
 #ifdef DEBUG
     if (gs_debug_c('x'))
@@ -386,6 +381,8 @@ gx_translate_to_fixed(register gs_state * pgs, fixed px, fixed py)
 	trace_matrix_fixed(&pgs->char_tm);
     }
 #endif
+    gx_setcurrentpoint(pgs, fixed2float(pgs->ctm.tx_fixed), fixed2float(pgs->ctm.ty_fixed));
+    pgs->current_point_valid = true;
     return 0;
 }
 
