@@ -1,22 +1,17 @@
-/* Copyright (C) 1997, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gscpixel.c,v 1.3.6.1.2.1 2003/01/17 00:49:02 giles Exp $ */
+/* $Id: gscpixel.c 8022 2007-06-05 22:23:38Z giles $ */
 /* DevicePixel color space and operation definition */
 #include "gx.h"
 #include "gserrors.h"
@@ -24,34 +19,37 @@
 #include "gxcspace.h"
 #include "gscpixel.h"
 #include "gxdevice.h"
+#include "gxistate.h"
+#include "gsovrc.h"
+#include "gsstate.h"
+#include "gzstate.h"
+#include "stream.h"
 
 /* Define the DevicePixel color space type. */
-private cs_proc_equal(gx_equal_DevicePixel);
 private cs_proc_restrict_color(gx_restrict_DevicePixel);
 private cs_proc_remap_concrete_color(gx_remap_concrete_DevicePixel);
 private cs_proc_concretize_color(gx_concretize_DevicePixel);
+private cs_proc_set_overprint(gx_set_overprint_DevicePixel);
+private cs_proc_serialize(gx_serialize_DevicePixel);
 private const gs_color_space_type gs_color_space_type_DevicePixel = {
     gs_color_space_index_DevicePixel, true, false,
     &st_base_color_space, gx_num_components_1,
-    gx_no_base_space, gx_equal_DevicePixel,
     gx_init_paint_1, gx_restrict_DevicePixel,
     gx_same_concrete_space,
     gx_concretize_DevicePixel, gx_remap_concrete_DevicePixel,
     gx_default_remap_color, gx_no_install_cspace,
-    gx_no_adjust_cspace_count, gx_no_adjust_color_count
+    gx_set_overprint_DevicePixel,
+    NULL, gx_no_adjust_color_count,
+    gx_serialize_DevicePixel,
+    gx_cspace_is_linear_default
 };
 
-/* Test whether one DevicePixel color space equals another. */
-private bool
-gx_equal_DevicePixel(const gs_color_space *pcs1, const gs_color_space *pcs2)
-{
-    return pcs1->params.pixel.depth == pcs2->params.pixel.depth;
-}
-
-/* Initialize a DevicePixel color space. */
+/* Create a DevicePixel color space. */
 int
-gs_cspace_init_DevicePixel(gs_color_space * pcs, int depth)
+gs_cspace_new_DevicePixel(gs_memory_t *mem, gs_color_space **ppcs, int depth)
 {
+    gs_color_space *pcs;
+
     switch (depth) {
 	case 1:
 	case 2:
@@ -64,8 +62,11 @@ gs_cspace_init_DevicePixel(gs_color_space * pcs, int depth)
 	default:
 	    return_error(gs_error_rangecheck);
     }
-    gs_cspace_init(pcs, &gs_color_space_type_DevicePixel, NULL);
+    pcs = gs_cspace_alloc(mem, &gs_color_space_type_DevicePixel);
+    if (pcs == NULL)
+	return_error(gs_error_VMerror);
     pcs->params.pixel.depth = depth;
+    *ppcs = pcs;
     return 0;
 }
 
@@ -95,10 +96,36 @@ gx_concretize_DevicePixel(const gs_client_color * pc, const gs_color_space * pcs
 }
 
 private int
-gx_remap_concrete_DevicePixel(const frac * pconc,
+gx_remap_concrete_DevicePixel(const frac * pconc, const gs_color_space * pcs,
 	gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
 			      gs_color_select_t select)
 {
     color_set_pure(pdc, pconc[0] & ((1 << dev->color_info.depth) - 1));
     return 0;
 }
+
+/* DevicePixel disables overprint */
+private int
+gx_set_overprint_DevicePixel(const gs_color_space * pcs, gs_state * pgs)
+{
+    gs_overprint_params_t   params;
+
+    params.retain_any_comps = false;
+    pgs->effective_overprint_mode = 0;
+    return gs_state_update_overprint(pgs, &params);
+}
+
+
+/* ---------------- Serialization. -------------------------------- */
+
+private int 
+gx_serialize_DevicePixel(const gs_color_space * pcs, stream * s)
+{
+    const gs_device_pixel_params * p = &pcs->params.pixel;
+    uint n;
+    int code = gx_serialize_cspace_type(pcs, s);
+
+    if (code < 0)
+	return code;
+    return sputs(s, (const byte *)&p->depth, sizeof(p->depth), &n);
+}
