@@ -1,22 +1,17 @@
-/* Copyright (C) 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gscrd.c,v 1.2.6.1.2.1 2003/01/17 00:49:02 giles Exp $ */
+/* $Id: gscrd.c 8022 2007-06-05 22:23:38Z giles $ */
 /* CIE color rendering dictionary creation */
 #include "math_.h"
 #include "memory_.h"
@@ -109,17 +104,17 @@ EncodeABC_cached_C(floatp in, const gs_cie_render * pcrd)
 private float
 EncodeLMN_cached_L(floatp in, const gs_cie_render * pcrd)
 {
-    return gs_cie_cached_value(in, &pcrd->caches.EncodeLMN[0].floats);
+    return gs_cie_cached_value(in, &pcrd->caches.EncodeLMN.caches[0].floats);
 }
 private float
 EncodeLMN_cached_M(floatp in, const gs_cie_render * pcrd)
 {
-    return gs_cie_cached_value(in, &pcrd->caches.EncodeLMN[1].floats);
+    return gs_cie_cached_value(in, &pcrd->caches.EncodeLMN.caches[1].floats);
 }
 private float
 EncodeLMN_cached_N(floatp in, const gs_cie_render * pcrd)
 {
-    return gs_cie_cached_value(in, &pcrd->caches.EncodeLMN[2].floats);
+    return gs_cie_cached_value(in, &pcrd->caches.EncodeLMN.caches[2].floats);
 }
 
 private frac
@@ -257,7 +252,7 @@ gs_cie_render1_build(gs_cie_render ** ppcrd, gs_memory_t * mem,
 
     rc_alloc_struct_1(pcrd, gs_cie_render, &st_cie_render1, mem,
 		      return_error(gs_error_VMerror), cname);
-    pcrd->id = gs_next_ids(1);
+    pcrd->id = gs_next_ids(mem, 1);
     /* Initialize pointers for the GC. */
     pcrd->client_data = 0;
     pcrd->RenderTable.lookup.table = 0;
@@ -281,7 +276,9 @@ gs_cie_render1_build(gs_cie_render ** ppcrd, gs_memory_t * mem,
  * default values.
  */
 int
-gs_cie_render1_init_from(gs_cie_render * pcrd, void *client_data,
+gs_cie_render1_init_from(const gs_memory_t *mem,
+			 gs_cie_render * pcrd, 
+			 void *client_data,
 			 const gs_cie_render * pfrom_crd,
 			 const gs_vector3 * WhitePoint,
 			 const gs_vector3 * BlackPoint,
@@ -296,7 +293,7 @@ gs_cie_render1_init_from(gs_cie_render * pcrd, void *client_data,
 			 const gs_range3 * RangeABC,
 			 const gs_cie_render_table_t * RenderTable)
 {
-    pcrd->id = gs_next_ids(1);
+    pcrd->id = gs_next_ids(mem, 1);
     pcrd->client_data = client_data;
     pcrd->points.WhitePoint = *WhitePoint;
     pcrd->points.BlackPoint =
@@ -311,7 +308,7 @@ gs_cie_render1_init_from(gs_cie_render * pcrd, void *client_data,
 	!memcmp(&pcrd->EncodeLMN, &EncodeLMN_from_cache,
 		sizeof(EncodeLMN_from_cache))
 	)
-	memcpy(pcrd->caches.EncodeLMN, pfrom_crd->caches.EncodeLMN,
+	memcpy(&pcrd->caches.EncodeLMN, &pfrom_crd->caches.EncodeLMN,
 	       sizeof(pcrd->caches.EncodeLMN));
     pcrd->RangeLMN = *(RangeLMN ? RangeLMN : &Range3_default);
     pcrd->MatrixABC = *(MatrixABC ? MatrixABC : &Matrix3_default);
@@ -345,7 +342,8 @@ gs_cie_render1_init_from(gs_cie_render * pcrd, void *client_data,
  * Initialize a CRD without the option of copying cached values.
  */
 int
-gs_cie_render1_initialize(gs_cie_render * pcrd, void *client_data,
+gs_cie_render1_initialize(const gs_memory_t *mem,
+			  gs_cie_render * pcrd, void *client_data,
 			  const gs_vector3 * WhitePoint,
 			  const gs_vector3 * BlackPoint,
 			  const gs_matrix3 * MatrixPQR,
@@ -359,7 +357,7 @@ gs_cie_render1_initialize(gs_cie_render * pcrd, void *client_data,
 			  const gs_range3 * RangeABC,
 			  const gs_cie_render_table_t * RenderTable)
 {
-    return gs_cie_render1_init_from(pcrd, client_data, NULL,
+    return gs_cie_render1_init_from(mem, pcrd, client_data, NULL,
 				    WhitePoint, BlackPoint,
 				    MatrixPQR, RangePQR, TransformPQR,
 				    MatrixLMN, EncodeLMN, RangeLMN,
