@@ -1,22 +1,17 @@
-/* Copyright (C) 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gsdsrc.c,v 1.2.6.1.2.1 2003/01/17 00:49:02 giles Exp $ */
+/* $Id: gsdsrc.c 8022 2007-06-05 22:23:38Z giles $ */
 /* DataSource procedures */
 
 #include "memory_.h"
@@ -50,17 +45,29 @@ private RELOC_PTRS_WITH(data_source_reloc_ptrs, gs_data_source_t *psrc)
 RELOC_PTRS_END
 
 /* Access data from a string or a byte object. */
-/* Does *not* check bounds. */
+/* Does check bounds, and returns 0 data oob. Spec calls for rangecheck,
+   but CPSI implementation silently gives (bogus) data. */
 int
 data_source_access_string(const gs_data_source_t * psrc, ulong start,
 			  uint length, byte * buf, const byte ** ptr)
 {
     const byte *p = psrc->data.str.data + start;
 
-    if (ptr)
-	*ptr = p;
-    else
-	memcpy(buf, p, length);
+    if (start + length <= psrc->data.str.size) {
+	if (ptr)
+	    *ptr = p;
+	else
+	    memcpy(buf, p, length);
+    } else {
+	if (start < psrc->data.str.size) {
+	    uint oklen = psrc->data.str.size - start;
+	    memcpy(buf, p, oklen);
+	    memset(buf + oklen, 0, length - oklen);
+	} else {
+	    memset(buf, 0, length);
+	}
+	*ptr = buf;
+    }
     return 0;
 }
 /* access_bytes is identical to access_string, but has a different */
