@@ -1,24 +1,20 @@
-/* Copyright (C) 1996, 1997, 1998, 1999, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gsimage.c,v 1.4.2.1.2.1 2003/01/17 00:49:02 giles Exp $ */
+/* $Id: gsimage.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Image setup procedures for Ghostscript library */
 #include "memory_.h"
+#include "math_.h"
 #include "gx.h"
 #include "gserrors.h"
 #include "gsstruct.h"
@@ -29,8 +25,9 @@
 #include "gxdevice.h"
 #include "gxiparam.h"
 #include "gxpath.h"		/* for gx_effective_clip_path */
+#include "gximask.h"
 #include "gzstate.h"
-
+#include "gsutil.h"
 
 /*
   The main internal invariant for the gs_image machinery is
@@ -150,6 +147,43 @@ private RELOC_PTRS_WITH(gs_image_enum_reloc_ptrs, gs_image_enum *eptr)
 }
 RELOC_PTRS_END
 
+private int
+is_image_visible(const gs_image_common_t * pic, gs_state * pgs, gx_clip_path *pcpath)
+{
+    /* HACK : We need the source image size here, 
+       but gs_image_common_t doesn't pass it.
+       We would like to move Width, Height to gs_image_common,
+       but gs_image2_t appears to have those fields of double type.
+     */
+    if (pic->type->begin_typed_image == gx_begin_image1) {
+	gs_image1_t *pim = (gs_image1_t *) pic;
+	gs_rect image_rect = {{0, 0}, {0, 0}};
+	gs_rect device_rect;
+	gs_int_rect device_int_rect;
+	gs_matrix mat;
+	int code;
+
+	image_rect.q.x = pim->Width;
+	image_rect.q.y = pim->Height;
+	code = gs_matrix_invert(&pic->ImageMatrix, &mat);
+	if (code < 0)
+	    return code;
+	code = gs_matrix_multiply(&mat, &ctm_only(pgs), &mat);
+	if (code < 0)
+	    return code;
+	code = gs_bbox_transform(&image_rect, &mat, &device_rect);
+	if (code < 0)
+	    return code;
+	device_int_rect.p.x = (int)floor(device_rect.p.x);
+	device_int_rect.p.y = (int)floor(device_rect.p.y);
+	device_int_rect.q.x = (int)ceil(device_rect.q.x);
+	device_int_rect.q.y = (int)ceil(device_rect.q.y);
+	if (!gx_cpath_rect_visible(pcpath, &device_int_rect))
+	    return 0;
+    }
+    return 1;
+}
+
 /* Create an image enumerator given image parameters and a graphics state. */
 int
 gs_image_begin_typed(const gs_image_common_t * pic, gs_state * pgs,
@@ -158,17 +192,43 @@ gs_image_begin_typed(const gs_image_common_t * pic, gs_state * pgs,
     gx_device *dev = gs_currentdevice(pgs);
     gx_clip_path *pcpath;
     int code = gx_effective_clip_path(pgs, &pcpath);
+    gx_device *dev2 = dev;
 
     if (code < 0)
 	return code;
+    /* Processing an image object operation */
+    gs_set_object_tag(pgs, GS_IMAGE_TAG);
+
     if (uses_color) {
 	gx_set_dev_color(pgs);
         code = gs_state_color_load(pgs);
         if (code < 0)
 	    return code;
     }
-    return gx_device_begin_typed_image(dev, (const gs_imager_state *)pgs,
+    /* Imagemask with shading color needs a special optimization
+       with converting the image into a clipping. 
+       Check for such case after gs_state_color_load is done,
+       because it can cause interpreter callout.
+     */
+    if (pic->type->begin_typed_image == &gx_begin_image1) {
+	gs_image_t *image = (gs_image_t *)pic;
+
+	if(image->ImageMask) {
+	    code = gx_image_fill_masked_start(dev, pgs->dev_color, pcpath, pgs->memory, &dev2);
+	    if (code < 0)
+		return code;
+	}
+    }
+    code = gx_device_begin_typed_image(dev2, (const gs_imager_state *)pgs,
 		NULL, pic, NULL, pgs->dev_color, pcpath, pgs->memory, ppie);
+    if (code < 0)
+	return code;
+    code = is_image_visible(pic, pgs, pcpath);
+    if (code < 0)
+	return code;
+    if (!code)	
+	(*ppie)->skipping = true;
+    return 0;
 }
 
 /* Allocate an image enumerator. */
@@ -211,9 +271,13 @@ gs_image_init(gs_image_enum * penum, const gs_image_t * pim, bool multi,
     } else {
 	if (pgs->in_cachedevice)
 	    return_error(gs_error_undefined);
-	if (image.ColorSpace == NULL)
-	    image.ColorSpace =
-		gs_cspace_DeviceGray((const gs_imager_state *)pgs);
+	if (image.ColorSpace == NULL) {
+            /*
+             * Use of a non-current color space is potentially
+             * incorrect, but it appears this case doesn't arise.
+             */
+	    image.ColorSpace = gs_cspace_new_DeviceGray(pgs->memory);
+        }
     }
     code = gs_image_begin_typed((const gs_image_common_t *)&image, pgs,
 				image.ImageMask | image.CombineWithColor,
@@ -276,19 +340,15 @@ begin_planes(gs_image_enum *penum)
     next_plane(penum);
 }
 
-/* Start processing a general image. */
-int
-gs_image_enum_init(gs_image_enum * penum, gx_image_enum_common_t * pie,
-		   const gs_data_image_t * pim, gs_state *pgs)
-{
-    return gs_image_common_init(penum, pie, pim, pgs->memory,
-				(pgs->in_charpath ? NULL :
-				 gs_currentdevice_inline(pgs)));
-}
 int
 gs_image_common_init(gs_image_enum * penum, gx_image_enum_common_t * pie,
-	    const gs_data_image_t * pim, gs_memory_t * mem, gx_device * dev)
+	    const gs_data_image_t * pim, gx_device * dev)
 {
+    /*
+     * HACK : For a compatibility with gs_image_cleanup_and_free_enum,
+     * penum->memory must be initialized in advance 
+     * with the memory heap that owns *penum.
+     */
     int i;
 
     if (pim->Width == 0 || pim->Height == 0) {
@@ -296,7 +356,6 @@ gs_image_common_init(gs_image_enum * penum, gx_image_enum_common_t * pie,
 	return 1;
     }
     image_enum_init(penum);
-    penum->memory = mem;
     penum->dev = dev;
     penum->info = pie;
     penum->num_planes = pie->num_planes;
@@ -312,7 +371,8 @@ gs_image_common_init(gs_image_enum * penum, gx_image_enum_common_t * pie,
     for (i = 0; i < pie->num_planes; ++i) {
 	penum->planes[i].pos = 0;
 	penum->planes[i].source.size = 0;	/* for gs_image_next_planes */
-	penum->planes[i].row.data = 0; /* for GC */
+	penum->planes[i].source.data = 0; /* for GC */
+        penum->planes[i].row.data = 0; /* for GC */
 	penum->planes[i].row.size = 0; /* ditto */
 	penum->image_planes[i].data_x = 0; /* just init once, never changes */
     }
@@ -324,6 +384,18 @@ gs_image_common_init(gs_image_enum * penum, gx_image_enum_common_t * pie,
     return 0;
 }
 
+/* Initialize an enumerator for a general image. 
+   penum->memory must be initialized in advance.
+*/
+int
+gs_image_enum_init(gs_image_enum * penum, gx_image_enum_common_t * pie,
+		   const gs_data_image_t * pim, gs_state *pgs)
+{
+    return gs_image_common_init(penum, pie, pim,
+				(pgs->in_charpath ? NULL :
+				 gs_currentdevice_inline(pgs)));
+}
+
 /* Return the set of planes wanted. */
 const byte *
 gs_image_planes_wanted(gs_image_enum *penum)
@@ -508,6 +580,7 @@ gs_image_next_planes(gs_image_enum * penum,
 	    if_debug2('b', "[b]used %d, code=%d\n", h, code);
 	    penum->error = code < 0;
 	}
+	penum->y += h;
 	/* Update positions and sizes. */
 	if (h == 0)
 	    break;
@@ -538,11 +611,38 @@ gs_image_next_planes(gs_image_enum * penum,
 }
 
 /* Clean up after processing an image. */
-void
-gs_image_cleanup(gs_image_enum * penum)
+/* Public for ghotpcl. */
+int
+gs_image_cleanup(gs_image_enum * penum, gs_state *pgs)
 {
+    int code = 0, code1;
+
     free_row_buffers(penum, penum->num_planes, "gs_image_cleanup(row)");
-    if (penum->dev != 0)
-	gx_image_end(penum->info, !penum->error);
+    if (penum->info != 0) {
+	if (dev_proc(penum->info->dev, pattern_manage)(penum->info->dev, 
+		    gs_no_id, NULL, pattern_manage__is_cpath_accum)) {
+	    /* Performing a conversion of imagemask into a clipping path. */
+	    gx_device *cdev = penum->info->dev;
+
+	    code = gx_image_end(penum->info, !penum->error); /* Releases penum->info . */
+	    code1 = gx_image_fill_masked_end(cdev, penum->dev, pgs->dev_color);
+	    if (code == 0)
+		code = code1;
+	} else
+	    code = gx_image_end(penum->info, !penum->error);
+    }
     /* Don't free the local enumerator -- the client does that. */
+
+    return code;
 }
+
+/* Clean up after processing an image and free the enumerator. */
+int
+gs_image_cleanup_and_free_enum(gs_image_enum * penum, gs_state *pgs)
+{
+    int code = gs_image_cleanup(penum, pgs);
+
+    gs_free_object(penum->memory, penum, "gs_image_cleanup_and_free_enum");
+    return code;
+}
+
