@@ -1,22 +1,17 @@
-/* Copyright (C) 1989, 1996, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gspaint.c,v 1.2.6.1.2.1 2003/01/17 00:49:03 giles Exp $ */
+/* $Id: gspaint.c 8161 2007-08-01 21:20:57Z ray $ */
 /* Painting procedures for Ghostscript library */
 #include "math_.h"		/* for fabs */
 #include "gx.h"
@@ -33,6 +28,10 @@
 #include "gxdevice.h"
 #include "gxdevmem.h"
 #include "gzcpath.h"
+#include "gxhldevc.h"
+#include "gsutil.h"
+
+extern bool CPSI_mode;
 
 /* Define the nominal size for alpha buffers. */
 #define abuf_nominal_SMALL 500
@@ -68,18 +67,39 @@ gs_erasepage(gs_state * pgs)
 int
 gs_fillpage(gs_state * pgs)
 {
-    gx_device *dev;
-    int code;
+    gx_device *dev = gs_currentdevice(pgs);
+    int code = 0;
     gs_logical_operation_t save_lop;
+    bool hl_color_available;
+
+    /* If we get here without a valid get_color_mapping_procs, fail */
+    if (dev_proc(dev, get_color_mapping_procs) == NULL || 
+        dev_proc(dev, get_color_mapping_procs) == gx_error_get_color_mapping_procs) {
+	eprintf1("\n   *** Error: No get_color_mapping_procs for device: %s\n", dev->dname);
+	return_error(gs_error_Fatal);
+    }
+    /* Processing a fill object operation */
+    gs_set_object_tag(pgs, GS_PATH_TAG);
 
     gx_set_dev_color(pgs);
-    dev = gs_currentdevice(pgs);
+    hl_color_available = gx_hld_is_hl_color_available((gs_imager_state *)pgs, 
+						    pgs->dev_color);
     /* Fill the page directly, ignoring clipping. */
     /* Use the default RasterOp. */
     save_lop = pgs->log_op;
     gs_init_rop(pgs);
-    code = gx_fill_rectangle(0, 0, dev->width, dev->height,
-			     pgs->dev_color, pgs);
+    if (hl_color_available) {
+	gs_fixed_rect rect;
+
+	rect.p.x = rect.p.y = 0;
+	rect.q.x = int2fixed(dev->width);
+	rect.q.y = int2fixed(dev->height);
+	code = dev_proc(pgs->device, fill_rectangle_hl_color)(pgs->device, 
+		&rect, (const gs_imager_state *)pgs, pgs->dev_color, NULL);
+    }
+    if (!hl_color_available || code == gs_error_rangecheck)
+	code = gx_fill_rectangle(0, 0, dev->width, dev->height,
+				 pgs->dev_color, pgs);
     pgs->log_op = save_lop;
     if (code < 0)
 	return code;
@@ -227,17 +247,19 @@ alpha_buffer_init(gs_state * pgs, fixed extra_x, fixed extra_y, int alpha_bits)
 }
 
 /* Release an alpha buffer. */
-private void
+private int
 alpha_buffer_release(gs_state * pgs, bool newpath)
 {
     gx_device_memory *mdev =
-    (gx_device_memory *) gs_currentdevice_inline(pgs);
+	(gx_device_memory *) gs_currentdevice_inline(pgs);
+    int code = (*dev_proc(mdev, close_device)) ((gx_device *) mdev);
 
-    (*dev_proc(mdev, close_device)) ((gx_device *) mdev);
-    scale_paths(pgs, -mdev->log2_scale.x, -mdev->log2_scale.y,
+    if (code >= 0)
+	scale_paths(pgs, -mdev->log2_scale.x, -mdev->log2_scale.y,
 		!(newpath && !gx_path_is_shared(pgs->path)));
     /* Reference counting will free mdev. */
     gx_set_device_only(pgs, mdev->target);
+    return code;
 }
 
 /* Fill the current path using a specified rule. */
@@ -251,9 +273,47 @@ fill_with_rule(gs_state * pgs, int rule)
     if (pgs->in_charpath)
 	code = gx_path_add_char_path(pgs->show_gstate->path, pgs->path,
 				     pgs->in_charpath);
-    else {
-	int abits, acode;
+    else if (gs_is_null_device(pgs->device)) {
+	/* Handle separately to prevent gs_state_color_load - bug 688308. */
+	gs_newpath(pgs);
+	code = 0;
+    } else {
+	int abits, acode, rcode = 0;
 
+	/* Here we need to distinguish text from vectors to compute the object tag.
+	   Actually we need to know whether this function is called to rasterize a character,
+	   or to rasterize a vector graphics to the output device.
+	   Currently we assume it works for the bitrgbtags device only,
+	   which is a low level device with a 4-component color model.
+	   We use the fact that with printers a character is usually being rendered 
+	   to a 1bpp cache device rather than to the output device.
+	   Therefore we hackly look whether the target device
+	   "has a color" : either it's a multicomponent color model,
+	   or it is not gray (such as a yellow separation).
+
+	   This check has several limitations :
+	   1. It doesn't work with -dNOCACHE.
+	   2. It doesn't work with large characters,
+	      which cannot fit into a cache cell and thus they
+	      render directly to the output device.
+	   3. It doesn't work for TextAlphaBits=2 or 4.
+	      We don't care of this case because
+	      text antialiasing usually usn't applied to printers.
+	   4. It doesn't work for things like with "(xyz) true charpath stroke".
+	      That's unfortunate, we'd like to improve someday.
+	   5. It doesn't work for high level devices when a Type 3 character is being constructed.
+	      This case is not important for low level devices
+	      (which a printer is), because low level device doesn't accept
+	      Type 3 charproc streams immediately.
+	   6. It doesn't work properly while an insiding testing,
+	      which sets gs_hit_device, which is uncolored.
+	 */
+        if (gx_device_has_color(gs_currentdevice(pgs))) {
+            gs_set_object_tag(pgs, GS_PATH_TAG);
+	}
+	else {
+            gs_set_object_tag(pgs, GS_TEXT_TAG);
+	}
 	gx_set_dev_color(pgs);
 	code = gs_state_color_load(pgs);
 	if (code < 0)
@@ -269,9 +329,11 @@ fill_with_rule(gs_state * pgs, int rule)
 	code = gx_fill_path(pgs->path, pgs->dev_color, pgs, rule,
 			    pgs->fill_adjust.x, pgs->fill_adjust.y);
 	if (acode > 0)
-	    alpha_buffer_release(pgs, code >= 0);
+	    rcode = alpha_buffer_release(pgs, code >= 0);
 	if (code >= 0)
 	    gs_newpath(pgs);
+	if (code >= 0 && rcode < 0)
+	    code = rcode;
 
     }
     return code;
@@ -311,9 +373,54 @@ gs_stroke(gs_state * pgs)
 	}
 	code = gx_path_add_char_path(pgs->show_gstate->path, pgs->path,
 				     pgs->in_charpath);
+    }
+    if (gs_is_null_device(pgs->device)) {
+	/* Handle separately to prevent gs_state_color_load. */
+	gs_newpath(pgs);
+	code = 0;
     } else {
-	int abits, acode;
+	int abits, acode, rcode = 0;
 
+        /* to distinguish text from vectors we hackly look at the
+           target device 1 bit per component is a cache and this is
+           text else it is a path */
+        if (gx_device_has_color(gs_currentdevice(pgs)))
+            gs_set_object_tag(pgs, GS_PATH_TAG);
+        else
+            gs_set_object_tag(pgs, GS_TEXT_TAG);
+
+	/* Here we need to distinguish text from vectors to compute the object tag.
+	   Actually we need to know whether this function is called to rasterize a character,
+	   or to rasterize a vector graphics to the output device.
+	   Currently we assume it works for the bitrgbtags device only,
+	   which is a low level device with a 4-component color model.
+	   We use the fact that with printers a character is usually being rendered 
+	   to a 1bpp cache device rather than to the output device.
+	   Therefore we hackly look whether the target device
+	   "has a color" : either it's a multicomponent color model,
+	   or it is not gray (such as a yellow separation).
+
+	   This check has several limitations :
+	   1. It doesn't work with -dNOCACHE.
+	   2. It doesn't work with large characters,
+	      which cannot fit into a cache cell and thus they
+	      render directly to the output device.
+	   3. It doesn't work for TextAlphaBits=2 or 4.
+	      We don't care of this case because
+	      text antialiasing usually usn't applied to printers.
+	   4. It doesn't work for things like with "(xyz) true charpath stroke".
+	      That's unfortunate, we'd like to improve someday.
+	   5. It doesn't work for high level devices when a Type 3 character is being constructed.
+	      This case is not important for low level devices
+	      (which a printer is), because low level device doesn't accept
+	      Type 3 charproc streams immediately.
+	 */
+        if (gx_device_has_color(gs_currentdevice(pgs))) {
+            gs_set_object_tag(pgs, GS_PATH_TAG);
+	}
+	else {
+            gs_set_object_tag(pgs, GS_TEXT_TAG);
+	}
 	gx_set_dev_color(pgs);
 	code = gs_state_color_load(pgs);
 	if (code < 0)
@@ -327,7 +434,7 @@ gs_stroke(gs_state * pgs)
 	     */
 	    float xxyy = fabs(pgs->ctm.xx) + fabs(pgs->ctm.yy);
 	    float xyyx = fabs(pgs->ctm.xy) + fabs(pgs->ctm.yx);
-	    float scale = 1 << (abits / 2);
+	    float scale = (float)(1 << (abits / 2));
 	    float orig_width = gs_currentlinewidth(pgs);
 	    float new_width = orig_width * scale;
 	    fixed extra_adjust =
@@ -363,11 +470,13 @@ gs_stroke(gs_state * pgs)
 	    gs_setflat(pgs, orig_flatness);
 	    gx_path_free(&spath, "gs_stroke");
 	    if (acode > 0)
-		alpha_buffer_release(pgs, code >= 0);
+		rcode = alpha_buffer_release(pgs, code >= 0);
 	} else
 	    code = gx_stroke_fill(pgs->path, pgs);
 	if (code >= 0)
 	    gs_newpath(pgs);
+	if (code >= 0 && rcode < 0)
+	    code = rcode;
     }
     return code;
 }
@@ -379,11 +488,20 @@ gs_strokepath(gs_state * pgs)
     gx_path spath;
     int code;
 
-    gx_path_init_local(&spath, pgs->memory);
+    gx_path_init_local(&spath, pgs->path->memory);
     code = gx_stroke_add(pgs->path, &spath, pgs);
     if (code < 0) {
 	gx_path_free(&spath, "gs_strokepath");
 	return code;
     }
-    return gx_path_assign_free(pgs->path, &spath);
+    code = gx_path_assign_free(pgs->path, &spath);
+    if (code < 0)
+	return code;
+    /* NB: needs testing with PCL */
+    if (CPSI_mode && gx_path_is_void(pgs->path))
+        pgs->current_point_valid = false;
+    else
+        gx_setcurrentpoint(pgs, fixed2float(spath.position.x), fixed2float(spath.position.y));
+    return 0;
+
 }
