@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gspath.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gspath.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Basic path routines for Ghostscript library */
 #include "gx.h"
 #include "math_.h"
@@ -50,7 +50,24 @@ gs_closepath(gs_state * pgs)
 int
 gs_upmergepath(gs_state * pgs)
 {
-    return gx_path_add_path(pgs->saved->path, pgs->path);
+    /*
+     * We really should be able to implement this as simply
+     *   return gx_path_add_path(pgs->saved->path, pgs->path);
+     * But because of the current_point members in the imager state,
+     * we can't.
+     */
+    gs_state *saved = pgs->saved;
+    int code;
+
+    code = gx_path_add_path(saved->path, pgs->path);
+    if (code < 0)
+	return code;
+    if (pgs->current_point_valid) {
+	saved->current_point = pgs->current_point;
+	saved->subpath_start = pgs->subpath_start;
+	saved->current_point_valid = true;
+    }
+    return code;
 }
 
 /* Get the current path (for internal use only). */
@@ -406,8 +423,12 @@ gs_clippath(gs_state * pgs)
 
     gx_path_init_local(&cpath, pgs->path->memory);
     code = gx_cpath_to_path(pgs->clip_path, &cpath);
-    if (code >= 0)
+    if (code >= 0) {
 	code = gx_path_assign_free(pgs->path, &cpath);
+	pgs->current_point.x = fixed2float(pgs->path->position.x);
+	pgs->current_point.y = fixed2float(pgs->path->position.y);
+	pgs->current_point_valid = true;
+    }
     if (code < 0)
 	gx_path_free(&cpath, "gs_clippath");
     return code;
