@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxclip.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gxclip.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Implementation of (path-based) clipping */
 #include "gx.h"
 #include "gxdevice.h"
@@ -101,7 +101,8 @@ private const gx_device_clip gs_clip_device =
   gx_default_fill_linear_color_scanline,
   gx_default_fill_linear_color_trapezoid,
   gx_default_fill_linear_color_triangle,
-  gx_forward_update_spot_equivalent_colors
+  gx_forward_update_spot_equivalent_colors,
+  gx_forward_ret_devn_params
  }
 };
 
