@@ -1,22 +1,17 @@
-/* Copyright (C) 1997 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxcllzw.c,v 1.2.6.1.2.1 2003/01/17 00:49:03 giles Exp $ */
+/* $Id: gxcllzw.c 8022 2007-06-05 22:23:38Z giles $ */
 /* LZW filter initialization for RAM-based band lists */
 #include "std.h"
 #include "gstypes.h"
@@ -28,13 +23,14 @@ private stream_LZW_state cl_LZWE_state;
 private stream_LZW_state cl_LZWD_state;
 
 /* Initialize the states to be copied. */
-void
+int
 gs_cl_lzw_init(gs_memory_t * mem)
 {
     s_LZW_set_defaults((stream_state *) & cl_LZWE_state);
     cl_LZWE_state.template = &s_LZWE_template;
     s_LZW_set_defaults((stream_state *) & cl_LZWD_state);
     cl_LZWD_state.template = &s_LZWD_template;
+    return 0;
 }
 
 /* Return the prototypes for compressing/decompressing the band list. */
