@@ -1,22 +1,17 @@
-/* Copyright (C) 1991, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxcpath.c,v 1.3.6.1.2.1 2003/01/17 00:49:03 giles Exp $ */
+/* $Id: gxcpath.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Implementation of clipping paths, other than actual clipping */
 #include "gx.h"
 #include "gserrors.h"
@@ -25,39 +20,41 @@
 #include "gsline.h"
 #include "gxdevice.h"
 #include "gxfixed.h"
+#include "gxpaint.h"
 #include "gscoord.h"		/* needs gsmatrix.h */
 #include "gxistate.h"
 #include "gzpath.h"
 #include "gzcpath.h"
-
-/* Imported from gxacpath.c */
-extern int gx_cpath_intersect_path_slow(P4(gx_clip_path *, gx_path *, int,
-					   gs_imager_state *));
+#include "gzacpath.h"
 
 /* Forward references */
-private void gx_clip_list_from_rectangle(P2(gx_clip_list *, gs_fixed_rect *));
+private void gx_clip_list_from_rectangle(gx_clip_list *, gs_fixed_rect *);
 
 /* Other structure types */
 public_st_clip_rect();
-private_st_clip_list();
+public_st_clip_list();
 public_st_clip_path();
 private_st_clip_rect_list();
 public_st_device_clip();
 private_st_cpath_enum();
+private_st_cpath_path_list();
 
 /* GC procedures for gx_clip_path */
 private 
-ENUM_PTRS_WITH(clip_path_enum_ptrs, gx_clip_path *cptr) return ENUM_USING(st_path, &cptr->path, sizeof(cptr->path), index - 1);
+ENUM_PTRS_WITH(clip_path_enum_ptrs, gx_clip_path *cptr) return ENUM_USING(st_path, &cptr->path, sizeof(cptr->path), index - 2);
 
 case 0:
-ENUM_RETURN((cptr->rect_list == &cptr->local_list ? 0 :
+return ENUM_OBJ((cptr->rect_list == &cptr->local_list ? 0 :
 	     cptr->rect_list));
+case 1:
+return ENUM_OBJ(cptr->path_list);
 ENUM_PTRS_END
 private
 RELOC_PTRS_WITH(clip_path_reloc_ptrs, gx_clip_path *cptr)
 {
     if (cptr->rect_list != &cptr->local_list)
 	RELOC_VAR(cptr->rect_list);
+    RELOC_VAR(cptr->path_list);
     RELOC_USING(st_path, &cptr->path, sizeof(gx_path));
 }
 RELOC_PTRS_END
@@ -99,6 +96,7 @@ private const gx_clip_list clip_list_empty = {
 
 private rc_free_proc(rc_free_cpath_list);
 private rc_free_proc(rc_free_cpath_list_local);
+private rc_free_proc(rc_free_cpath_path_list);
 
 /*
  * Initialize those parts of the contents of a clip path that aren't
@@ -112,7 +110,7 @@ cpath_init_rectangle(gx_clip_path * pcpath, gs_fixed_rect * pbox)
     pcpath->path_valid = false;
     pcpath->path.bbox = *pbox;
     gx_cpath_set_outer_box(pcpath);
-    pcpath->id = gs_next_ids(1);	/* path changed => change id */
+    pcpath->id = gs_next_ids(pcpath->path.memory, 1);	/* path changed => change id */
 }
 private void
 cpath_init_own_contents(gx_clip_path * pcpath)
@@ -121,6 +119,7 @@ cpath_init_own_contents(gx_clip_path * pcpath)
 
     null_rect.p.x = null_rect.p.y = null_rect.q.x = null_rect.q.y = 0;
     cpath_init_rectangle(pcpath, &null_rect);
+    pcpath->path_list = NULL;
 }
 private void
 cpath_share_own_contents(gx_clip_path * pcpath, const gx_clip_path * shared)
@@ -156,6 +155,7 @@ gx_cpath_init_contained_shared(gx_clip_path * pcpath,
 	pcpath->path.allocation = path_allocated_contained;
 	rc_increment(pcpath->path.segments);
 	rc_increment(pcpath->rect_list);
+	rc_increment(pcpath->path_list);
     } else {
 	int code = cpath_alloc_list(&pcpath->rect_list, mem, cname);
 
@@ -210,6 +210,8 @@ gx_cpath_init_local_shared(gx_clip_path * pcpath, const gx_clip_path * shared,
 	rc_increment(pcpath->path.segments);
 	pcpath->rect_list = shared->rect_list;
 	rc_increment(pcpath->rect_list);
+	pcpath->path_list = shared->path_list;
+	rc_increment(pcpath->path_list);
 	cpath_share_own_contents(pcpath, shared);
     } else {
 	gx_path_init_local(&pcpath->path, mem);
@@ -247,8 +249,10 @@ void
 gx_cpath_free(gx_clip_path * pcpath, client_name_t cname)
 {
     rc_decrement(pcpath->rect_list, cname);
+    rc_decrement(pcpath->path_list, cname);
     /* Clean up pointers for GC. */
     pcpath->rect_list = 0;
+    pcpath->path_list = 0;
     {
 	gx_path_allocation_t alloc = pcpath->path.allocation;
 
@@ -295,6 +299,7 @@ gx_cpath_assign_preserve(gx_clip_path * pcpto, gx_clip_path * pcpfrom)
 	rc_increment(fromlist);
 	rc_decrement(pcpto->rect_list, "gx_cpath_assign");
     }
+    rc_increment(pcpfrom->path_list);
     path = pcpto->path, *pcpto = *pcpfrom, pcpto->path = path;
     return 0;
 }
@@ -327,58 +332,109 @@ rc_free_cpath_list(gs_memory_t * mem, void *vrlist, client_name_t cname)
     gs_free_object(mem, vrlist, cname);
 }
 
+private void
+rc_free_cpath_path_list(gs_memory_t * mem, void *vplist, client_name_t cname)
+{
+    gx_cpath_path_list *plist = (gx_cpath_path_list *)vplist;
+    rc_decrement(plist->next, cname);
+    gx_path_free(&plist->path, cname);
+    gs_free_object(plist->path.memory, plist, cname);
+}
+
+/* Allocate a new clip path list node. The created node has a ref count
+   of 1, and "steals" the reference to next (i.e. does not increment
+   its reference count). */
+private int
+gx_cpath_path_list_new(gs_memory_t *mem, gx_clip_path *pcpath, int rule, 
+			gx_path *ppfrom, gx_cpath_path_list *next, gx_cpath_path_list **pnew)
+{
+    int code;
+    client_name_t cname = "gx_cpath_path_list_new";
+    gx_cpath_path_list *pcplist = gs_alloc_struct(mem, gx_cpath_path_list,
+						  &st_cpath_path_list, cname);
+
+    if (pcplist == 0)
+	return_error(gs_error_VMerror);
+    rc_init_free(pcplist, mem, 1, rc_free_cpath_path_list);
+    if (pcpath!=NULL && !pcpath->path_valid) {
+	code = gx_path_init_contained_shared(&pcplist->path, NULL, mem, cname);
+	if (code < 0)
+	    return code;
+	code = gx_cpath_to_path(pcpath, &pcplist->path);
+    } else {
+	gx_path_init_local(&pcplist->path, mem);
+	code = gx_path_assign_preserve(&pcplist->path, ppfrom);
+    }
+    if (code < 0)
+	return code;
+    pcplist->next = next;
+    pcplist->rule = rule;
+    *pnew = pcplist;
+    return 0;
+}
+
 /* ------ Clipping path accessing ------ */
 
+/* Synthesize a path from a clipping path. */
+int
+gx_cpath_to_path_synthesize(const gx_clip_path * pcpath, gx_path * ppath)
+{
+    /* Synthesize a path. */
+    gs_cpath_enum cenum;
+    gs_fixed_point pts[3];
+    int code;
+
+    gx_cpath_enum_init(&cenum, pcpath);
+    while ((code = gx_cpath_enum_next(&cenum, pts)) != 0) {
+	switch (code) {
+	    case gs_pe_moveto:
+		code = gx_path_add_point(ppath, pts[0].x, pts[0].y);
+		break;
+	    case gs_pe_lineto:
+		code = gx_path_add_line_notes(ppath, pts[0].x, pts[0].y,
+					   gx_cpath_enum_notes(&cenum));
+		break;
+	    case gs_pe_curveto:
+		code = gx_path_add_curve_notes(ppath, pts[0].x, pts[0].y,
+					       pts[1].x, pts[1].y,
+					       pts[2].x, pts[2].y,
+					   gx_cpath_enum_notes(&cenum));
+		break;
+	    case gs_pe_closepath:
+		code = gx_path_close_subpath_notes(ppath,
+					   gx_cpath_enum_notes(&cenum));
+		break;
+	    default:
+		if (code >= 0)
+		    code = gs_note_error(gs_error_unregistered);
+	}
+	if (code < 0)
+	    break;
+    }
+    return 0;
+}
+
 /* Return the path of a clipping path. */
 int
 gx_cpath_to_path(gx_clip_path * pcpath, gx_path * ppath)
 {
     if (!pcpath->path_valid) {
-	/* Synthesize a path. */
-	gs_cpath_enum cenum;
-	gs_fixed_point pts[3];
 	gx_path rpath;
 	int code;
 
 	gx_path_init_local(&rpath, pcpath->path.memory);
-	gx_cpath_enum_init(&cenum, pcpath);
-	while ((code = gx_cpath_enum_next(&cenum, pts)) != 0) {
-	    switch (code) {
-		case gs_pe_moveto:
-		    code = gx_path_add_point(&rpath, pts[0].x, pts[0].y);
-		    break;
-		case gs_pe_lineto:
-		    code = gx_path_add_line_notes(&rpath, pts[0].x, pts[0].y,
-					       gx_cpath_enum_notes(&cenum));
-		    break;
-		case gs_pe_curveto:
-		    code = gx_path_add_curve_notes(&rpath, pts[0].x, pts[0].y,
-						   pts[1].x, pts[1].y,
-						   pts[2].x, pts[2].y,
-					       gx_cpath_enum_notes(&cenum));
-		    break;
-		case gs_pe_closepath:
-		    code = gx_path_close_subpath_notes(&rpath,
-					       gx_cpath_enum_notes(&cenum));
-		    break;
-		default:
-		    if (code >= 0)
-			code = gs_note_error(gs_error_unregistered);
-	    }
-	    if (code < 0)
-		break;
-	}
-	if (code >= 0)
-	    code = gx_path_assign_free(&pcpath->path, &rpath);
+	code = gx_cpath_to_path_synthesize(pcpath, &rpath);
 	if (code < 0) {
 	    gx_path_free(&rpath, "gx_cpath_to_path error");
 	    return code;
 	}
+	code = gx_path_assign_free(&pcpath->path, &rpath);
+	if (code < 0)
+	    return code;
 	pcpath->path_valid = true;
     }
     return gx_path_assign_preserve(ppath, &pcpath->path);
 }
-
 /* Return the inner and outer check rectangles for a clipping path. */
 /* Return true iff the path is a rectangle. */
 bool
@@ -428,7 +484,7 @@ gx_cpath_list(const gx_clip_path *pcpath)
 }
 /* Internal non-const version of the same accessor. */
 inline private gx_clip_list *
-gx_cpath_list_private(gx_clip_path *pcpath)
+gx_cpath_list_private(const gx_clip_path *pcpath)
 {
     return &pcpath->rect_list->list;
 }
@@ -476,6 +532,20 @@ gx_cpath_reset(gx_clip_path * pcpath)
     return gx_cpath_from_rectangle(pcpath, &null_rect);
 }
 
+/* If a clipping path is a rectangle, return the rectangle. */
+const gs_fixed_rect *
+cpath_is_rectangle(const gx_clip_path * pcpath)
+{
+    if (pcpath->path_valid)
+	return NULL;
+    if (pcpath->inner_box.p.x != pcpath->path.bbox.p.x ||
+	pcpath->inner_box.p.y != pcpath->path.bbox.p.y ||
+	pcpath->inner_box.q.x != pcpath->path.bbox.q.x ||
+	pcpath->inner_box.q.y != pcpath->path.bbox.q.y)
+	return NULL;
+    return &pcpath->inner_box;
+}
+
 /* Intersect a new clipping path with an old one. */
 /* Flatten the new path first (in a copy) if necessary. */
 int
@@ -485,9 +555,10 @@ gx_cpath_clip(gs_state *pgs, gx_clip_path *pcpath,
     return gx_cpath_intersect(pcpath, ppath_orig, rule,
 			      (gs_imager_state *)pgs);
 }
+
 int
-gx_cpath_intersect(gx_clip_path *pcpath, /*const*/ gx_path *ppath_orig,
-		   int rule, gs_imager_state *pis)
+gx_cpath_intersect_with_params(gx_clip_path *pcpath, /*const*/ gx_path *ppath_orig,
+		   int rule, gs_imager_state *pis, const gx_fill_params * params)
 {
     gx_path fpath;
     /*const*/ gx_path *ppath = ppath_orig;
@@ -504,7 +575,7 @@ gx_cpath_intersect(gx_clip_path *pcpath, /*const*/ gx_path *ppath_orig,
 	    return code;
 	ppath = &fpath;
     }
-    /**************** SHOULD CHANGE THIS TO KEEP PATH ****************/
+
     if (gx_cpath_inner_box(pcpath, &old_box) &&
 	((code = gx_path_is_rectangle(ppath, &new_box)) ||
 	 gx_path_is_void(ppath))
@@ -521,6 +592,28 @@ gx_cpath_intersect(gx_clip_path *pcpath, /*const*/ gx_path *ppath_orig,
 	    new_box.q = new_box.p;
 	    changed = 1;
 	} else {
+	    if (params != NULL) {
+		/* Called from gx_default_fill_path for converting 
+		   a filling path into a clipping path.  
+		   Apply same adjustment as for filling the path. */
+		gs_fixed_point adjust = params->adjust;
+		fixed adjust_xl, adjust_xu, adjust_yl, adjust_yu;
+
+		if (adjust.x == -1)
+		    adjust_xl = adjust_xu = adjust_yl = adjust_yu = 0;
+		else {
+		    if (params->fill_zero_width)
+			gx_adjust_if_empty(&new_box, &adjust);
+		    adjust_xl = (adjust.x == fixed_half ? fixed_half - fixed_epsilon : adjust.x);
+		    adjust_yl = (adjust.y == fixed_half ? fixed_half - fixed_epsilon : adjust.y);
+		    adjust_xu = adjust.x;
+		    adjust_yu = adjust.y;
+		}
+		new_box.p.x = int2fixed(fixed2int_pixround(new_box.p.x - adjust_xl));
+		new_box.p.y = int2fixed(fixed2int_pixround(new_box.p.y - adjust_yl));
+		new_box.q.x = int2fixed(fixed2int_pixround(new_box.q.x + adjust_xu));
+		new_box.q.y = int2fixed(fixed2int_pixround(new_box.q.y + adjust_yu));
+	    }
 	    /* Intersect the two rectangles if necessary. */
 	    if (old_box.p.x > new_box.p.x)
 		new_box.p.x = old_box.p.x, ++changed;
@@ -539,6 +632,8 @@ gx_cpath_intersect(gx_clip_path *pcpath, /*const*/ gx_path *ppath_orig,
 	    return 0;
 	}
 	/* Release the existing path. */
+	rc_decrement(pcpath->path_list, "gx_cpath_intersect");
+	pcpath->path_list = NULL;
 	gx_path_new(&pcpath->path);
 	ppath->bbox = new_box;
 	cpath_set_rectangle(pcpath, &new_box);
@@ -548,7 +643,8 @@ gx_cpath_intersect(gx_clip_path *pcpath, /*const*/ gx_path *ppath_orig,
 	    pcpath->path_valid = true;
 	}
     } else {
-	/* Existing clip path is not a rectangle.  Intersect the slow way. */
+	/* New clip path is nontrivial.  Intersect the slow way. */
+	gx_cpath_path_list *next = pcpath->path_list;
 	bool path_valid =
 	    gx_cpath_inner_box(pcpath, &old_box) &&
 	    gx_path_bbox(ppath, &new_box) >= 0 &&
@@ -556,16 +652,38 @@ gx_cpath_intersect(gx_clip_path *pcpath, /*const*/ gx_path *ppath_orig,
 					new_box.p.x, new_box.p.y,
 					new_box.q.x, new_box.q.y);
 
-	code = gx_cpath_intersect_path_slow(pcpath, ppath, rule, pis);
-	if (code >= 0 && path_valid) {
+	if (!path_valid && next == NULL) {
+	    code = gx_cpath_path_list_new(pcpath->path.memory, pcpath, pcpath->rule, 
+					    &pcpath->path, NULL, &next);
+	    if (code < 0)
+		goto ex;
+	}
+	code = gx_cpath_intersect_path_slow(pcpath, (params != NULL ? ppath_orig : ppath), 
+			    rule, pis, params);
+	if (code < 0)
+	    goto ex;
+	if (path_valid) {
 	    gx_path_assign_preserve(&pcpath->path, ppath_orig);
 	    pcpath->path_valid = true;
+	    pcpath->rule = rule;
+	} else {
+	    code = gx_cpath_path_list_new(pcpath->path.memory, NULL, rule, 
+					    ppath_orig, next, &pcpath->path_list);
 	}
     }
+ex:
     if (ppath != ppath_orig)
 	gx_path_free(ppath, "gx_cpath_clip");
     return code;
 }
+int
+gx_cpath_intersect(gx_clip_path *pcpath, /*const*/ gx_path *ppath_orig,
+		   int rule, gs_imager_state *pis)
+{
+    return gx_cpath_intersect_with_params(pcpath, ppath_orig,
+		   rule, pis, NULL);
+}
+
 
 /* Scale a clipping path by a power of 2. */
 int
@@ -604,8 +722,10 @@ gx_cpath_scale_exp2_shared(gx_clip_path * pcpath, int log2_scale_x,
 		SCALE_V(ymax, log2_scale_y);
 #undef SCALE_V
 	    }
+	list->xmin = arith_rshift(list->xmin, -log2_scale_x);
+	list->xmax = arith_rshift(list->xmax, -log2_scale_x);
     }
-    pcpath->id = gs_next_ids(1);	/* path changed => change id */
+    pcpath->id = gs_next_ids(pcpath->path.memory, 1);	/* path changed => change id */
     return 0;
 }
 
@@ -652,7 +772,7 @@ gx_clip_list_from_rectangle(register gx_clip_list * clp,
 
 /* Start enumerating a clipping path. */
 int
-gx_cpath_enum_init(gs_cpath_enum * penum, gx_clip_path * pcpath)
+gx_cpath_enum_init(gs_cpath_enum * penum, const gx_clip_path * pcpath)
 {
     if ((penum->using_path = pcpath->path_valid)) {
 	gx_path_enum_init(&penum->path_enum, &pcpath->path);
@@ -876,12 +996,76 @@ gx_clip_list_free(gx_clip_list * clp, gs_memory_t * mem)
     gx_clip_list_init(clp);
 }
 
+/* Check whether a rectangle has a non-empty intersection with a clipping patch. */
+bool 
+gx_cpath_rect_visible(gx_clip_path * pcpath, gs_int_rect *prect)
+{
+    const gx_clip_rect *pr;
+    const gx_clip_list *list = &pcpath->rect_list->list;
+
+    switch (list->count) {
+	case 0:
+	    return false;
+	case 1:
+	    pr = &list->single;
+	    break;
+	default:
+	    pr = list->head;
+    }
+    for (; pr != 0; pr = pr->next) {
+	if (pr->xmin > prect->q.x)
+	    continue;
+	if (pr->xmax < prect->p.x)
+	    continue;
+	if (pr->ymin > prect->q.y)
+	    continue;
+	if (pr->ymax < prect->p.y)
+	    continue;
+	return true;
+    }
+    return false;
+}
+
+int
+gx_cpath_copy(const gx_clip_path * from, gx_clip_path * pcpath)
+{   /* *pcpath must be initialized. */
+    gx_clip_rect *r, *s;
+    gx_clip_list *l = &pcpath->rect_list->list;
+
+    pcpath->path_valid = false;
+    /* NOTE: pcpath->path still contains the old path. */
+    if (pcpath->path_list)
+	rc_decrement(pcpath->path_list, "gx_cpath_copy");
+    pcpath->path_list = NULL;
+    pcpath->rule = from->rule;
+    pcpath->outer_box = from->outer_box;
+    pcpath->inner_box = from->inner_box;
+    l->single = from->rect_list->list.single;
+    for (r = from->rect_list->list.head; r != NULL; r = r->next) {
+	s = gs_alloc_struct(from->rect_list->rc.memory, gx_clip_rect, &st_clip_rect, "gx_cpath_copy");
+	if (s == NULL)
+	    return_error(gs_error_VMerror);
+	*s = *r;
+	s->next = NULL;
+	if (l->tail) {
+	    s->prev = l->tail;
+	    l->tail->next = s;
+	} else {
+	    l->head = s;
+	    s->prev = NULL;
+	}
+	l->tail = s;
+    }
+    l->count = from->rect_list->list.count;
+    return 0;
+}
+
 /* ------ Debugging printout ------ */
 
 #ifdef DEBUG
 
 /* Print a clipping list. */
-void
+private void
 gx_clip_list_print(const gx_clip_list *list)
 {
     const gx_clip_rect *pr;
