@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxdcolor.c 6651 2006-03-13 16:18:19Z stefan $ */
+/*$Id: gxdcolor.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Pure and null device color implementation */
 #include "gx.h"
 #include "memory_.h"
@@ -417,13 +417,18 @@ gx_dc_pure_fill_masked(const gx_device_color * pdevc, const byte * data,
 	gx_color_index scolors[2];
 	gx_color_index tcolors[2];
 
-	scolors[0] = gx_device_black(dev);
-	scolors[1] = gx_device_white(dev);
+	if ( lop != lop_default ) {
+	    scolors[0] = gx_device_white(dev);
+	    scolors[1] = gx_device_black(dev);
+	} else {
+	    scolors[0] = gx_device_black(dev);
+	    scolors[1] = gx_device_white(dev);
+        }
 	tcolors[0] = tcolors[1] = pdevc->colors.pure;
 	return (*dev_proc(dev, strip_copy_rop))
 	    (dev, data, data_x, raster, id, scolors,
 	     NULL, tcolors, x, y, w, h, 0, 0,
-	     (invert ? rop3_invert_S(lop) : lop) | lop_S_transparent);
+	     (invert ? rop3_invert_S(lop) : lop) | (rop3_S | lop_S_transparent));
     }
 }
 
