@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gximage.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gximage.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Generic image support */
 #include "memory_.h"
 #include "gx.h"
@@ -55,7 +55,7 @@ gs_data_image_t_init(gs_data_image_t * pim, int num_components)
 }
 void
 gs_pixel_image_t_init(gs_pixel_image_t * pim,
-		      const gs_color_space * color_space)
+		      gs_color_space * color_space)
 {
     int num_components;
 
@@ -85,6 +85,7 @@ gx_image_enum_common_init(gx_image_enum_common_t * piec,
     piec->procs = piep;
     piec->dev = dev;
     piec->id = gs_next_ids(dev->memory, 1);
+    piec->skipping = false;
     switch (format) {
 	case gs_image_format_chunky:
 	    piec->num_planes = 1;
@@ -224,7 +225,7 @@ gx_image_no_sput(const gs_image_common_t *pic, stream *s,
 
 int
 gx_image_no_sget(gs_image_common_t *pic, stream *s,
-		 const gs_color_space *pcs)
+		 gs_color_space *pcs)
 {
     return_error(gs_error_rangecheck);
 }
@@ -438,7 +439,7 @@ sget_variable_uint(stream *s, uint *pw)
  */
 int
 gx_pixel_image_sget(gs_pixel_image_t *pim, stream *s,
-		    const gs_color_space *pcs)
+		    gs_color_space *pcs)
 {
     uint control;
     float decode_default_1 = 1;
