@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gximage1.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gximage1.c 8022 2007-06-05 22:23:38Z giles $ */
 /* ImageType 1 initialization */
 #include "gx.h"
 #include "gserrors.h"
@@ -47,7 +47,7 @@ const gx_image_type_t gs_image_type_mask1 = {
 
 /* Define the procedures for initializing gs_image_ts to default values. */
 void
-gs_image_t_init_adjust(gs_image_t * pim, const gs_color_space * color_space,
+gs_image_t_init_adjust(gs_image_t * pim, gs_color_space * color_space,
 		       bool adjust)
 {
     gs_pixel_image_t_init((gs_pixel_image_t *) pim, color_space);
@@ -111,7 +111,7 @@ gx_image1_sput(const gs_image_common_t *pic, stream *s,
 
 private int
 gx_image1_sget(gs_image_common_t *pic, stream *s,
-	       const gs_color_space *pcs)
+	       gs_color_space *pcs)
 {
     gs_image1_t *const pim = (gs_image1_t *)pic;
     int code = gx_pixel_image_sget((gs_pixel_image_t *)pim, s, pcs);
@@ -171,7 +171,7 @@ gx_image1_mask_sput(const gs_image_common_t *pic, stream *s,
 
 private int
 gx_image1_mask_sget(gs_image_common_t *pic, stream *s,
-		    const gs_color_space *ignore_pcs)
+		    gs_color_space *ignore_pcs)
 {
     gs_image1_t *const pim = (gs_image1_t *)pic;
     int code;
