@@ -1,22 +1,17 @@
-/* Copyright (C) 1997, 1998, 1999, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gximage3.c,v 1.6.6.1.2.1 2003/01/17 00:49:04 giles Exp $ */
+/* $Id: gximage3.c 8022 2007-06-05 22:23:38Z giles $ */
 /* ImageType 3 image implementation */
 #include "math_.h"		/* for ceil, floor */
 #include "memory_.h"
@@ -53,7 +48,7 @@ private const gx_image_enum_procs_t image3_enum_procs = {
 
 /* Initialize an ImageType 3 image. */
 void
-gs_image3_t_init(gs_image3_t * pim, const gs_color_space * color_space,
+gs_image3_t_init(gs_image3_t * pim, gs_color_space * color_space,
 		 gs_image3_interleave_type_t interleave_type)
 {
     gs_pixel_image_t_init((gs_pixel_image_t *) pim, color_space);
@@ -112,6 +107,7 @@ make_mid_default(gx_device **pmidev, gx_device *dev, int width, int height,
     midev->bitmap_memory = mem;
     midev->width = width;
     midev->height = height;
+    check_device_separable((gx_device *)midev);
     gx_device_fill_in_procs((gx_device *)midev);
     code = dev_proc(midev, open_device)((gx_device *)midev);
     if (code < 0) {
@@ -183,7 +179,7 @@ gx_begin_image3(gx_device * dev,
  * Begin a generic ImageType 3 image, with client handling the creation of
  * the mask image and mask clip devices.
  */
-private bool check_image3_extent(P2(floatp mask_coeff, floatp data_coeff));
+private bool check_image3_extent(floatp mask_coeff, floatp data_coeff);
 int
 gx_begin_image3_generic(gx_device * dev,
 			const gs_imager_state *pis, const gs_matrix *pmat,
@@ -247,7 +243,14 @@ gx_begin_image3_generic(gx_device * dev,
 	fabs(mi_pixel.ty - mi_mask.ty) >= 0.5
 	)
 	return_error(gs_error_rangecheck);
+#ifdef DEBUG
     {
+	/* Although the PLRM says that the Mask and Image *must* be the same size,  */
+	/* Adobe CPSI (and other RIPS) ignore this and process anyway. Note that we */
+	/* are not compatible if the Mask Height than the Data (pixel) Height. CPSI */
+	/* de-interleaves the mask from the data image and stops at the Mask Height */
+	/* Problem detected with Genoa 468-03 (part of file 468-01.ps)              */
+	/*****           fixme: When Data Image Height > Mask Height            *****/
 	gs_point ep, em;
 
 	if ((code = gs_point_transform(pim->Width, pim->Height, &mi_pixel,
@@ -258,8 +261,9 @@ gx_begin_image3_generic(gx_device * dev,
 	    )
 	    return code;
 	if (fabs(ep.x - em.x) >= 0.5 || fabs(ep.y - em.y) >= 0.5)
-	    return_error(gs_error_rangecheck);
+	    code = gs_error_rangecheck;	/* leave the check in for debug breakpoint */
     }
+#endif /* DEBUG */
     penum = gs_alloc_struct(mem, gx_image3_enum_t, &st_image3_enum,
 			    "gx_begin_image3");
     if (penum == 0)
@@ -330,8 +334,9 @@ gx_begin_image3_generic(gx_device * dev,
 	(code = gs_bbox_transform(&mrect, &mat, &mrect)) < 0
 	)
 	return code;
-    origin.x = floor(mrect.p.x);
-    origin.y = floor(mrect.p.y);
+
+    origin.x = (mrect.p.x < 0) ? (int)ceil(mrect.p.x) : (int)floor(mrect.p.x);
+    origin.y = (mrect.p.y < 0) ? (int)ceil(mrect.p.y) : (int)floor(mrect.p.y);
     code = make_mid(&mdev, dev, (int)ceil(mrect.q.x) - origin.x,
 		    (int)ceil(mrect.q.y) - origin.y, mem);
     if (code < 0)
@@ -350,7 +355,7 @@ gx_begin_image3_generic(gx_device * dev,
 	gx_drawing_color dcolor;
 	gs_matrix m_mat;
 
-	color_set_pure(&dcolor, 1);
+	set_nonclient_dev_color(&dcolor, 1);
 	/*
 	 * Adjust the translation for rendering the mask to include a
 	 * negative translation by origin.{x,y} in device space.
@@ -725,9 +730,9 @@ gx_image3_end_image(gx_image_enum_common_t * info, bool draw_last)
     int mcode = gx_image_end(penum->mask_info, draw_last);
     gx_device *pcdev = penum->pcdev;
     int pcode = gx_image_end(penum->pixel_info, draw_last);
+    int code1 = gs_closedevice(pcdev);
+    int code2 = gs_closedevice(mdev);
 
-    gs_closedevice(pcdev);
-    gs_closedevice(mdev);
     gs_free_object(mem, penum->mask_data,
 		   "gx_image3_end_image(mask_data)");
     gs_free_object(mem, penum->pixel_data,
@@ -735,5 +740,5 @@ gx_image3_end_image(gx_image_enum_common_t * info, bool draw_last)
     gs_free_object(mem, pcdev, "gx_image3_end_image(pcdev)");
     gs_free_object(mem, mdev, "gx_image3_end_image(mdev)");
     gs_free_object(mem, penum, "gx_image3_end_image");
-    return (pcode < 0 ? pcode : mcode);
+    return (pcode < 0 ? pcode : mcode < 0 ? mcode : code1 < 0 ? code1 : code2);
 }
