@@ -1,22 +1,17 @@
-/* Copyright (C) 1998 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxpageq.c,v 1.2.6.1.2.1 2003/01/17 00:49:04 giles Exp $ */
+/* $Id: gxpageq.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Page queue implementation */
 
 /* Initial version 2/1/98 by John Desrosiers (soho@crl.com) */
@@ -60,9 +55,9 @@ private_st_gx_page_queue();
 
 /* ------------ Forward Decl's --------------------------- */
 private gx_page_queue_entry_t *	/* removed entry, 0 if none avail */
-    gx_page_queue_remove_first(P1(
-				  gx_page_queue_t * queue	/* page queue to retrieve from */
-				  ));
+    gx_page_queue_remove_first(
+			       gx_page_queue_t * queue	/* page queue to retrieve from */
+			       );
 
 
 /* --------------------Procedures------------------------- */
@@ -277,7 +272,7 @@ gx_page_queue_enqueue(
 /* Add page to a page queue */
 /* Even if an error is returned, entry will have been added to queue! */
 int				/* rets 0 ok, gs_error_VMerror if error */
-gx_page_queue_add_page(
+gx_page_queue_add_page(gx_device_clist_writer *const pcwdev,
 			  gx_page_queue_t * queue,	/* page queue to add to */
 			  gx_page_queue_action_t action,	/* action code to queue */
 			  const gx_band_page_info_t * page_info,  /* bandinfo incl. bandlist (or 0) */
@@ -301,8 +296,10 @@ gx_page_queue_add_page(
     entry->action = action;
     if (page_info != 0)
 	entry->page_info = *page_info;
-    else
+    else {
 	entry->page_info = null_page_info;
+	entry->page_info.io_procs = pcwdev->page_info.io_procs;
+    }
     entry->num_copies = page_count;
 
     /* Stick onto page queue & signal */
