@@ -1,22 +1,17 @@
-/* Copyright (C) 2001 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: sarc4.c,v 1.5.2.2.2.1 2003/01/17 00:49:05 giles Exp $ */
+/* $Id: sarc4.c 8022 2007-06-05 22:23:38Z giles $ */
 
 /* Arcfour cipher and filter implementation */
 
@@ -115,3 +110,19 @@ s_arcfour_process(stream_state * ss, stream_cursor_read * pr,
 const stream_template s_arcfour_template = {
     &st_arcfour_state, NULL, s_arcfour_process, 1, 1
 };
+
+/* (de)crypt a section of text in a buffer -- the procedure is the same
+ * in each direction. see strimpl.h for return codes.
+ */
+int
+s_arcfour_process_buffer(stream_arcfour_state *ss, byte *buf, int buf_size)
+{
+    stream_cursor_read r;
+    stream_cursor_write w;
+    const bool unused = false;
+
+    r.ptr = w.ptr = buf - 1;
+    r.limit = w.limit = buf - 1 + buf_size;
+    return s_arcfour_process((stream_state *)ss, &r, &w, unused);
+}
+
