@@ -1,22 +1,17 @@
-/* Copyright (C) 1993, 1995, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: zdevice2.c,v 1.4.2.2.2.1 2003/01/17 00:49:05 giles Exp $ */
+/* $Id: zdevice2.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Level 2 device operators */
 #include "math_.h"
 #include "memory_.h"
@@ -33,9 +28,12 @@
 #include "gxdevice.h"
 #include "gsstate.h"
 
+/* Exported for zfunc4.c */
+int z2copy(i_ctx_t *);
+
 /* Forward references */
-private int z2copy_gstate(P1(i_ctx_t *));
-private int push_callout(P2(i_ctx_t *, const char *));
+private int z2copy_gstate(i_ctx_t *);
+private int push_callout(i_ctx_t *, const char *);
 
 /* Extend the `copy' operator to deal with gstates. */
 /* This is done with a hack -- we know that gstates are the only */
@@ -256,7 +254,7 @@ restore_page_device(const gs_state * pgs_old, const gs_state * pgs_new)
     gx_device *dev_new;
     gx_device *dev_t1;
     gx_device *dev_t2;
-    bool samepagedevice = obj_eq(&gs_int_gstate(pgs_old)->pagedevice,
+    bool samepagedevice = obj_eq(dev_old->memory, &gs_int_gstate(pgs_old)->pagedevice,
     	&gs_int_gstate(pgs_new)->pagedevice);
 
     if ((dev_t1 = (*dev_proc(dev_old, get_page_device)) (dev_old)) == 0)
@@ -369,7 +367,7 @@ push_callout(i_ctx_t *i_ctx_p, const char *callout_name)
     int code;
 
     check_estack(1);
-    code = name_enter_string(callout_name, esp + 1);
+    code = name_enter_string(imemory, callout_name, esp + 1);
     if (code < 0)
 	return code;
     ++esp;
