@@ -1,22 +1,17 @@
-/* Copyright (C) 1989, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: zdict.c,v 1.3.6.1.2.1 2003/01/17 00:49:05 giles Exp $ */
+/* $Id: zdict.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Dictionary operators */
 #include "ghost.h"
 #include "oper.h"
@@ -64,8 +59,18 @@ zbegin(i_ctx_t *i_ctx_p)
 
     check_type(*op, t_dictionary);
     check_dict_read(*op);
-    if (dsp == dstop)
-	return_error(e_dictstackoverflow);
+    if ( dsp == dstop ) { 
+        int code = ref_stack_extend(&d_stack, 1);
+
+        if ( code < 0 ) {
+            if (code == e_dictstackoverflow) {
+                /* Adobe doesn't restore the operand that caused stack */
+                /* overflow. We do the same to match CET 20-02-02      */
+                pop(1);
+            }
+            return code;
+        }
+    }
     ++dsp;
     ref_assign(dsp, op);
     dict_set_top();
@@ -106,7 +111,7 @@ zop_def(i_ctx_t *i_ctx_p)
     switch (r_type(op1)) {
 	case t_name: {
 	    /* We can use the fast single-probe lookup here. */
-	    uint nidx = name_index(op1);
+	    uint nidx = name_index(imemory, op1);
 	    uint htemp;
 
 	    if_dict_find_name_by_index_top(nidx, htemp, pvslot) {
@@ -202,10 +207,15 @@ private int
 zundef(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
+    os_ptr op1 = op - 1;
+    int code;
 
-    check_type(op[-1], t_dictionary);
-    check_dict_write(op[-1]);
-    idict_undef(op - 1, op);	/* ignore undefined error */
+    check_type(*op1, t_dictionary);
+    if (i_ctx_p->in_superexec == 0)
+	check_dict_write(*op1);
+    code = idict_undef(op1, op);
+    if (code < 0 && code != e_undefined) /* ignore undefined error */
+	return code;
     pop(2);
     return 0;
 }
@@ -217,10 +227,20 @@ zknown(i_ctx_t *i_ctx_p)
     os_ptr op = osp;
     register os_ptr op1 = op - 1;
     ref *pvalue;
+    int code;
 
     check_type(*op1, t_dictionary);
     check_dict_read(*op1);
-    make_bool(op1, (dict_find(op1, op, &pvalue) > 0 ? 1 : 0));
+    code = dict_find(op1, op, &pvalue);
+    switch (code) {
+    case e_dictfull:
+	code = 0;
+    case 0: case 1:
+	break;
+    default:
+	return code;
+    }
+    make_bool(op1, code);
     pop(1);
     return 0;
 }
@@ -239,10 +259,14 @@ zwhere(i_ctx_t *i_ctx_p)
 	const ref *const bot = rsenum.ptr;
 	const ref *pdref = bot + rsenum.size;
 	ref *pvalue;
+	int code;
 
 	while (pdref-- > bot) {
 	    check_dict_read(*pdref);
-	    if (dict_find(pdref, op, &pvalue) > 0) {
+	    code = dict_find(pdref, op, &pvalue);
+	    if (code < 0 && code != e_dictfull)
+		return code;
+	    if (code > 0) {
 		push(1);
 		ref_assign(op - 1, pdref);
 		make_true(op);
@@ -266,7 +290,7 @@ zcopy_dict(i_ctx_t *i_ctx_p)
     check_type(*op1, t_dictionary);
     check_dict_read(*op1);
     check_dict_write(*op);
-    if (!dict_auto_expand &&
+    if (!imemory->gs_lib_ctx->dict_auto_expand &&
 	(dict_length(op) != 0 || dict_maxlength(op) < dict_length(op1))
 	)
 	return_error(e_rangecheck);
@@ -317,9 +341,14 @@ zdictstack(i_ctx_t *i_ctx_p)
     os_ptr op = osp;
     uint count = ref_stack_count(&d_stack);
 
-    check_write_type(*op, t_array);
     if (!level2_enabled)
 	count--;		/* see dstack.h */
+    if (!r_is_array(op))
+	return_op_typecheck(op);
+    if (r_size(op) < count)
+        return_error(e_rangecheck);
+    if (!r_has_type_attrs(op, t_array, a_write)) 
+        return_error(e_invalidaccess);
     return ref_stack_store(&d_stack, op, count, 0, 0, true, idmemory,
 			   "dictstack");
 }
@@ -335,29 +364,6 @@ zcleardictstack(i_ctx_t *i_ctx_p)
 
 /* ------ Extensions ------ */
 
-/* <dict1> <dict2> .dictcopynew <dict2> */
-private int
-zdictcopynew(i_ctx_t *i_ctx_p)
-{
-    os_ptr op = osp;
-    os_ptr op1 = op - 1;
-    int code;
-
-    check_type(*op1, t_dictionary);
-    check_dict_read(*op1);
-    check_type(*op, t_dictionary);
-    check_dict_write(*op);
-    /* This is only recognized in Level 2 mode. */
-    if (!dict_auto_expand)
-	return_error(e_undefined);
-    code = idict_copy_new(op1, op);
-    if (code < 0)
-	return code;
-    ref_assign(op1, op);
-    pop(1);
-    return 0;
-}
-
 /* -mark- <key0> <value0> <key1> <value1> ... .dicttomark <dict> */
 /* This is the Level 2 >> operator. */
 private int
@@ -391,6 +397,35 @@ zdicttomark(i_ctx_t *i_ctx_p)
     return code;
 }
 
+/* <dict1> <dict2> .forcecopynew <dict2> */
+/*
+ * This operator is a special-purpose accelerator for use by 'restore' (see
+ * gs_dps1.ps).  Note that this operator does *not* require that dict2 be
+ * writable.  Hence it is in the same category of "dangerous" operators as
+ * .forceput and .forceundef.
+ */
+private int
+zforcecopynew(i_ctx_t *i_ctx_p)
+{
+    os_ptr op = osp;
+    os_ptr op1 = op - 1;
+    int code;
+
+    check_type(*op1, t_dictionary);
+    check_dict_read(*op1);
+    check_type(*op, t_dictionary);
+    /*check_dict_write(*op);*/	/* see above */
+    /* This is only recognized in Level 2 mode. */
+    if (!imemory->gs_lib_ctx->dict_auto_expand)
+	return_error(e_undefined);
+    code = idict_copy_new(op1, op);
+    if (code < 0)
+	return code;
+    ref_assign(op1, op);
+    pop(1);
+    return 0;
+}
+
 /* <dict> <key> .forceundef - */
 /*
  * This forces an "undef" even if the dictionary is not writable.
@@ -456,7 +491,8 @@ zsetmaxlength(i_ctx_t *i_ctx_p)
     int code;
 
     check_type(*op1, t_dictionary);
-    check_dict_write(*op1);
+    if (i_ctx_p->in_superexec == 0)
+	check_dict_write(*op1);
     check_type(*op, t_integer);
 #if arch_sizeof_int < arch_sizeof_long
     check_int_leu(*op, max_uint);
@@ -494,11 +530,21 @@ const op_def zdict1_op_defs[] = {
 };
 const op_def zdict2_op_defs[] = {
 		/* Extensions */
-    {"2.dictcopynew", zdictcopynew},
     {"1.dicttomark", zdicttomark},
+    {"2.forcecopynew", zforcecopynew},
     {"2.forceundef", zforceundef},
     {"2.knownget", zknownget},
     {"1.knownundef", zknownundef},
     {"2.setmaxlength", zsetmaxlength},
+	/*
+	 * In Level 2, >> is a synonym for .dicttomark, and undef for
+	 * .undef.  By giving the former their own entries, they will not be
+	 * "eq" to .dicttomark and .undef, but that doesn't matter, since
+	 * we're doing this only for the sake of Adobe- compatible error
+	 * stacks.
+	 */
+    op_def_begin_level2(),
+    {"1>>", zdicttomark},
+    {"2undef", zundef},
     op_def_end(0)
 };
