@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zfcid1.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: zfcid1.c 8128 2007-07-16 16:58:11Z alexcher $ */
 /* CIDFontType 1 and 2 operators */
 #include "memory_.h"
 #include "ghost.h"
@@ -95,7 +95,7 @@ z11_CIDMap_proc(gs_font_cid2 *pfont, gs_glyph glyph)
 	return prgnum->value.intval;
     default:			/* array type */
         code = string_array_access_proc(pfont->memory, pcidmap, 1, cid * gdbytes,
-					gdbytes, &data);
+					gdbytes, NULL, NULL, &data);
 
 	if (code < 0)
 	    return code;
@@ -104,6 +104,8 @@ z11_CIDMap_proc(gs_font_cid2 *pfont, gs_glyph glyph)
     }
     for (i = 0; i < gdbytes; ++i)
 	gnum = (gnum << 8) + data[i];
+    if (gnum >= pfont->data.trueNumGlyphs)
+	return_error(e_invalidfont);
     return gnum;
 }
 
@@ -133,8 +135,8 @@ z11_get_outline(gs_font_type42 * pfont, uint glyph_index,
 #define GET_S16_MSB(p) (int)((GET_U16_MSB(p) ^ 0x8000) - 0x8000)
 
 private int
-z11_get_metrics(gs_font_type42 * pfont, uint glyph_index, int wmode,
-		float sbw[4])
+z11_get_metrics(gs_font_type42 * pfont, uint glyph_index, 
+		gs_type42_metrics_options_t options, float sbw[4])
 {
     gs_font_cid2 *const pfcid = (gs_font_cid2 *)pfont;
     int skip = pfcid->cidata.MetricsCount << 1;
@@ -142,26 +144,34 @@ z11_get_metrics(gs_font_type42 * pfont, uint glyph_index, int wmode,
     const byte *pmetrics;
     int lsb, width;
     int code = 0;
+    int wmode = gs_type42_metrics_options_wmode(options);
 
     gdata.memory = pfont->memory;
     if (wmode >= skip >> 2 ||
 	(code = pfcid->cidata.orig_procs.get_outline(pfont, glyph_index, &gdata)) < 0 ||
 	gdata.bits.size < skip
 	)
-	return pfcid->cidata.orig_procs.get_metrics(pfont, glyph_index, wmode,
-						    sbw);
-    pmetrics = gdata.bits.data + skip - 4 - (wmode << 2);
-    lsb = GET_S16_MSB(pmetrics + 2);
-    width = GET_U16_MSB(pmetrics + 0);
-    {
-	double factor = 1.0 / pfont->data.unitsPerEm;
+	return pfcid->cidata.orig_procs.get_metrics(pfont, glyph_index, options, sbw);
+    if(gs_type42_metrics_options_bbox_requested(options)) {
+	code = pfcid->cidata.orig_procs.get_metrics(pfont, glyph_index, 
+			gs_type42_metrics_options_BBOX, sbw);;
+	if (code < 0)
+	    return code;
+    }
+    if (gs_type42_metrics_options_sbw_requested(options)) {
+	pmetrics = gdata.bits.data + skip - 4 - (wmode << 2);
+	lsb = GET_S16_MSB(pmetrics + 2);
+	width = GET_U16_MSB(pmetrics + 0);
+	{
+	    double factor = 1.0 / pfont->data.unitsPerEm;
 
-	if (wmode) {
-	    sbw[0] = 0, sbw[1] = -lsb * factor;
-	    sbw[2] = 0, sbw[3] = -width * factor;
-	} else {
-	    sbw[0] = lsb * factor, sbw[1] = 0;
-	    sbw[2] = width * factor, sbw[3] = 0;
+	    if (wmode) {
+		sbw[0] = 0, sbw[1] = -lsb * factor;
+		sbw[2] = 0, sbw[3] = -width * factor;
+	    } else {
+		sbw[0] = lsb * factor, sbw[1] = 0;
+		sbw[2] = width * factor, sbw[3] = 0;
+	    }
 	}
     }
     gs_glyph_data_free(&gdata, "z11_get_metrics");
@@ -269,7 +279,7 @@ zbuildfont11(i_ctx_t *i_ctx_p)
     if (MetricsCount & 1)	/* only allowable values are 0, 2, 4 */
 	return_error(e_rangecheck);
     code = dict_find_string(op, "File", &pfile);
-    if (code < 0 && code != e_dictfull)
+    if (code < 0 && code != e_undefined)
 	return code;
     if (code > 0) {
 	ref *file_table_pos, *a, v;
