@@ -1,29 +1,24 @@
-/* Copyright (C) 1994, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: zfdctd.c,v 1.6.2.1.2.1 2003/01/17 00:49:06 giles Exp $ */
+/* $Id: zfdctd.c 8022 2007-06-05 22:23:38Z giles $ */
 /* DCTDecode filter creation */
 #include "memory_.h"
 #include "stdio_.h"		/* for jpeglib.h */
 #include "jpeglib_.h"
 #include "ghost.h"
 #include "oper.h"
-#include "gsmalloc.h"		/* for gs_memory_default */
+#include "gsmemory.h"
 #include "strimpl.h"
 #include "sdct.h"
 #include "sjpeg.h"
@@ -36,13 +31,27 @@ private_st_jpeg_decompress_data();
 /* Import the parameter processing procedure from sddparam.c */
 stream_state_proc_put_params(s_DCTD_put_params, stream_DCT_state);
 
+/* Find the memory that will be used for allocating the stream. */
+private gs_ref_memory_t *
+find_stream_memory(i_ctx_t *i_ctx_p, int npop, uint space)
+{
+    uint use_space = max(space, avm_global);
+    os_ptr sop = osp - npop;
+
+    if (r_has_type(sop, t_dictionary)) {
+	--sop;
+    }
+    use_space = max(use_space, r_space(sop));
+    return idmemory->spaces_indexed[use_space >> r_space_shift];
+}
+
 /* <source> <dict> DCTDecode/filter <file> */
 /* <source> DCTDecode/filter <file> */
 private int
 zDCTD(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
-    gs_memory_t *mem = (gs_memory_t *)(i_ctx_p->memory.current);
+    gs_memory_t *mem;
     stream_DCT_state state;
     dict_param_list list;
     jpeg_decompress_data *jddp;
@@ -50,6 +59,11 @@ zDCTD(i_ctx_t *i_ctx_p)
     const ref *dop;
     uint dspace;
 
+    if (r_has_type(op, t_dictionary))
+	dop = op, dspace = r_space(op);
+    else
+	dop = 0, dspace = 0;
+    mem = (gs_memory_t *)find_stream_memory(i_ctx_p, 0, dspace);
     /* First allocate space for IJG parameters. */
     jddp = gs_alloc_struct_immovable(mem,jpeg_decompress_data,
       &st_jpeg_decompress_data, "zDCTD");
@@ -64,10 +78,6 @@ zDCTD(i_ctx_t *i_ctx_p)
     if ((code = gs_jpeg_create_decompress(&state)) < 0)
 	goto fail;		/* correct to do jpeg_destroy here */
     /* Read parameters from dictionary */
-    if (r_has_type(op, t_dictionary))
-	dop = op, dspace = r_space(op);
-    else
-	dop = 0, dspace = 0;
     if ((code = dict_param_list_read(&list, dop, NULL, false, iimemory)) < 0)
 	goto fail;
     if ((code = s_DCTD_put_params((gs_param_list *) & list, &state)) < 0)
