@@ -1,22 +1,17 @@
-/* Copyright (C) 1999, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: zfunc4.c,v 1.7.4.2.2.1 2003/01/17 00:49:06 giles Exp $ */
+/* $Id: zfunc4.c 8022 2007-06-05 22:23:38Z giles $ */
 /* PostScript language support for FunctionType 4 (PS Calculator) Functions */
 #include "memory_.h"
 #include "ghost.h"
@@ -29,7 +24,10 @@
 #include "ifunc.h"
 #include "iname.h"
 #include "dstack.h"
-
+#include "ialloc.h"
+#include "gzstate.h"	    /* these are needed to check if device is pdfwrite */
+#include "gxdevcli.h"	    /* these are needed to check if device is pdfwrite */
+#include "string_.h"	    /* these are needed to check if device is pdfwrite */
 /*
  * FunctionType 4 functions are not defined in the PostScript language.  We
  * provide support for them because they are needed for PDF 1.3.  In
@@ -43,7 +41,7 @@
  *
  * The following list is taken directly from the PDF 1.3 documentation.
  */
-#define XOP(zfn) int zfn(P1(i_ctx_t *))
+#define XOP(zfn) int zfn(i_ctx_t *)
 XOP(zabs); XOP(zand); XOP(zatan); XOP(zbitshift);
 XOP(zceiling); XOP(zcos); XOP(zcvi); XOP(zcvr);
 XOP(zdiv); XOP(zexp); XOP(zfloor); XOP(zidiv);
@@ -125,17 +123,38 @@ psc_fixup(byte *p, byte *to)
     p[2] = (byte)skip;
 }
 
+/* Check whether the ref is a given operator or resolves to it */
+private bool
+resolves_to_oper(i_ctx_t *i_ctx_p, const ref *pref, const op_proc_t proc)
+{
+    if (!r_has_attr(pref, a_executable))
+        return false;
+    if (r_btype(pref) == t_operator) {
+        return pref->value.opproc == proc;
+    } else if (r_btype(pref) == t_name) {
+        ref * val;
+        if (dict_find(systemdict, pref, &val) <= 0)
+	    return false;
+        if (r_btype(val) != t_operator)
+	    return false;
+        if (!r_has_attr(val, a_executable))
+	    return false;
+        return val->value.opproc == proc;
+    }
+   else
+      return false;
+}
+
 /*
  * Check a calculator function for validity, optionally storing its encoded
  * representation and add the size of the encoded representation to *psize.
  * Note that we arbitrarily limit the depth of procedure nesting.  pref is
  * known to be a procedure.
  */
-#define MAX_PSC_FUNCTION_NESTING 10
 private int
 check_psc_function(i_ctx_t *i_ctx_p, const ref *pref, int depth, byte *ops, int *psize)
 {
-    long i;
+    uint i;
     uint size = r_size(pref);
 
     for (i = 0; i < size; ++i) {
@@ -145,7 +164,7 @@ check_psc_function(i_ctx_t *i_ctx_p, const ref *pref, int depth, byte *ops, int
 	ref * delp;
 	int code;
 
-	array_get(pref, i, &elt);
+	array_get(imemory, pref, i, &elt);
 	switch (r_btype(&elt)) {
 	case t_integer: {
 	    int i = elt.value.intval;
@@ -180,7 +199,7 @@ check_psc_function(i_ctx_t *i_ctx_p, const ref *pref, int depth, byte *ops, int
 	case t_name:
 	    if (!r_has_attr(&elt, a_executable))
 		return_error(e_rangecheck);
-	    name_string_ref(&elt, &elt);
+	    name_string_ref(imemory, &elt, &elt);
 	    if (!bytes_compare(elt.value.bytes, r_size(&elt),
 			       (const byte *)"true", 4)) {
 		*p = PtCr_true;
@@ -218,26 +237,34 @@ check_psc_function(i_ctx_t *i_ctx_p, const ref *pref, int depth, byte *ops, int
 		return_error(e_typecheck);
 	    if (depth == MAX_PSC_FUNCTION_NESTING)
 		return_error(e_limitcheck);
-	    if ((code = array_get(pref, ++i, &elt2)) < 0)
+	    if ((code = array_get(imemory, pref, ++i, &elt2)) < 0)
 		return code;
 	    *psize += 3;
 	    code = check_psc_function(i_ctx_p, &elt, depth + 1, ops, psize);
 	    if (code < 0)
 		return code;
-	    /* Check for {proc} if | {proc1} {proc2} ifelse */
-#define R_IS_OPER(pref, proc)\
-  (r_btype(pref) == t_operator && r_has_attr(pref, a_executable) &&\
-   (pref)->value.opproc == proc)
-	    if (R_IS_OPER(&elt2, zif)) {
+	    /* Check for { proc } repeat | {proc} if | {proc1} {proc2} ifelse */
+	    if (resolves_to_oper(i_ctx_p, &elt2, zrepeat)) {
+		/* We can't handle 'repeat' with pdfwrite since it emits FunctionType 4 */
+		if (strcmp(i_ctx_p->pgs->device->dname, "pdfwrite") == 0)
+		    return_error(e_rangecheck);
+		if (ops) {
+		    *p = PtCr_repeat;
+		    psc_fixup(p, ops + *psize);
+		    p = ops + *psize;
+		    *p++ = PtCr_repeat_end;
+		}
+		*psize += 1;	/* extra room for repeat_end */
+	    } else if (resolves_to_oper(i_ctx_p, &elt2, zif)) {
 		if (ops) {
 		    *p = PtCr_if;
 		    psc_fixup(p, ops + *psize);
 		}
 	    } else if (!r_is_proc(&elt2))
 		return_error(e_rangecheck);
-	    else if ((code == array_get(pref, ++i, &elt3)) < 0)
+    	    else if ((code == array_get(imemory, pref, ++i, &elt3)) < 0)
 		return code;
-	    else if (R_IS_OPER(&elt3, zifelse)) {
+	    else if (resolves_to_oper(i_ctx_p, &elt3, zifelse)) {
 		if (ops) {
 		    *p = PtCr_if;
 		    psc_fixup(p, ops + *psize + 3);
@@ -250,10 +277,9 @@ check_psc_function(i_ctx_t *i_ctx_p, const ref *pref, int depth, byte *ops, int
 		    return code;
 		if (ops)
 		    psc_fixup(p, ops + *psize);
-	    } else
+	    } else 
 		return_error(e_rangecheck);
-#undef R_IS_OPER
-	}
+	    }	 /* end 'default' */
 	}
     next:
 	DO_NOTHING;
