@@ -1,22 +1,17 @@
-/* Copyright (C) 2001 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zicc.c,v 1.2.2.2.2.1 2003/01/17 00:49:06 giles Exp $ */
+/* $Id: zicc.c 8022 2007-06-05 22:23:38Z giles $ */
 /* ICCBased color operators */
 
 #include "math_.h"
@@ -34,7 +29,7 @@
 #include "idparam.h"
 #include "igstate.h"
 #include "icie.h"
-
+#include "ialloc.h"
 
 /*
  *   <dict>  .seticcspace  -
@@ -70,7 +65,7 @@ zseticcspace(i_ctx_t * i_ctx_p)
     int edepth = ref_stack_count(&e_stack);
     int                     code;
     gs_color_space *        pcs;
-    const gs_color_space *  palt_cs;
+    gs_color_space *  palt_cs;
     ref *                   pnval;
     ref *                   pstrmval;
     stream *                s;
@@ -113,7 +108,8 @@ zseticcspace(i_ctx_t * i_ctx_p)
      * space, we use the range values only to restrict the set of input
      * values; they are not used for normalization.
      */
-    code = dict_floats_param( op,
+    code = dict_floats_param( imemory, 
+			      op,
                               "Range",
                               2 * ncomps,
                               range_buff,
@@ -138,9 +134,8 @@ zseticcspace(i_ctx_t * i_ctx_p)
     }
 
     /* record the current space as the alternative color space */
-    memmove( &pcs->params.icc.alt_space,
-             palt_cs,
-             sizeof(pcs->params.icc.alt_space) );
+    pcs->base_space = palt_cs;
+    rc_increment(palt_cs);
 
     code = gx_load_icc_profile(picc_info);
     if (code < 0)
