@@ -1,22 +1,17 @@
-/* Copyright (C) 1997, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: zimage3.c,v 1.3.6.1.2.1 2003/01/17 00:49:06 giles Exp $ */
+/* $Id: zimage3.c 8022 2007-06-05 22:23:38Z giles $ */
 /* LanguageLevel 3 ImageTypes (3 & 4 - masked images) */
 #include "memory_.h"
 #include "ghost.h"
@@ -30,7 +25,7 @@
 #include "idparam.h"
 #include "igstate.h"
 #include "iimage.h"
-#include "iimage2.h"
+#include "ialloc.h"
 
 /* <dict> .image3 - */
 private int
@@ -57,9 +52,10 @@ zimage3(i_ctx_t *i_ctx_p)
 	)
 	return_error(e_rangecheck);
     if ((code = pixel_image_params(i_ctx_p, pDataDict,
-				   (gs_pixel_image_t *)&image, &ip_data,
-				   12)) < 0 ||
-	(mcode = code = data_image_params(pMaskDict, &image.MaskDict, &ip_mask, false, 1, 12)) < 0 ||
+			(gs_pixel_image_t *)&image, &ip_data,
+			12, false, gs_currentcolorspace(igs))) < 0 ||
+	(mcode = code = data_image_params(imemory, pMaskDict, &image.MaskDict,
+				   &ip_mask, false, 1, 12, false)) < 0 ||
 	(code = dict_int_param(pDataDict, "ImageType", 1, 1, 0, &ignored)) < 0 ||
 	(code = dict_int_param(pMaskDict, "ImageType", 1, 1, 0, &ignored)) < 0
 	)
@@ -99,11 +95,11 @@ zimage4(i_ctx_t *i_ctx_p)
 
     gs_image4_t_init(&image, NULL);
     code = pixel_image_params(i_ctx_p, op, (gs_pixel_image_t *)&image, &ip,
-			      12);
+			      12, false, gs_currentcolorspace(igs));
     if (code < 0)
 	return code;
-    code = dict_int_array_check_param(op, "MaskColor", num_components * 2,
-				      colors, 0, e_rangecheck);
+    code = dict_int_array_check_param(imemory, op, "MaskColor",
+       num_components * 2, colors, 0, e_rangecheck);
     /* Clamp the color values to the unsigned range. */
     if (code == num_components) {
 	image.MaskColor_is_range = false;
