@@ -1,22 +1,17 @@
-/* Copyright (C) 1989, 1995, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: zpath1.c,v 1.2.6.1.2.1 2003/01/17 00:49:06 giles Exp $ */
+/* $Id: zpath1.c 8022 2007-06-05 22:23:38Z giles $ */
 /* PostScript Level 1 additional path operators */
 #include "memory_.h"
 #include "ghost.h"
@@ -30,9 +25,9 @@
 #include "store.h"
 
 /* Forward references */
-private int common_arc(P2(i_ctx_t *,
-	  int (*)(P6(gs_state *, floatp, floatp, floatp, floatp, floatp))));
-private int common_arct(P2(i_ctx_t *, float *));
+private int common_arc(i_ctx_t *,
+	  int (*)(gs_state *, floatp, floatp, floatp, floatp, floatp));
+private int common_arct(i_ctx_t *, float *);
 
 /* <x> <y> <r> <ang1> <ang2> arc - */
 int
@@ -51,7 +46,7 @@ zarcn(i_ctx_t *i_ctx_p)
 /* Common code for arc[n] */
 private int
 common_arc(i_ctx_t *i_ctx_p,
-      int (*aproc)(P6(gs_state *, floatp, floatp, floatp, floatp, floatp)))
+      int (*aproc)(gs_state *, floatp, floatp, floatp, floatp, floatp))
 {
     os_ptr op = osp;
     double xyra[5];		/* x, y, r, ang1, ang2 */
@@ -145,7 +140,7 @@ zclippath(i_ctx_t *i_ctx_p)
 
 /* <bool> .pathbbox <llx> <lly> <urx> <ury> */
 private int
-zpathbbox(i_ctx_t *i_ctx_p)
+z1pathbbox(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     gs_rect box;
@@ -163,9 +158,29 @@ zpathbbox(i_ctx_t *i_ctx_p)
     return 0;
 }
 
+/*
+ * In order to match Adobe output on a Genoa test, pathbbox must be an
+ * operator, not an operator procedure, even though it has a trivial
+ * definition as a procedure.
+ */
+private int
+zpathbbox(i_ctx_t *i_ctx_p)
+{
+    os_ptr op = osp;
+    int code;
+
+    push(1);
+    make_false(op);
+    code = z1pathbbox(i_ctx_p);
+    if (code < 0) {
+	pop(1);			/* remove the Boolean */
+    }
+    return code;
+}
+
 /* <moveproc> <lineproc> <curveproc> <closeproc> pathforall - */
-private int path_continue(P1(i_ctx_t *));
-private int path_cleanup(P1(i_ctx_t *));
+private int path_continue(i_ctx_t *);
+private int path_cleanup(i_ctx_t *);
 private int
 zpathforall(i_ctx_t *i_ctx_p)
 {
@@ -196,7 +211,7 @@ zpathforall(i_ctx_t *i_ctx_p)
     return o_push_estack;
 }
 /* Continuation procedure for pathforall */
-private void pf_push(P3(i_ctx_t *, gs_point *, int));
+private void pf_push(i_ctx_t *, gs_point *, int);
 private int
 path_continue(i_ctx_t *i_ctx_p)
 {
@@ -274,7 +289,8 @@ const op_def zpath1_op_defs[] =
     {"4pathforall", zpathforall},
     {"0reversepath", zreversepath},
     {"0strokepath", zstrokepath},
-    {"1.pathbbox", zpathbbox},
+    {"1.pathbbox", z1pathbbox},
+    {"0pathbbox", zpathbbox},
 		/* Internal operators */
     {"0%path_continue", path_continue},
     op_def_end(0)
