@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zshade.c 6729 2006-05-08 17:43:01Z ray $ */
+/* $Id: zshade.c 8022 2007-06-05 22:23:38Z giles $ */
 /* PostScript language interface to shading */
 #include "memory_.h"
 #include "ghost.h"
@@ -24,6 +24,7 @@
 #include "gsstruct.h"		/* must precede gsshade.h */
 #include "gsshade.h"
 #include "gsuid.h"
+#include "gscie.h"
 #include "stream.h"		/* for files.h */
 #include "files.h"
 #include "ialloc.h"
@@ -158,21 +159,19 @@ build_shading(i_ctx_t *i_ctx_p, build_shading_proc_t proc)
 
     check_type(*op, t_dictionary);
     params.ColorSpace = 0;
+    params.cie_joint_caches = 0;
     params.Background = 0;
     /* Collect parameters common to all shading types. */
     {
-	const gs_color_space *pcs_orig = gs_currentcolorspace(igs);
-	int num_comp = gs_color_space_num_components(pcs_orig);
-	gs_color_space *pcs;
+	gs_color_space *pcs = gs_currentcolorspace(igs);
+	int num_comp = gs_color_space_num_components(pcs);
 
-	if (num_comp < 0)	/* Pattern color space */
-	    return_error(e_rangecheck);
-	pcs = ialloc_struct(gs_color_space, &st_color_space,
-			    "build_shading");
-	if (pcs == 0)
-	    return_error(e_VMerror);
-	gs_cspace_init_from(pcs, pcs_orig);
+	if (num_comp < 0) {	/* Pattern color space */
+            gs_errorinfo_put_pair_from_dict(i_ctx_p, op, "ColorSpace");
+	    return_error(e_typecheck);
+        }
 	params.ColorSpace = pcs;
+	rc_increment(pcs);
 	if (dict_find_string(op, "Background", &pvalue) > 0) {
 	    gs_client_color *pcc =
 		ialloc_struct(gs_client_color, &st_client_color,
@@ -187,35 +186,56 @@ build_shading(i_ctx_t *i_ctx_p, build_shading_proc_t proc)
 	    code = dict_floats_param(imemory, op, "Background",
 				     gs_color_space_num_components(pcs),
 				     pcc->paint.values, NULL);
-	    if (code < 0)
+	    if (code < 0) {
+                gs_errorinfo_put_pair_from_dict(i_ctx_p, op, "Background");
 		goto fail;
+            }
 	}
     }
     if (dict_find_string(op, "BBox", &pvalue) <= 0)
 	params.have_BBox = false;
     else if ((code = dict_floats_param(imemory, op, "BBox", 
 				       4, box, NULL)) == 4) {
-	params.BBox.p.x = box[0];
-	params.BBox.p.y = box[1];
-	params.BBox.q.x = box[2];
-	params.BBox.q.y = box[3];
+	/* Adobe Interpreters accept denormalised BBox - bug 688937 */
+        if (box[0] <= box[2]) {
+            params.BBox.p.x = box[0];
+ 	    params.BBox.q.x = box[2];
+        } else {
+            params.BBox.p.x = box[2];
+ 	    params.BBox.q.x = box[0];
+        }
+	if (box[1] <= box[3]) {
+            params.BBox.p.y = box[1];
+	    params.BBox.q.y = box[3];
+        } else {
+            params.BBox.p.y = box[3];
+	    params.BBox.q.y = box[1];
+        }
 	params.have_BBox = true;
-    } else
+    } else {
+        gs_errorinfo_put_pair_from_dict(i_ctx_p, op, "BBox");
 	goto fail;
+    }
     code = dict_bool_param(op, "AntiAlias", false, &params.AntiAlias);
-    if (code < 0)
+    if (code < 0) {
+        gs_errorinfo_put_pair_from_dict(i_ctx_p, op, "AntiAlias");
 	goto fail;
+    }
     /* Finish building the shading. */
     code = (*proc)(i_ctx_p, op, &params, &psh, imemory);
     if (code < 0)
 	goto fail;
+    if (gx_color_space_needs_cie_caches(psh->params.ColorSpace)) {
+	rc_decrement(psh->params.cie_joint_caches, "build_shading");
+	psh->params.cie_joint_caches = gx_currentciecaches(igs);
+	rc_increment(psh->params.cie_joint_caches);
+    }
     make_istruct_new(op, 0, psh);
     return code;
 fail:
     gs_free_object(imemory, params.Background, "Background");
     if (params.ColorSpace) {
-	gs_cspace_release(params.ColorSpace);
-	gs_free_object(imemory, params.ColorSpace, "ColorSpace");
+	rc_decrement_only(params.ColorSpace, "build_shading");
     }
     return (code < 0 ? code : gs_note_error(e_rangecheck));
 }
@@ -223,7 +243,7 @@ fail:
 /* Collect a Function value. */
 private int
 build_shading_function(i_ctx_t *i_ctx_p, const ref * op, gs_function_t ** ppfn,
-		       int num_inputs, gs_memory_t *mem)
+	         int num_inputs, gs_memory_t *mem, const float *shading_domain)
 {
     ref *pFunction;
     int code;
@@ -247,7 +267,8 @@ build_shading_function(i_ctx_t *i_ctx_p, const ref * op, gs_function_t ** ppfn,
 	    ref rsubfn;
 
 	    array_get(imemory, pFunction, (long)i, &rsubfn);
-	    code = fn_build_function(i_ctx_p, &rsubfn, &Functions[i], mem);
+	    code = fn_build_function(i_ctx_p, &rsubfn, &Functions[i], mem,
+               shading_domain, num_inputs);
 	    if (code < 0)
 		break;
 	}
@@ -261,7 +282,8 @@ build_shading_function(i_ctx_t *i_ctx_p, const ref * op, gs_function_t ** ppfn,
 	if (code < 0)
 	    gs_function_AdOt_free_params(&params, mem);
     } else {
-	code = fn_build_function(i_ctx_p, pFunction, ppfn, mem);
+	code = fn_build_function(i_ctx_p, pFunction, ppfn, mem,
+            shading_domain, num_inputs);
 	if (code < 0)
 	    return code;
 	if ((*ppfn)->params.m != num_inputs) {
@@ -278,9 +300,15 @@ build_shading_function(i_ctx_t *i_ctx_p, const ref * op, gs_function_t ** ppfn,
  * Adobe interpreters follow PLRM in this respect and we follow them.
  */
 private int
-check_indexed_vs_function(const gs_color_space *pcs, const gs_function_t *foo)
-{ if (foo && gs_color_space_get_index(pcs) == gs_color_space_index_Indexed)
-    return_error(e_rangecheck);
+check_indexed_vs_function(i_ctx_t *i_ctx_p, const ref *op,
+                          const gs_color_space *pcs, const gs_function_t *funct)
+{ if (funct && gs_color_space_get_index(pcs) == gs_color_space_index_Indexed) {
+    static const char fn[] = "Function";
+    ref *f;
+    if (dict_find_string(op, fn, &f) > 0)
+        gs_errorinfo_put_pair(i_ctx_p, fn, sizeof(fn) - 1, f);
+    return_error(e_typecheck);  /* CET 12-14a */
+  }
   return 0;
 }
 
@@ -299,21 +327,38 @@ build_shading_1(i_ctx_t *i_ctx_p, const ref * op, const gs_shading_params_t * pc
     *(gs_shading_params_t *)&params = *pcommon;
     gs_make_identity(&params.Matrix);
     params.Function = 0;
-    if ((code = dict_floats_param(imemory, op, "Domain", 
-				  4, params.Domain,
-				  default_Domain)) < 0 ||
-	(dict_find_string(op, "Matrix", &pmatrix) > 0 &&
-	 (code = read_matrix(imemory, pmatrix, &params.Matrix)) < 0) ||
-	(code = build_shading_function(i_ctx_p, op, &params.Function, 2, mem)) < 0 ||
-	(code = check_indexed_vs_function(params.ColorSpace, params.Function)) < 0 ||
-	(code = gs_shading_Fb_init(ppsh, &params, mem)) < 0
-	) {
-	gs_free_object(mem, params.Function, "Function");
-	return code;
+    code = dict_floats_param_errorinfo(i_ctx_p, op, "Domain", 
+				  4, params.Domain, default_Domain);
+    if (code < 0)
+        goto out;
+    if (params.Domain[0] > params.Domain[1] || params.Domain[2] > params.Domain[3]) {
+        gs_errorinfo_put_pair_from_dict(i_ctx_p, op, "Domain"); 
+        code = gs_note_error(e_rangecheck);
+        goto out; /* CPSI 3017 and CET 12-14b reject un-normalised domain */
     }
-    if (params.Function == 0)		/* Function is required */
-	return_error(e_undefined);
-    return 0;
+    if (dict_find_string(op, "Matrix", &pmatrix) > 0 ) {
+        code = read_matrix(imemory, pmatrix, &params.Matrix);
+        if (code < 0) {
+            gs_errorinfo_put_pair_from_dict(i_ctx_p, op, "Matrix"); 
+            goto out;
+        }
+    }
+    code = build_shading_function(i_ctx_p, op, &params.Function, 2, mem, params.Domain);
+    if (code < 0)
+        goto out;
+    if (params.Function == 0) {  /* Function is required */
+	code = gs_note_error(e_undefined);
+	gs_errorinfo_put_pair_from_dict(i_ctx_p, op, "Function");
+        goto out;
+    }
+    code = check_indexed_vs_function(i_ctx_p, op, params.ColorSpace, params.Function);
+    if (code < 0)
+        goto out;
+    code = gs_shading_Fb_init(ppsh, &params, mem);
+ out:;
+    if (code < 0 && params.Function)
+	gs_free_object(mem, params.Function, "Function");
+    return code;
 }
 /* <dict> .buildshading1 <shading_struct> */
 private int
@@ -335,9 +380,9 @@ build_directional_shading(i_ctx_t *i_ctx_p, const ref * op, float *Coords, int n
 
     *pFunction = 0;
     if (code < 0 ||
-	(code = dict_floats_param(imemory, op, "Domain", 2, Domain,
+	(code = dict_floats_param_errorinfo(i_ctx_p, op, "Domain", 2, Domain,
 				  default_Domain)) < 0 ||
-	(code = build_shading_function(i_ctx_p, op, pFunction, 1, mem)) < 0
+	(code = build_shading_function(i_ctx_p, op, pFunction, 1, mem, Domain)) < 0
 	)
 	return code;
     if (!*pFunction)
@@ -374,7 +419,7 @@ build_shading_2(i_ctx_t *i_ctx_p, const ref * op, const gs_shading_params_t * pc
     if ((code = build_directional_shading(i_ctx_p, op, params.Coords, 4,
 					  params.Domain, &params.Function,
 					  params.Extend, mem)) < 0 ||
-	(code = check_indexed_vs_function(params.ColorSpace, params.Function)) < 0 ||
+	(code = check_indexed_vs_function(i_ctx_p, op, params.ColorSpace, params.Function)) < 0 ||
 	(code = gs_shading_A_init(ppsh, &params, mem)) < 0
 	) {
 	gs_free_object(mem, params.Function, "Function");
@@ -400,7 +445,7 @@ build_shading_3(i_ctx_t *i_ctx_p, const ref * op, const gs_shading_params_t * pc
     if ((code = build_directional_shading(i_ctx_p, op, params.Coords, 6,
 					  params.Domain, &params.Function,
 					  params.Extend, mem)) < 0 ||
-	(code = check_indexed_vs_function(params.ColorSpace, params.Function)) < 0 ||
+	(code = check_indexed_vs_function(i_ctx_p, op, params.ColorSpace, params.Function)) < 0 ||
 	(code = gs_shading_R_init(ppsh, &params, mem)) < 0
 	) {
 	gs_free_object(mem, params.Function, "Function");
@@ -462,7 +507,7 @@ build_mesh_shading(i_ctx_t *i_ctx_p, const ref * op,
 	    default:
 		return_error(e_typecheck);
 	}
-    code = build_shading_function(i_ctx_p, op, pFunction, 1, mem);
+    code = build_shading_function(i_ctx_p, op, pFunction, 1, mem, NULL);
     if (code < 0) {
 	gs_free_object(mem, data, "build_mesh_shading");
 	return code;
@@ -529,7 +574,7 @@ build_shading_4(i_ctx_t *i_ctx_p, const ref * op, const gs_shading_params_t * pc
     if ((code =
 	 build_mesh_shading(i_ctx_p, op, (gs_shading_mesh_params_t *)&params,
 			    &params.Decode, &params.Function, mem)) < 0 ||
-	(code = check_indexed_vs_function(params.ColorSpace, params.Function)) < 0 ||
+	(code = check_indexed_vs_function(i_ctx_p, op, params.ColorSpace, params.Function)) < 0 ||
 	(code = flag_bits_param(op, (gs_shading_mesh_params_t *)&params,
 				&params.BitsPerFlag)) < 0 ||
 	(code = gs_shading_FfGt_init(ppsh, &params, mem)) < 0
@@ -558,7 +603,7 @@ build_shading_5(i_ctx_t *i_ctx_p, const ref * op, const gs_shading_params_t * pc
     if ((code =
 	 build_mesh_shading(i_ctx_p, op, (gs_shading_mesh_params_t *)&params,
 			    &params.Decode, &params.Function, mem)) < 0 ||
-	(code = check_indexed_vs_function(params.ColorSpace, params.Function)) < 0 ||
+	(code = check_indexed_vs_function(i_ctx_p, op, params.ColorSpace, params.Function)) < 0 ||
 	(code = dict_int_param(op, "VerticesPerRow", 2, max_int, 0,
 			       &params.VerticesPerRow)) < 0 ||
 	(code = gs_shading_LfGt_init(ppsh, &params, mem)) < 0
@@ -587,7 +632,7 @@ build_shading_6(i_ctx_t *i_ctx_p, const ref * op, const gs_shading_params_t * pc
     if ((code =
 	 build_mesh_shading(i_ctx_p, op, (gs_shading_mesh_params_t *)&params,
 			    &params.Decode, &params.Function, mem)) < 0 ||
-	(code = check_indexed_vs_function(params.ColorSpace, params.Function)) < 0 ||
+	(code = check_indexed_vs_function(i_ctx_p, op, params.ColorSpace, params.Function)) < 0 ||
 	(code = flag_bits_param(op, (gs_shading_mesh_params_t *)&params,
 				&params.BitsPerFlag)) < 0 ||
 	(code = gs_shading_Cp_init(ppsh, &params, mem)) < 0
@@ -616,7 +661,7 @@ build_shading_7(i_ctx_t *i_ctx_p, const ref * op, const gs_shading_params_t * pc
     if ((code =
 	 build_mesh_shading(i_ctx_p, op, (gs_shading_mesh_params_t *)&params,
 			    &params.Decode, &params.Function, mem)) < 0 ||
-	(code = check_indexed_vs_function(params.ColorSpace, params.Function)) < 0 ||
+	(code = check_indexed_vs_function(i_ctx_p, op, params.ColorSpace, params.Function)) < 0 ||
 	(code = flag_bits_param(op, (gs_shading_mesh_params_t *)&params,
 				&params.BitsPerFlag)) < 0 ||
 	(code = gs_shading_Tpp_init(ppsh, &params, mem)) < 0
