@@ -1,22 +1,17 @@
-/* Copyright (C) 1989, 1995, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: zstring.c,v 1.2.6.1.2.1 2003/01/17 00:49:06 giles Exp $ */
+/* $Id: zstring.c 8022 2007-06-05 22:23:38Z giles $ */
 /* String operators */
 #include "memory_.h"
 #include "ghost.h"
@@ -56,7 +51,11 @@ zstring(i_ctx_t *i_ctx_p)
     byte *sbody;
     uint size;
 
-    check_int_leu(*op, max_string_size);
+    check_type(*op, t_integer);
+    if (op->value.intval < 0 ) 
+        return_error(e_rangecheck);
+    if (op->value.intval > max_string_size ) 
+        return_error(e_limitcheck); /* to match Distiller */
     size = op->value.intval;
     sbody = ialloc_string(size, "string");
     if (sbody == 0)
@@ -73,7 +72,7 @@ znamestring(i_ctx_t *i_ctx_p)
     os_ptr op = osp;
 
     check_type(*op, t_name);
-    name_string_ref(op, op);
+    name_string_ref(imemory, op, op);
     return 0;
 }
 
@@ -86,8 +85,8 @@ zanchorsearch(i_ctx_t *i_ctx_p)
     os_ptr op1 = op - 1;
     uint size = r_size(op);
 
-    check_read_type(*op1, t_string);
     check_read_type(*op, t_string);
+    check_read_type(*op1, t_string);
     if (size <= r_size(op1) && !memcmp(op1->value.bytes, op->value.bytes, size)) {
 	os_ptr op0 = op;
 
@@ -149,6 +148,28 @@ found:
     return 0;
 }
 
+/* <string> <charstring> .stringbreak <int|null> */
+private int
+zstringbreak(i_ctx_t *i_ctx_p)
+{
+    os_ptr op = osp;
+    uint i, j;
+
+    check_read_type(op[-1], t_string);
+    check_read_type(*op, t_string);
+    /* We can't use strpbrk here, because C doesn't allow nulls in strings. */
+    for (i = 0; i < r_size(op - 1); ++i)
+	for (j = 0; j < r_size(op); ++j)
+	    if (op[-1].value.const_bytes[i] == op->value.const_bytes[j]) {
+		make_int(op - 1, i);
+		goto done;
+	    }
+    make_null(op - 1);
+ done:
+    pop(1);
+    return 0;
+}
+
 /* <obj> <pattern> .stringmatch <bool> */
 private int
 zstringmatch(i_ctx_t *i_ctx_p)
@@ -163,7 +184,7 @@ zstringmatch(i_ctx_t *i_ctx_p)
 	    check_read(*op1);
 	    goto cmp;
 	case t_name:
-	    name_string_ref(op1, op1);	/* can't fail */
+	    name_string_ref(imemory, op1, op1);	/* can't fail */
 cmp:
 	    result = string_match(op1->value.const_bytes, r_size(op1),
 				  op->value.const_bytes, r_size(op),
@@ -186,6 +207,7 @@ const op_def zstring_op_defs[] =
     {"1.namestring", znamestring},
     {"2search", zsearch},
     {"1string", zstring},
+    {"2.stringbreak", zstringbreak},
     {"2.stringmatch", zstringmatch},
     op_def_end(0)
 };
